@@ -11,7 +11,7 @@ INDEX
 INDEX
 	_localeconv_r
 
-SYNOPSIS
+ANSI_SYNOPSIS
 	#include <locale.h>
 	char *setlocale(int <[category]>, const char *<[locale]>);
 	lconv *localeconv(void);
@@ -20,6 +20,22 @@ SYNOPSIS
                         int <[category]>, const char *<[locale]>);
 	lconv *_localeconv_r(void *<[reent]>);
 
+TRAD_SYNOPSIS
+	#include <locale.h>
+	char *setlocale(<[category]>, <[locale]>)
+	int <[category]>;
+	char *<[locale]>;
+
+	lconv *localeconv();
+
+	char *_setlocale_r(<[reent]>, <[category]>, <[locale]>)
+	char *<[reent]>;
+	int <[category]>;
+	char *<[locale]>;
+
+	lconv *_localeconv_r(<[reent]>);
+	char *<[reent]>;
+
 DESCRIPTION
 <<setlocale>> is the facility defined by ANSI C to condition the
 execution environment for international collating and formatting
@@ -74,19 +90,15 @@ Cygwin additionally supports locales from the file
 (<<"">> is also accepted; if given, the settings are read from the
 corresponding LC_* environment variables and $LANG according to POSIX rules.)
 
-This implementation also supports the modifiers <<"cjknarrow">> and
-<<"cjkwide">>, which affect how the functions <<wcwidth>> and <<wcswidth>>
-handle characters from the "CJK Ambiguous Width" category of characters
-described at http://www.unicode.org/reports/tr11/#Ambiguous.
-These characters have a width of 1 for singlebyte charsets and a width of 2
-for multibyte charsets other than UTF-8.
-For UTF-8, their width depends on the language specifier:
+This implementation also supports the modifier <<"cjknarrow">>, which
+affects how the functions <<wcwidth>> and <<wcswidth>> handle characters
+from the "CJK Ambiguous Width" category of characters described at
+http://www.unicode.org/reports/tr11/#Ambiguous. These characters have a width
+of 1 for singlebyte charsets and a width of 2 for multibyte charsets
+other than UTF-8. For UTF-8, their width depends on the language specifier:
 it is 2 for <<"zh">> (Chinese), <<"ja">> (Japanese), and <<"ko">> (Korean),
-and 1 for everything else. Specifying <<"cjknarrow">> or <<"cjkwide">>
-forces a width of 1 or 2, respectively, independent of charset and language.
-
-This implementation also supports the modifier <<"cjksingle">>
-to enforce single-width character properties.
+and 1 for everything else. Specifying <<"cjknarrow">> forces a width of 1,
+independent of charset and language.
 
 If you use <<NULL>> as the <[locale]> argument, <<setlocale>> returns a
 pointer to the string representing the current locale.  The acceptable
@@ -293,9 +305,10 @@ static char *currentlocale (void);
 #endif /* _MB_CAPABLE */
 
 char *
-_setlocale_r (struct _reent *p,
-       int category,
-       const char *locale)
+_DEFUN(_setlocale_r, (p, category, locale),
+       struct _reent *p _AND
+       int category _AND
+       _CONST char *locale)
 {
 #ifndef _MB_CAPABLE
   if (locale)
@@ -466,7 +479,7 @@ currentlocale ()
 extern void __set_ctype (struct __locale_t *, const char *charset);
 
 char *
-__loadlocale (struct __locale_t *loc, int category, char *new_locale)
+__loadlocale (struct __locale_t *loc, int category, const char *new_locale)
 {
   /* At this point a full-featured system would just load the locale
      specific data from the locale files.
@@ -483,15 +496,9 @@ __loadlocale (struct __locale_t *loc, int category, char *new_locale)
   int mbc_max;
   wctomb_p l_wctomb;
   mbtowc_p l_mbtowc;
-  int cjksingle = 0;
   int cjknarrow = 0;
-  int cjkwide = 0;
 
-  /* Avoid doing everything twice if nothing has changed.
-
-     duplocale relies on this test to go wrong so the locale is actually
-     duplicated when required.  Any change here has to be synced with a
-     matching change in duplocale. */
+  /* Avoid doing everything twice if nothing has changed. */
   if (!strcmp (new_locale, loc->categories[category]))
     return loc->categories[category];
 
@@ -510,7 +517,7 @@ __loadlocale (struct __locale_t *loc, int category, char *new_locale)
 
 restart:
   if (!locale)
-    locale = new_locale;
+    locale = (char *) new_locale;
   else if (locale != tmp_locale)
     {
       locale = __set_locale_from_locale_alias (locale, tmp_locale);
@@ -598,16 +605,12 @@ restart:
     }
   if (c && c[0] == '@')
     {
-      /* Modifier "cjksingle" is recognized to enforce single-width mode. */
-      /* Modifiers "cjknarrow" or "cjkwide" are recognized to modify the
-         behaviour of wcwidth() and wcswidth() for East Asian languages.
+      /* Modifier */
+      /* Only one modifier is recognized right now.  "cjknarrow" is used
+         to modify the behaviour of wcwidth() for East Asian languages.
          For details see the comment at the end of this function. */
-      if (!strcmp (c + 1, "cjksingle"))
-	cjksingle = 1;
-      else if (!strcmp (c + 1, "cjknarrow"))
+      if (!strcmp (c + 1, "cjknarrow"))
 	cjknarrow = 1;
-      else if (!strcmp (c + 1, "cjkwide"))
-	cjkwide = 1;
     }
   /* We only support this subset of charsets. */
   switch (charset[0])
@@ -899,23 +902,17 @@ restart:
       loc->wctomb = l_wctomb;
       loc->mbtowc = l_mbtowc;
       __set_ctype (loc, charset);
-      /* Set CJK width mode (1: ambiguous-wide, 0: normal, -1: disabled). */
       /* Determine the width for the "CJK Ambiguous Width" category of
          characters. This is used in wcwidth(). Assume single width for
          single-byte charsets, and double width for multi-byte charsets
          other than UTF-8. For UTF-8, use double width for the East Asian
          languages ("ja", "ko", "zh"), and single width for everything else.
-         Single width can also be forced with the "@cjknarrow" modifier.
-         Double width can also be forced with the "@cjkwide" modifier.
-       */
-      loc->cjk_lang = cjkwide ||
-		      (!cjknarrow && mbc_max > 1
-		       && (charset[0] != 'U'
-			   || strncmp (locale, "ja", 2) == 0
-			   || strncmp (locale, "ko", 2) == 0
-			   || strncmp (locale, "zh", 2) == 0));
-      if (cjksingle)
-	loc->cjk_lang = -1;	/* Disable CJK dual-width */
+         Single width can also be forced with the "@cjknarrow" modifier. */
+      loc->cjk_lang = !cjknarrow && mbc_max > 1
+		      && (charset[0] != 'U'
+			  || strncmp (locale, "ja", 2) == 0
+			  || strncmp (locale, "ko", 2) == 0
+			  || strncmp (locale, "zh", 2) == 0);
 #ifdef __HAVE_LOCALE_INFO__
       ret = __ctype_load_locale (loc, locale, (void *) l_wctomb, charset,
 				 mbc_max);
@@ -981,7 +978,7 @@ __get_locale_env (struct _reent *p, int category)
 #endif /* _MB_CAPABLE */
 
 int
-__locale_mb_cur_max (void)
+_DEFUN_VOID (__locale_mb_cur_max)
 {
 #ifdef __HAVE_LOCALE_INFO__
   return __get_current_ctype_locale ()->mb_cur_max[0];
@@ -990,7 +987,6 @@ __locale_mb_cur_max (void)
 #endif
 }
 
-#ifdef __HAVE_LOCALE_INFO__
 const char *
 __locale_ctype_ptr_l (struct __locale_t *locale)
 {
@@ -1000,15 +996,15 @@ __locale_ctype_ptr_l (struct __locale_t *locale)
 const char *
 __locale_ctype_ptr (void)
 {
-  return __get_current_locale ()->ctype_ptr;
+  return __get_current_locale (_REENT)->ctype_ptr;
 }
-#endif /* __HAVE_LOCALE_INFO__ */
 
 #ifndef _REENT_ONLY
 
 char *
-setlocale (int category,
-	const char *locale)
+_DEFUN (setlocale, (category, locale),
+	int category _AND
+	_CONST char *locale)
 {
   return _setlocale_r (_REENT, category, locale);
 }
