@@ -13,7 +13,11 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of the University nor the names of its contributors
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *	This product includes software developed by the University of
+ *	California, Berkeley and its contributors.
+ * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -59,7 +63,7 @@ INDEX
 INDEX
 	_vasnprintf_r
 
-SYNOPSIS
+ANSI_SYNOPSIS
 	#include <stdio.h>
 	#include <stdarg.h>
 	int vprintf(const char *<[fmt]>, va_list <[list]>);
@@ -194,9 +198,10 @@ static char *rcsid = "$Id$";
 #ifdef INTEGER_ONLY
 #ifndef _FVWRITE_IN_STREAMIO
 int
-__ssputs_r (struct _reent *ptr,
-       FILE *fp,
-       const char *buf,
+_DEFUN(__ssputs_r, (ptr, fp, buf, len),
+       struct _reent *ptr _AND
+       FILE *fp _AND
+       _CONST char *buf _AND
        size_t len)
 {
 	register int w;
@@ -222,7 +227,7 @@ __ssputs_r (struct _reent *ptr,
 			str = (unsigned char *)_malloc_r (ptr, newsize);
 			if (!str)
 			{
-				_REENT_ERRNO(ptr) = ENOMEM;
+				ptr->_errno = ENOMEM;
 				goto err;
 			}
 			memcpy (str, fp->_bf._base, curpos);
@@ -237,7 +242,7 @@ __ssputs_r (struct _reent *ptr,
 				_free_r (ptr, fp->_bf._base);
 				/* Ensure correct errno, even if free
 				 * changed it.  */
-				_REENT_ERRNO(ptr) = ENOMEM;
+				ptr->_errno = ENOMEM;
 				goto err;
 			}
 		}
@@ -249,7 +254,7 @@ __ssputs_r (struct _reent *ptr,
 	}
 	if (len < w)
 		w = len;
-	(void)memmove ((void *) fp->_p, (void *) buf, (size_t) (w));
+	(void)memmove ((_PTR) fp->_p, (_PTR) buf, (size_t) (w));
 	fp->_w -= w;
 	fp->_p += w;
 
@@ -262,14 +267,15 @@ err:
 #endif
 
 int
-__ssprint_r (struct _reent *ptr,
-       FILE *fp,
+_DEFUN(__ssprint_r, (ptr, fp, uio),
+       struct _reent *ptr _AND
+       FILE *fp _AND
        register struct __suio *uio)
 {
 	register size_t len;
 	register int w;
 	register struct __siov *iov;
-	register const char *p = NULL;
+	register _CONST char *p = NULL;
 
 	iov = uio->uio_iov;
 	len = 0;
@@ -306,7 +312,7 @@ __ssprint_r (struct _reent *ptr,
 				str = (unsigned char *)_malloc_r (ptr, newsize);
 				if (!str)
 				{
-					_REENT_ERRNO(ptr) = ENOMEM;
+					ptr->_errno = ENOMEM;
 					goto err;
 				}
 				memcpy (str, fp->_bf._base, curpos);
@@ -321,7 +327,7 @@ __ssprint_r (struct _reent *ptr,
 					_free_r (ptr, fp->_bf._base);
 					/* Ensure correct errno, even if free
 					 * changed it.  */
-					_REENT_ERRNO(ptr) = ENOMEM;
+					ptr->_errno = ENOMEM;
 					goto err;
 				}
 			}
@@ -333,7 +339,7 @@ __ssprint_r (struct _reent *ptr,
 		}
 		if (len < w)
 			w = len;
-		(void)memmove ((void *) fp->_p, (void *) p, (size_t) (w));
+		(void)memmove ((_PTR) fp->_p, (_PTR) p, (size_t) (w));
 		fp->_w -= w;
 		fp->_p += w;
 		w = len;          /* pretend we copied all */
@@ -353,7 +359,7 @@ err:
 }
 #else /* !INTEGER_ONLY */
 #ifndef _FVWRITE_IN_STREAMIO
-int __ssputs_r (struct _reent *, FILE *, const char *, size_t);
+int __ssputs_r (struct _reent *, FILE *, _CONST char *, size_t);
 #endif
 int __ssprint_r (struct _reent *, FILE *, register struct __suio *);
 #endif /* !INTEGER_ONLY */
@@ -363,14 +369,15 @@ int __ssprint_r (struct _reent *, FILE *, register struct __suio *);
 
 #ifndef _FVWRITE_IN_STREAMIO
 int
-__sfputs_r (struct _reent *ptr,
-       FILE *fp,
-       const char *buf,
+_DEFUN(__sfputs_r, (ptr, fp, buf, len),
+       struct _reent *ptr _AND
+       FILE *fp _AND
+       _CONST char *buf _AND
        size_t len)
 {
 	register int i;
 
-#if defined _WIDE_ORIENT && (!defined _ELIX_LEVEL || _ELIX_LEVEL >= 4)
+#ifdef _WIDE_ORIENT
 	if (fp->_flags2 & __SWID) {
 		wchar_t *p;
 
@@ -396,8 +403,9 @@ __sfputs_r (struct _reent *ptr,
  * then reset it so that it can be reused.
  */
 int
-__sprint_r (struct _reent *ptr,
-       FILE *fp,
+_DEFUN(__sprint_r, (ptr, fp, uio),
+       struct _reent *ptr _AND
+       FILE *fp _AND
        register struct __suio *uio)
 {
 	register int err = 0;
@@ -406,7 +414,7 @@ __sprint_r (struct _reent *ptr,
 		uio->uio_iovcnt = 0;
 		return (0);
 	}
-#if defined _WIDE_ORIENT && (!defined _ELIX_LEVEL || _ELIX_LEVEL >= 4)
+#ifdef _WIDE_ORIENT
 	if (fp->_flags2 & __SWID) {
 		struct __siov *iov;
 		wchar_t *p;
@@ -434,7 +442,7 @@ out:
 }
 #else /* !INTEGER_ONLY */
 #ifndef _FVWRITE_IN_STREAMIO
-int __sfputs_r (struct _reent *, FILE *, const char *buf, size_t);
+int __sfputs_r (struct _reent *, FILE *, _CONST char *buf, size_t);
 #endif
 int __sprint_r (struct _reent *, FILE *, register struct __suio *);
 #endif /* !INTEGER_ONLY */
@@ -448,9 +456,10 @@ int __sprint_r (struct _reent *, FILE *, register struct __suio *);
  * Make sure to avoid inlining.
  */
 _NOINLINE_STATIC int
-__sbprintf (struct _reent *rptr,
-       register FILE *fp,
-       const char *fmt,
+_DEFUN(__sbprintf, (rptr, fp, fmt, ap),
+       struct _reent *rptr _AND
+       register FILE *fp   _AND
+       _CONST char *fmt  _AND
        va_list ap)
 {
 	int ret;
@@ -501,8 +510,8 @@ __sbprintf (struct _reent *rptr,
 
 # ifdef _NO_LONGDBL
 
-extern char *_dtoa_r (struct _reent *, double, int,
-			      int, int *, int *, char **);
+extern char *_dtoa_r _PARAMS((struct _reent *, double, int,
+			      int, int *, int *, char **));
 
 #  define _PRINTF_FLOAT_TYPE double
 #  define _DTOA_R _dtoa_r
@@ -510,14 +519,17 @@ extern char *_dtoa_r (struct _reent *, double, int,
 
 # else /* !_NO_LONGDBL */
 
-extern char *_ldtoa_r (struct _reent *, _LONG_DOUBLE, int,
-			      int, int *, int *, char **);
+extern char *_ldtoa_r _PARAMS((struct _reent *, _LONG_DOUBLE, int,
+			      int, int *, int *, char **));
 
-extern int _ldcheck (_LONG_DOUBLE *);
+extern int _EXFUN(_ldcheck,(_LONG_DOUBLE *));
 
 #  define _PRINTF_FLOAT_TYPE _LONG_DOUBLE
 #  define _DTOA_R _ldtoa_r
-#  define FREXP frexpl
+/* FIXME - frexpl is not yet supported; and cvt infloops if (double)f
+   converts a finite value into infinity.  */
+/* #  define FREXP frexpl */
+#  define FREXP(f,e) ((_LONG_DOUBLE) frexp ((double)f, e))
 # endif /* !_NO_LONGDBL */
 
 static char *cvt(struct _reent *, _PRINTF_FLOAT_TYPE, int, int, char *, int *,
@@ -559,7 +571,7 @@ static int exponent(char *, int, int);
 #endif
 
 typedef quad_t * quad_ptr_t;
-typedef void *void_ptr_t;
+typedef _PTR     void_ptr_t;
 typedef char *   char_ptr_t;
 typedef long *   long_ptr_t;
 typedef int  *   int_ptr_t;
@@ -593,9 +605,9 @@ union arg_val
 };
 
 static union arg_val *
-get_arg (struct _reent *data, int n, char *fmt,
+_EXFUN(get_arg, (struct _reent *data, int n, char *fmt,
                  va_list *ap, int *numargs, union arg_val *args,
-                 int *arg_type, char **last_fmt);
+                 int *arg_type, char **last_fmt));
 #endif /* !_NO_POS_ARGS */
 
 /*
@@ -632,12 +644,13 @@ get_arg (struct _reent *data, int n, char *fmt,
 # define GROUPING	0x400		/* use grouping ("'" flag) */
 #endif
 
-int _VFPRINTF_R (struct _reent *, FILE *, const char *, va_list);
+int _EXFUN(_VFPRINTF_R, (struct _reent *, FILE *, _CONST char *, va_list));
 
 #ifndef STRING_ONLY
 int
-VFPRINTF (FILE * fp,
-       const char *fmt0,
+_DEFUN(VFPRINTF, (fp, fmt0, ap),
+       FILE * fp         _AND
+       _CONST char *fmt0 _AND
        va_list ap)
 {
   int result;
@@ -647,9 +660,10 @@ VFPRINTF (FILE * fp,
 #endif /* STRING_ONLY */
 
 int
-_VFPRINTF_R (struct _reent *data,
-       FILE * fp,
-       const char *fmt0,
+_DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
+       struct _reent *data _AND
+       FILE * fp           _AND
+       _CONST char *fmt0   _AND
        va_list ap)
 {
 	register char *fmt;	/* format string */
@@ -722,9 +736,9 @@ _VFPRINTF_R (struct _reent *data,
 	 * below longer.
 	 */
 #define	PADSIZE	16		/* pad chunk size */
-	static const char blanks[PADSIZE] =
+	static _CONST char blanks[PADSIZE] =
 	 {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '};
-	static const char zeroes[PADSIZE] =
+	static _CONST char zeroes[PADSIZE] =
 	 {'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
 
 #ifdef _MB_CAPABLE
@@ -868,7 +882,7 @@ _VFPRINTF_R (struct _reent *data,
 		fp->_bf._base = fp->_p = _malloc_r (data, 64);
 		if (!fp->_p)
 		{
-			_REENT_ERRNO(data) = ENOMEM;
+			data->_errno = ENOMEM;
 			return EOF;
 		}
 		fp->_bf._size = 64;
@@ -1161,7 +1175,7 @@ reswitch:	switch (ch) {
 			if (ch == 'C' || (flags & LONGINT)) {
 				mbstate_t ps;
 
-				memset ((void *)&ps, '\0', sizeof (mbstate_t));
+				memset ((_PTR)&ps, '\0', sizeof (mbstate_t));
 				if ((size = (int)_wcrtomb_r (data, cp,
 					       (wchar_t)GET_ARG (N, ap, wint_t),
 						&ps)) == -1) {
@@ -1231,8 +1245,6 @@ reswitch:	switch (ch) {
 				break;
 			}
 			if (isnan (_fpvalue)) {
-				if (signbit (_fpvalue))
-					sign = '-';
 				if (ch <= 'G') /* 'A', 'E', 'F', or 'G' */
 					cp = "NAN";
 				else
@@ -1264,8 +1276,6 @@ reswitch:	switch (ch) {
 				break;
 			}
 			if (expt == 1) {
-				if (signbit (_fpvalue))
-					sign = '-';
 				if (ch <= 'G') /* 'A', 'E', 'F', or 'G' */
 					cp = "NAN";
 				else
@@ -1374,7 +1384,7 @@ reswitch:	switch (ch) {
 		case 'm':  /* extension */
 			{
 				int dummy;
-				cp = _strerror_r (data, _REENT_ERRNO(data), 1, &dummy);
+				cp = _strerror_r (data, data->_errno, 1, &dummy);
 			}
 			flags &= ~LONGINT;
 			goto string;
@@ -1445,11 +1455,11 @@ string:
 #ifdef _MB_CAPABLE
 			if (ch == 'S' || (flags & LONGINT)) {
 				mbstate_t ps;
-				const wchar_t *wcp;
+				_CONST wchar_t *wcp;
 
-				wcp = (const wchar_t *)cp;
+				wcp = (_CONST wchar_t *)cp;
 				size = m = 0;
-				memset ((void *)&ps, '\0', sizeof (mbstate_t));
+				memset ((_PTR)&ps, '\0', sizeof (mbstate_t));
 
 				/* Count number of bytes needed for multibyte
 				   string that will be produced from widechar
@@ -1477,7 +1487,7 @@ string:
 						fp->_flags |= __SERR;
 						goto error;
 					}
-					wcp = (const wchar_t *)cp;
+					wcp = (_CONST wchar_t *)cp;
 				}
 
 				if (size == 0)
@@ -1495,7 +1505,7 @@ string:
 					cp = buf;
 
 				/* Convert widechar string to multibyte string. */
-				memset ((void *)&ps, '\0', sizeof (mbstate_t));
+				memset ((_PTR)&ps, '\0', sizeof (mbstate_t));
 				if (_wcsrtombs_r (data, cp, &wcp, size, &ps)
 				    != size) {
 					fp->_flags |= __SERR;
@@ -1960,7 +1970,7 @@ exponent(char *p0, int exp, int fmtch)
    the STRING_ONLY/INTEGER_ONLY versions here. */
 #if defined (STRING_ONLY) && defined(INTEGER_ONLY)
 
-const __CH_CLASS __chclass[256] = {
+_CONST __CH_CLASS __chclass[256] = {
   /* 00-07 */  OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,
   /* 08-0f */  OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,
   /* 10-17 */  OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,
@@ -1995,7 +2005,7 @@ const __CH_CLASS __chclass[256] = {
   /* f8-ff */  OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,   OTHER,
 };
 
-const __STATE __state_table[MAX_STATE][MAX_CH_CLASS] = {
+_CONST __STATE __state_table[MAX_STATE][MAX_CH_CLASS] = {
   /*             '0'     '1-9'     '$'     MODFR    SPEC    '.'     '*'    FLAG    OTHER */
   /* START */  { SFLAG,   WDIG,    DONE,   SMOD,    DONE,   SDOT,  VARW,   SFLAG,  DONE },
   /* SFLAG */  { SFLAG,   WDIG,    DONE,   SMOD,    DONE,   SDOT,  VARW,   SFLAG,  DONE },
@@ -2010,7 +2020,7 @@ const __STATE __state_table[MAX_STATE][MAX_CH_CLASS] = {
   /* VPDIG */  { DONE,    DONE,    PREC,   DONE,    DONE,   DONE,  DONE,   DONE,   DONE },
 };
 
-const __ACTION __action_table[MAX_STATE][MAX_CH_CLASS] = {
+_CONST __ACTION __action_table[MAX_STATE][MAX_CH_CLASS] = {
   /*             '0'     '1-9'     '$'     MODFR    SPEC    '.'     '*'    FLAG    OTHER */
   /* START */  { NOOP,    NUMBER,  NOOP,   GETMOD,  GETARG, NOOP,  NOOP,   NOOP,   NOOP },
   /* SFLAG */  { NOOP,    NUMBER,  NOOP,   GETMOD,  GETARG, NOOP,  NOOP,   NOOP,   NOOP },
@@ -2029,13 +2039,14 @@ const __ACTION __action_table[MAX_STATE][MAX_CH_CLASS] = {
 
 /* function to get positional parameter N where n = N - 1 */
 static union arg_val *
-get_arg (struct _reent *data,
-       int n,
-       char *fmt,
-       va_list *ap,
-       int *numargs_p,
-       union arg_val *args,
-       int *arg_type,
+_DEFUN(get_arg, (data, n, fmt, ap, numargs_p, args, arg_type, last_fmt),
+       struct _reent *data _AND
+       int n               _AND
+       char *fmt           _AND
+       va_list *ap         _AND
+       int *numargs_p      _AND
+       union arg_val *args _AND
+       int *arg_type       _AND
        char **last_fmt)
 {
   int ch;
@@ -2083,8 +2094,6 @@ get_arg (struct _reent *data,
 
       if (*fmt == '\0')
 	break;
-
-      fmt++;
 # endif /* ! _MB_CAPABLE */
       state = START;
       flags = 0;
