@@ -5,7 +5,7 @@
  * Redistribution and use in source and binary forms are permitted
  * provided that the above copyright notice and this paragraph are
  * duplicated in all such forms and that any documentation,
- * and/or other materials related to such
+ * advertising materials, and other materials related to such
  * distribution and use acknowledge that the software was developed
  * by the University of California, Berkeley.  The name of the
  * University may not be used to endorse or promote products derived
@@ -32,7 +32,7 @@ INDEX
 INDEX
 	_vswscanf
 
-SYNOPSIS
+ANSI_SYNOPSIS
 	#include <stdio.h>
 	#include <stdarg.h>
 	int vwscanf(const wchar_t *__restrict <[fmt]>, va_list <[list]>);
@@ -48,6 +48,40 @@ SYNOPSIS
 	int _vswscanf(struct _reent *<[reent]>, const wchar_t *<[str]>,
                        const wchar_t *<[fmt]>, va_list <[list]>);
 
+TRAD_SYNOPSIS
+	#include <stdio.h>
+	#include <varargs.h>
+	int vwscanf( <[fmt]>, <[ist]>)
+	wchar_t *__restrict <[fmt]>;
+	va_list <[list]>;
+
+	int vfwscanf( <[fp]>, <[fmt]>, <[list]>)
+	FILE *__restrict <[fp]>;
+	wchar_t *__restrict <[fmt]>;
+	va_list <[list]>;
+
+	int vswscanf( <[str]>, <[fmt]>, <[list]>)
+	wchar_t *__restrict <[str]>;
+	wchar_t *__restrict <[fmt]>;
+	va_list <[list]>;
+
+	int _vwscanf( <[reent]>, <[fmt]>, <[ist]>)
+	struct _reent *<[reent]>;
+	wchar_t *<[fmt]>;
+	va_list <[list]>;
+
+	int _vfwscanf( <[reent]>, <[fp]>, <[fmt]>, <[list]>)
+	struct _reent *<[reent]>;
+	FILE *<[fp]>;
+	wchar_t *<[fmt]>;
+	va_list <[list]>;
+
+	int _vswscanf( <[reent]>, <[str]>, <[fmt]>, <[list]>)
+	struct _reent *<[reent]>;
+	wchar_t *<[str]>;
+	wchar_t *<[fmt]>;
+	va_list <[list]>;
+
 DESCRIPTION
 <<vwscanf>>, <<vfwscanf>>, and <<vswscanf>> are (respectively) variants
 of <<wscanf>>, <<fwscanf>>, and <<swscanf>>.  They differ only in
@@ -138,7 +172,7 @@ C99, POSIX-1.2008
 #define _NO_LONGDBL
 #if defined _WANT_IO_LONG_DOUBLE && (LDBL_MANT_DIG > DBL_MANT_DIG)
 #undef _NO_LONGDBL
-extern _LONG_DOUBLE _wcstold_r (wchar_t *s, wchar_t **sptr);
+extern _LONG_DOUBLE _wcstold_r _PARAMS((wchar_t *s, wchar_t **sptr));
 #endif
 
 #include "floatio.h"
@@ -185,7 +219,6 @@ static void * get_arg (int, va_list *, int *, void **);
 #define	SUPPRESS	0x10	/* suppress assignment */
 #define	POINTER		0x20	/* weird %p pointer (`fake hex') */
 #define	NOSKIP		0x40	/* do not skip blanks */
-#define	MALLOC		0x80	/* handle 'm' modifier */
 
 /*
  * The following are used in numeric conversions only:
@@ -226,8 +259,9 @@ static void * get_arg (int, va_list *, int *, void **);
 #ifndef _REENT_ONLY
 
 int
-VFWSCANF (register FILE *__restrict fp,
-       const wchar_t *__restrict fmt,
+_DEFUN(VFWSCANF, (fp, fmt, ap),
+       register FILE *__restrict fp _AND
+       _CONST wchar_t *__restrict fmt _AND
        va_list ap)
 {
   struct _reent *reent = _REENT;
@@ -237,8 +271,9 @@ VFWSCANF (register FILE *__restrict fp,
 }
 
 int
-__SVFWSCANF (register FILE *fp,
-       wchar_t const *fmt0,
+_DEFUN(__SVFWSCANF, (fp, fmt0, ap),
+       register FILE *fp _AND
+       wchar_t _CONST *fmt0 _AND
        va_list ap)
 {
   return __SVFWSCANF_R (_REENT, fp, fmt0, ap);
@@ -247,9 +282,10 @@ __SVFWSCANF (register FILE *fp,
 #endif /* !_REENT_ONLY */
 
 int
-_VFWSCANF_R (struct _reent *data,
-       register FILE *fp,
-       const wchar_t *fmt,
+_DEFUN(_VFWSCANF_R, (data, fp, fmt, ap),
+       struct _reent *data _AND
+       register FILE *fp   _AND
+       _CONST wchar_t *fmt    _AND
        va_list ap)
 {
   CHECK_INIT(data, fp);
@@ -262,8 +298,9 @@ _VFWSCANF_R (struct _reent *data,
  * regular ungetwc which will drag in file I/O items we don't need.
  * So, we create our own trimmed-down version.  */
 static wint_t
-_sungetwc_r (struct _reent *data,
-	wint_t wc,
+_DEFUN(_sungetwc_r, (data, fp, ch),
+	struct _reent *data _AND
+	wint_t wc           _AND
 	register FILE *fp)
 {
   if (wc == WEOF)
@@ -272,22 +309,57 @@ _sungetwc_r (struct _reent *data,
   /* After ungetc, we won't be at eof anymore */
   fp->_flags &= ~__SEOF;
 
-  /* All ungetwc usage in scanf un-gets the current character, so
-   * just back up over the string if we aren't at the start
+  /*
+   * If we are in the middle of ungetwc'ing, just continue.
+   * This may require expanding the current ungetc buffer.
    */
-  if (fp->_bf._base != NULL && fp->_p > fp->_bf._base)
+
+  if (HASUB (fp))
+    {
+      if (fp->_r >= fp->_ub._size && __submore (data, fp))
+        {
+          return EOF;
+        }
+      fp->_p -= sizeof (wchar_t);
+      *fp->_p = (wchar_t) wc;
+      fp->_r += sizeof (wchar_t);
+      return wc;
+    }
+
+  /*
+   * If we can handle this by simply backing up, do so,
+   * but never replace the original character.
+   * (This makes swscanf() work when scanning `const' data.)
+   */
+
+  if (fp->_bf._base != NULL && fp->_p > fp->_bf._base
+      && ((wchar_t *)fp->_p)[-1] == wc)
     {
       fp->_p -= sizeof (wchar_t);
       fp->_r += sizeof (wchar_t);
+      return wc;
     }
 
+  /*
+   * Create an ungetc buffer.
+   * Initially, we will use the `reserve' buffer.
+   */
+
+  fp->_ur = fp->_r;
+  fp->_up = fp->_p;
+  fp->_ub._base = fp->_ubuf;
+  fp->_ub._size = sizeof (fp->_ubuf);
+  fp->_p = &fp->_ubuf[sizeof (fp->_ubuf) - sizeof (wchar_t)];
+  *(wchar_t *) fp->_p = wc;
+  fp->_r = 2;
   return wc;
 }
 
-extern int __ssrefill_r (struct _reent *ptr, register FILE * fp);
+extern int __ssrefill_r _PARAMS ((struct _reent *ptr, register FILE * fp));
 
 static size_t
-_sfgetwc_r (struct _reent * ptr,
+_DEFUN(_sfgetwc_r, (ptr, fp),
+       struct _reent * ptr _AND
        FILE * fp)
 {
   wchar_t wc;
@@ -302,9 +374,10 @@ _sfgetwc_r (struct _reent * ptr,
 #endif /* STRING_ONLY */
 
 int
-__SVFWSCANF_R (struct _reent *rptr,
-       register FILE *fp,
-       wchar_t const *fmt0,
+_DEFUN(__SVFWSCANF_R, (rptr, fp, fmt0, ap),
+       struct _reent *rptr _AND
+       register FILE *fp   _AND
+       wchar_t _CONST *fmt0   _AND
        va_list ap)
 {
   register wchar_t *fmt = (wchar_t *) fmt0;
@@ -327,7 +400,7 @@ __SVFWSCANF_R (struct _reent *rptr,
 
   mbstate_t mbs;                /* value to keep track of multibyte state */
 
-  #define CCFN_PARAMS	(struct _reent *, const wchar_t *, wchar_t **, int)
+  #define CCFN_PARAMS	_PARAMS((struct _reent *, const wchar_t *, wchar_t **, int))
   unsigned long (*ccfn)CCFN_PARAMS=0;	/* conversion function (wcstol/wcstoul) */
   wchar_t buf[BUF];		/* buffer for numeric conversions */
   const wchar_t *ccls;          /* character class start */
@@ -336,7 +409,7 @@ __SVFWSCANF_R (struct _reent *rptr,
   wint_t wi;                    /* handy wint_t */
   char *mbp = NULL;             /* multibyte string pointer for %c %s %[ */
   size_t nconv;                 /* number of bytes in mb. conversion */
-  char mbbuf[MB_LEN_MAX];	/* temporary mb. character buffer */
+  char mbbuf[MB_LEN_MAX];       /* temporary mb. character buffer */
 
   char *cp;
   short *sp;
@@ -351,130 +424,9 @@ __SVFWSCANF_R (struct _reent *rptr,
 #ifndef _NO_LONGLONG
   long long *llp;
 #endif
-#ifdef _WANT_IO_C99_FORMATS
-#define _WANT_IO_POSIX_EXTENSIONS
-#endif
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-  /* POSIX requires that fwscanf frees all allocated strings from 'm'
-     conversions in case it returns EOF.  m_ptr is used to keep track.
-     It will be allocated on the stack the first time an 'm' conversion
-     takes place, and it will be free'd on return from the function.
-     This implementation tries to save space by only allocating 8
-     pointer slots at a time.  Most scenarios should never have to call
-     realloc again.  This implementation allows only up to 65528 'm'
-     conversions per fwscanf invocation for now.  That should be enough
-     for almost all scenarios, right? */
-  struct m_ptrs {
-    void ***m_arr;		/* Array of pointer args to 'm' conversion */
-    uint16_t m_siz;		/* Number of slots in m_arr */
-    uint16_t m_cnt;		/* Number of valid entries in m_arr */
-  } *m_ptr = NULL;
-  #define init_m_ptr()							\
-    do									\
-      {									\
-	if (!m_ptr)							\
-	  {								\
-	    m_ptr = (struct m_ptrs *) alloca (sizeof *m_ptr);		\
-	    m_ptr->m_arr = NULL;					\
-	    m_ptr->m_siz = 0;						\
-	    m_ptr->m_cnt = 0;						\
-	  }								\
-      }									\
-    while (0)
-  #define push_m_ptr(arg)						\
-    do									\
-      {									\
-	if (m_ptr->m_cnt >= m_ptr->m_siz)				\
-	  {								\
-	    void ***n = NULL;						\
-									\
-	    if (m_ptr->m_siz + 8 > 0 && m_ptr->m_siz + 8 < UINT16_MAX)	\
-	      n = (void ***) realloc (m_ptr->m_arr,			\
-				      (m_ptr->m_siz + 8) *		\
-				      sizeof (void **));		\
-	    if (!n)							\
-	      {								\
-		nassigned = EOF;					\
-		goto match_failure;					\
-	      }								\
-	    m_ptr->m_arr = n;						\
-	    m_ptr->m_siz += 8;						\
-	  }								\
-	m_ptr->m_arr[m_ptr->m_cnt++] = (void **) (arg);				\
-      }									\
-    while (0)
-  #define alloc_m_ptr(_type, _p, _p0, _p_p, _w)				\
-    ({									\
-      _p_p = GET_ARG (N, ap, _type **);					\
-      if (!_p_p)							\
-	goto match_failure;						\
-      _p0 = (_type *) malloc ((_w) * sizeof (_type));			\
-      if (!_p0)								\
-	{								\
-	  nassigned = EOF;						\
-	  goto match_failure;						\
-	}								\
-      *_p_p = _p0;							\
-      push_m_ptr (_p_p);						\
-      _p = _p0;								\
-      _w;								\
-    })
-  /* For char output, check if there's room for at least MB_CUR_MAX
-     characters. */
-  #define realloc_m_ptr(_type, _p, _p0, _p_p, _w)			\
-    ({									\
-      size_t _nw = (_w);						\
-      ptrdiff_t _dif = _p - _p0;					\
-      if (_p_p &&							\
-	  ((sizeof (_type) == 1 && _dif >= _nw - MB_CUR_MAX)		\
-	   || _dif >= _nw))						\
-	{								\
-	  _p0 = (_type *) realloc (_p0, (_nw << 1) * sizeof (_type));	\
-	  if (!_p0)							\
-	    {								\
-	      nassigned = EOF;						\
-	      goto match_failure;					\
-	    }								\
-	  _p = _p0 + _dif;						\
-	  *_p_p = _p0;							\
-	  _nw <<= 1;							\
-	}								\
-      _nw;								\
-    })
-  #define shrink_m_ptr(_type, _p_p, _w, _cw)				\
-    ({									\
-	size_t _nw = (_w);						\
-	if (_p_p && _nw < _cw)						\
-	  {								\
-	    _type *_np_p = (_type *)					\
-			   realloc (*_p_p, _nw * sizeof (_type));	\
-	    if (_np_p)							\
-	      *_p_p = _np_p;						\
-	  }								\
-    })
-  #define free_m_ptr()							\
-    do									\
-      {									\
-	if (m_ptr)							\
-	  {								\
-	    if (nassigned == EOF)					\
-	      {								\
-		int i;							\
-		for (i = 0; i < m_ptr->m_cnt; ++i)			\
-		  {							\
-		    free (*m_ptr->m_arr[i]);				\
-		    *m_ptr->m_arr[i] = NULL;				\
-		  }							\
-	      }								\
-	    if (m_ptr->m_arr)						\
-	      free (m_ptr->m_arr);					\
-	  }								\
-      }									\
-    while (0)
-#endif
 
   /* `basefix' is used to avoid `if' tests in the integer scanner */
-  static const short basefix[17] =
+  static _CONST short basefix[17] =
     {10, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
 
   /* Macro to support positional arguments */
@@ -566,14 +518,9 @@ __SVFWSCANF_R (struct _reent *rptr,
 	  continue;
 
 	case L'*':
-	  if ((flags & (CHAR | SHORT | LONG | LONGDBL | SUPPRESS | MALLOC))
-	      || width)
-	    goto match_failure;
 	  flags |= SUPPRESS;
 	  goto again;
 	case L'l':
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 #if defined _WANT_IO_C99_FORMATS || !defined _NO_LONGLONG
 	  if (*fmt == L'l')	/* Check for 'll' = long long (SUSv3) */
 	    {
@@ -585,14 +532,10 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    flags |= LONG;
 	  goto again;
 	case L'L':
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 	  flags |= LONGDBL;
 	  goto again;
 	case L'h':
 #ifdef _WANT_IO_C99_FORMATS
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 	  if (*fmt == 'h')	/* Check for 'hh' = char int (SUSv3) */
 	    {
 	      ++fmt;
@@ -604,16 +547,12 @@ __SVFWSCANF_R (struct _reent *rptr,
 	  goto again;
 #ifdef _WANT_IO_C99_FORMATS
 	case L'j': /* intmax_t */
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 	  if (sizeof (intmax_t) == sizeof (long))
 	    flags |= LONG;
 	  else
 	    flags |= LONGDBL;
 	  goto again;
 	case L't': /* ptrdiff_t */
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 	  if (sizeof (ptrdiff_t) < sizeof (int))
 	    /* POSIX states ptrdiff_t is 16 or more bits, as
 	       is short.  */
@@ -630,8 +569,6 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    flags |= LONGDBL;
 	  goto again;
 	case L'z': /* size_t */
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL))
-	    goto match_failure;
 	  if (sizeof (size_t) < sizeof (int))
 	    /* POSIX states size_t is 16 or more bits, as is short.  */
 	    flags |= SHORT;
@@ -647,14 +584,6 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    flags |= LONGDBL;
 	  goto again;
 #endif /* _WANT_IO_C99_FORMATS */
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	case 'm':
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL | MALLOC))
-	    goto match_failure;
-	  init_m_ptr ();
-	  flags |= MALLOC;
-	  goto again;
-#endif
 
 	case L'0':
 	case L'1':
@@ -666,15 +595,11 @@ __SVFWSCANF_R (struct _reent *rptr,
 	case L'7':
 	case L'8':
 	case L'9':
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL | MALLOC))
-	    goto match_failure;
 	  width = width * 10 + c - L'0';
 	  goto again;
 
 #ifndef _NO_POS_ARGS
 	case L'$':
-	  if (flags & (CHAR | SHORT | LONG | LONGDBL | SUPPRESS | MALLOC))
-	    goto match_failure;
 	  if (width <= MAX_POS_ARGS)
 	    {
 	      N = width - 1;
@@ -744,6 +669,7 @@ __SVFWSCANF_R (struct _reent *rptr,
 	  break;
 
 	case L'[':
+	  ccls = fmt;
 	  if (*fmt == '^')
 	    {
 	      cclcompl = 1;
@@ -751,7 +677,6 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    }
 	  else
 	    cclcompl = 0;
-	  ccls = fmt;
 	  if (*fmt == ']')
 	    fmt++;
 	  while (*fmt != '\0' && *fmt != ']')
@@ -815,7 +740,14 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    }
 	  continue;
 
-	default:
+	  /*
+	   * Disgusting backwards compatibility hacks.	XXX
+	   */
+	case L'\0':		/* compat */
+	  _newlib_flockfile_exit (fp);
+	  return EOF;
+
+	default:		/* compat */
 	  goto match_failure;
 	}
 
@@ -844,83 +776,56 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    width = 1;
           if (flags & LONG)
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      wchar_t **p_p = NULL;
-	      wchar_t *p0 = NULL;
-	      size_t p_siz = 0;
-#endif
-
-	      if (flags & SUPPRESS)
-		;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      else if (flags & MALLOC)
-		p_siz = alloc_m_ptr (wchar_t, p, p0, p_p, 32);
-#endif
-	      else
+	      if (!(flags & SUPPRESS))
 		p = GET_ARG(N, ap, wchar_t *);
 	      n = 0;
 	      while (width-- != 0 && (wi = _fgetwc_r (rptr, fp)) != WEOF)
 		{
 		  if (!(flags & SUPPRESS))
-		    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		      /* Check before ++ because we never add a \0 */
-		      p_siz = realloc_m_ptr (wchar_t, p, p0, p_p, p_siz);
-#endif
-		      *p++ = (wchar_t) wi;
-		    }
+		    *p++ = (wchar_t) wi;
 		  n++;
 		}
 	      if (n == 0)
 		goto input_failure;
 	      nread += n;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      shrink_m_ptr (wchar_t, p_p, p - p0, p_siz);
-#endif
 	      if (!(flags & SUPPRESS))
 		nassigned++;
 	    }
 	  else
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      char **mbp_p = NULL;
-	      char *mbp0 = NULL;
-	      size_t mbp_siz = 0;
-#endif
-
-	      if (flags & SUPPRESS)
-		mbp = mbbuf;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      else if (flags & MALLOC)
-		mbp_siz = alloc_m_ptr (char, mbp, mbp0, mbp_p, 32);
-#endif
-	      else
+	      if (!(flags & SUPPRESS))
 		mbp = GET_ARG(N, ap, char *);
 	      n = 0;
-	      memset ((void *)&mbs, '\0', sizeof (mbstate_t));
+	      memset ((_PTR)&mbs, '\0', sizeof (mbstate_t));
 	      while (width != 0 && (wi = _fgetwc_r (rptr, fp)) != WEOF)
 		{
-		  nconv = _wcrtomb_r (rptr, mbp, wi, &mbs);
-		  if (nconv == (size_t) -1)
-		    goto input_failure;
-		  /* Ignore high surrogate in width counting */
-		  if (nconv != 0 || mbs.__count != -4)
-		    width--;
-		  if (!(flags & SUPPRESS))
+		  if (width >= MB_CUR_MAX && !(flags & SUPPRESS))
 		    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		      mbp_siz = realloc_m_ptr (char, mbp, mbp0, mbp_p, mbp_siz);
-#endif
-		      mbp += nconv;
+		      nconv = _wcrtomb_r (rptr, mbp, wi, &mbs);
+		      if (nconv == (size_t) -1)
+			goto input_failure;
 		    }
+		  else
+		    {
+		      nconv = _wcrtomb_r (rptr, mbbuf, wi, &mbs);
+		      if (nconv == (size_t) -1)
+			goto input_failure;
+		      if (nconv > width)
+			{
+			  _ungetwc_r (rptr, wi, fp);
+			  break;
+			}
+		      if (!(flags & SUPPRESS))
+			memcpy(mbp, mbbuf, nconv);
+		    }
+		  if (!(flags & SUPPRESS))
+		    mbp += nconv;
+		  width -= nconv;
 		  n++;
 		}
 	      if (n == 0)
 		goto input_failure;
 	      nread += n;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      shrink_m_ptr (char, mbp_p, mbp - mbp0, mbp_siz);
-#endif
 	      if (!(flags & SUPPRESS))
 		nassigned++;
 	    }
@@ -929,7 +834,7 @@ __SVFWSCANF_R (struct _reent *rptr,
 	case CT_CCL:
 	  /* scan a (nonempty) character class (sets NOSKIP) */
 	  if (width == 0)
-	    width = SIZE_MAX;		/* `infinity' */
+	    width = (size_t) ~0;		/* `infinity' */
 	  /* take only those things in the class */
 	  if ((flags & SUPPRESS) && (flags & LONG))
 	    {
@@ -944,68 +849,46 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    }
 	  else if (flags & LONG)
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      wchar_t **p_p = NULL;
-	      size_t p_siz = 0;
-
-	      if (flags & MALLOC)
-		p_siz = alloc_m_ptr (wchar_t, p, p0, p_p, 32);
-	      else
-#endif
-		p0 = p = GET_ARG(N, ap, wchar_t *);
+	      p0 = p = GET_ARG(N, ap, wchar_t *);
 	      while ((wi = _fgetwc_r (rptr, fp)) != WEOF
 		     && width-- != 0 && INCCL (wi))
-		{
-		  *p++ = (wchar_t) wi;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		  p_siz = realloc_m_ptr (wchar_t, p, p0, p_p, p_siz);
-#endif
-		}
+		*p++ = (wchar_t) wi;
 	      if (wi != WEOF)
 		_ungetwc_r (rptr, wi, fp);
 	      n = p - p0;
 	      if (n == 0)
 		goto match_failure;
 	      *p = L'\0';
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      shrink_m_ptr (wchar_t, p_p, n + 1, p_siz);
-#endif
 	      nassigned++;
 	    }
 	  else
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      char **mbp_p = NULL;
-	      char *mbp0 = NULL;
-	      size_t mbp_siz = 0;
-#endif
-
-	      if (flags & SUPPRESS)
-		mbp = mbbuf;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      else if (flags & MALLOC)
-		mbp_siz = alloc_m_ptr (char, mbp, mbp0, mbp_p, 32);
-#endif
-	      else
+	      if (!(flags & SUPPRESS))
 		mbp = GET_ARG(N, ap, char *);
 	      n = 0;
-	      memset ((void *) &mbs, '\0', sizeof (mbstate_t));
+	      memset ((_PTR) &mbs, '\0', sizeof (mbstate_t));
 	      while ((wi = _fgetwc_r (rptr, fp)) != WEOF
-		     && width != 0 && INCCL (wi))
+		     && width-- != 0 && INCCL (wi))
 		{
-		  nconv = _wcrtomb_r (rptr, mbp, wi, &mbs);
-		  if (nconv == (size_t) -1)
-		    goto input_failure;
-		  /* Ignore high surrogate in width counting */
-		  if (nconv != 0 || mbs.__count != -4)
-		    width--;
-		  if (!(flags & SUPPRESS))
+		  if (width >= MB_CUR_MAX && !(flags & SUPPRESS))
 		    {
-		      mbp += nconv;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		      mbp_siz = realloc_m_ptr (char, mbp, mbp0, mbp_p, mbp_siz);
-#endif
+		      nconv = _wcrtomb_r (rptr, mbp, wi, &mbs);
+		      if (nconv == (size_t) -1)
+			goto input_failure;
 		    }
+		  else
+		    {
+		      nconv = wcrtomb(mbbuf, wi, &mbs);
+		      if (nconv == (size_t) -1)
+			goto input_failure;
+		      if (nconv > width)
+			break;
+		      if (!(flags & SUPPRESS))
+			memcpy(mbp, mbbuf, nconv);
+		    }
+		  if (!(flags & SUPPRESS))
+		    mbp += nconv;
+		  width -= nconv;
 		  n++;
 		}
 	      if (wi != WEOF)
@@ -1013,9 +896,6 @@ __SVFWSCANF_R (struct _reent *rptr,
 	      if (!(flags & SUPPRESS))
 		{
 		  *mbp = 0;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		  shrink_m_ptr (char, mbp_p, mbp - mbp0 + 1, mbp_siz);
-#endif
 		  nassigned++;
 		}
 	    }
@@ -1025,7 +905,7 @@ __SVFWSCANF_R (struct _reent *rptr,
 	case CT_STRING:
 	  /* like CCL, but zero-length string OK, & no NOSKIP */
 	  if (width == 0)
-            width = SIZE_MAX;
+            width = (size_t)~0;
 	  if ((flags & SUPPRESS) && (flags & LONG))
 	    {
 	      while ((wi = _fgetwc_r (rptr, fp)) != WEOF
@@ -1036,65 +916,45 @@ __SVFWSCANF_R (struct _reent *rptr,
 	    }
 	  else if (flags & LONG)
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-              wchar_t **p_p = NULL;
-              size_t p_siz = 0;
-
-              if (flags & MALLOC)
-                p_siz = alloc_m_ptr (wchar_t, p, p0, p_p, 32);
-              else
-#endif
-		p0 = p = GET_ARG(N, ap, wchar_t *);
+	      p0 = p = GET_ARG(N, ap, wchar_t *);
 	      while ((wi = _fgetwc_r (rptr, fp)) != WEOF
 		     && width-- != 0 && !iswspace (wi))
 		{
 		  *p++ = (wchar_t) wi;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		  p_siz = realloc_m_ptr (wchar_t, p, p0, p_p, p_siz);
-#endif
 		  nread++;
 		}
 	      if (wi != WEOF)
 		_ungetwc_r (rptr, wi, fp);
 	      *p = L'\0';
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      shrink_m_ptr (wchar_t, p_p, p - p0 + 1, p_siz);
-#endif
 	      nassigned++;
 	    }
 	  else
 	    {
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      char **mbp_p = NULL;
-	      char *mbp0 = NULL;
-	      size_t mbp_siz = 0;
-#endif
-
-	      if (flags & SUPPRESS)
-		mbp = mbbuf;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-	      else if (flags & MALLOC)
-		mbp_siz = alloc_m_ptr (char, mbp, mbp0, mbp_p, 32);
-#endif
-	      else
+	      if (!(flags & SUPPRESS))
 		mbp = GET_ARG(N, ap, char *);
-	      memset ((void *) &mbs, '\0', sizeof (mbstate_t));
+	      memset ((_PTR) &mbs, '\0', sizeof (mbstate_t));
 	      while ((wi = _fgetwc_r (rptr, fp)) != WEOF
 		     && width != 0 && !iswspace (wi))
 		{
-		  nconv = wcrtomb(mbp, wi, &mbs);
-		  if (nconv == (size_t)-1)
-		    goto input_failure;
-		  /* Ignore high surrogate in width counting */
-		  if (nconv != 0 || mbs.__count != -4)
-		    width--;
-		  if (!(flags & SUPPRESS))
+		  if (width >= MB_CUR_MAX && !(flags & SUPPRESS))
 		    {
-		      mbp += nconv;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		      mbp_siz = realloc_m_ptr (char, mbp, mbp0, mbp_p, mbp_siz);
-#endif
+		      nconv = wcrtomb(mbp, wi, &mbs);
+		      if (nconv == (size_t)-1)
+			goto input_failure;
 		    }
+		  else
+		    {
+		      nconv = wcrtomb(mbbuf, wi, &mbs);
+		      if (nconv == (size_t)-1)
+			goto input_failure;
+		      if (nconv > width)
+			break;
+		      if (!(flags & SUPPRESS))
+			memcpy(mbp, mbbuf, nconv);
+		    }
+		  if (!(flags & SUPPRESS))
+		    mbp += nconv;
+		  width -= nconv;
 		  nread++;
 		}
 	      if (wi != WEOF)
@@ -1102,9 +962,6 @@ __SVFWSCANF_R (struct _reent *rptr,
 	      if (!(flags & SUPPRESS))
 		{
 		  *mbp = 0;
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-		  shrink_m_ptr (char, mbp_p, mbp - mbp0 + 1, mbp_siz);
-#endif
 		  nassigned++;
 		}
 	    }
@@ -1316,14 +1173,14 @@ __SVFWSCANF_R (struct _reent *rptr,
 	  char nancount = 0;
 	  char infcount = 0;
 #ifdef hardway
-	  if (width == 0 || width > sizeof (buf) / sizeof (*buf) - 1)
+	  if (width == 0 || width > sizeof (buf) - 1)
 #else
 	  /* size_t is unsigned, hence this optimisation */
-	  if (width - 1 > sizeof (buf) / sizeof (*buf) - 2)
+	  if (width - 1 > sizeof (buf) - 2)
 #endif
 	    {
-	      width_left = width - (sizeof (buf) / sizeof (*buf) - 1);
-	      width = sizeof (buf) / sizeof (*buf) - 1;
+	      width_left = width - (sizeof (buf) - 1);
+	      width = sizeof (buf) - 1;
 	    }
 	  flags |= SIGNOK | NDIGITS | DPTOK | EXPOK;
 	  zeroes = 0;
@@ -1574,10 +1431,8 @@ __SVFWSCANF_R (struct _reent *rptr,
 
 		  /* If there might not be enough space for the new exponent,
 		     truncate some trailing digits to make room.  */
-		  if (exp_start >= buf + sizeof (buf) / sizeof (*buf)
-				   - MAX_LONG_LEN)
-		    exp_start = buf + sizeof (buf) / sizeof (*buf)
-				- MAX_LONG_LEN - 1;
+		  if (exp_start >= buf + sizeof (buf) - MAX_LONG_LEN)
+		    exp_start = buf + sizeof (buf) - MAX_LONG_LEN - 1;
                  swprintf (exp_start, MAX_LONG_LEN, L"e%ld", new_exp);
 		}
 
@@ -1603,7 +1458,7 @@ __SVFWSCANF_R (struct _reent *rptr,
 		{
 		  flp = GET_ARG (N, ap, float *);
 		  if (isnan (res))
-		    *flp = nanf ("");
+		    *flp = nanf (NULL);
 		  else
 		    *flp = res;
 		}
@@ -1619,14 +1474,12 @@ input_failure:
      should have been set prior to here.  On EOF failure (including
      invalid format string), return EOF if no matches yet, else number
      of matches made prior to failure.  */
-  nassigned = nassigned && !(fp->_flags & __SERR) ? nassigned : EOF;
+  _newlib_flockfile_exit (fp);
+  return nassigned && !(fp->_flags & __SERR) ? nassigned : EOF;
 match_failure:
 all_done:
   /* Return number of matches, which can be 0 on match failure.  */
   _newlib_flockfile_end (fp);
-#ifdef _WANT_IO_POSIX_EXTENSIONS
-  free_m_ptr ();
-#endif
   return nassigned;
 }
 
