@@ -23,7 +23,7 @@ INDEX
 INDEX
 	_strtod_r
 
-SYNOPSIS
+ANSI_SYNOPSIS
         #include <stdlib.h>
         double strtod(const char *restrict <[str]>, char **restrict <[tail]>);
         float strtof(const char *restrict <[str]>, char **restrict <[tail]>);
@@ -42,6 +42,21 @@ SYNOPSIS
         double _strtod_r(void *<[reent]>,
                          const char *restrict <[str]>, char **restrict <[tail]>);
 
+TRAD_SYNOPSIS
+        #include <stdlib.h>
+        double strtod(<[str]>,<[tail]>)
+        char *<[str]>;
+        char **<[tail]>;
+
+        float strtof(<[str]>,<[tail]>)
+        char *<[str]>;
+        char **<[tail]>;
+
+        double _strtod_r(<[reent]>,<[str]>,<[tail]>)
+	char *<[reent]>;
+        char *<[str]>;
+        char **<[tail]>;
+
 DESCRIPTION
 	<<strtod>>, <<strtof>>, <<strtold>> parse the character string
 	<[str]>, producing a substring which can be converted to a double,
@@ -137,6 +152,7 @@ THIS SOFTWARE.
 #include <string.h>
 #include "mprec.h"
 #include "gdtoa.h"
+#include "gd_qnan.h"
 #include "../locale/setlocale.h"
 
 /* #ifndef NO_FENV_H */
@@ -151,7 +167,7 @@ THIS SOFTWARE.
 #undef tinytens
 /* The factor of 2^106 in tinytens[4] helps us avoid setting the underflow */
 /* flag unnecessarily.  It leads to a song and dance at the end of strtod. */
-static const double tinytens[] = { 1e-16, 1e-32,
+static _CONST double tinytens[] = { 1e-16, 1e-32,
 #ifdef _DOUBLE_IS_32BITS
 				    0.0, 0.0, 0.0
 #else
@@ -171,17 +187,10 @@ static const double tinytens[] = { 1e-16, 1e-32,
 #define Rounding Flt_Rounds
 #endif
 
-#ifdef IEEE_MC68k
-#define _0 0
-#define _1 1
-#else
-#define _0 1
-#define _1 0
-#endif
-
 #ifdef Avoid_Underflow /*{*/
  static double
-sulp (U x,
+_DEFUN (sulp, (x, scale),
+       	U x _AND
 	int scale)
 {
         U u;
@@ -203,9 +212,10 @@ sulp (U x,
 #ifndef NO_HEX_FP
 
 static void
-ULtod (__ULong *L,
-	__ULong *bits,
-	Long exp,
+_DEFUN (ULtod, (L, bits, exp, k),
+	__ULong *L _AND
+	__ULong *bits _AND
+	Long exp _AND
 	int k)
 {
 	switch(k & STRTOG_Retmask) {
@@ -248,7 +258,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 #endif
 	int bb2, bb5, bbe, bd2, bd5, bbbits, bs2, c, decpt, dsign,
 		 e, e1, esign, i, j, k, nd, nd0, nf, nz, nz0, sign;
-	const char *s, *s0, *s1;
+	_CONST char *s, *s0, *s1;
 	double aadj, adj;
 	U aadj1, rv, rv0;
 	Long L;
@@ -263,13 +273,8 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 #ifdef Honor_FLT_ROUNDS
 	int rounding;
 #endif
-#ifdef __HAVE_LOCALE_INFO__
-	const char *decimal_point = __get_numeric_locale(loc)->decimal_point;
-	const int dec_len = strlen (decimal_point);
-#else
-	const char *decimal_point = ".";
-	const int dec_len = 1;
-#endif
+	struct lconv *lconv = __localeconv_l (loc);
+	int dec_len = strlen (lconv->decimal_point);
 
 	delta = bs = bd = NULL;
 	sign = nz0 = nz = decpt = 0;
@@ -298,7 +303,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 	if (*s == '0') {
 #ifndef NO_HEX_FP
 		{
-		static const FPI fpi = { 53, 1-1023-53+1, 2046-1023-53+1, 1, SI };
+		static _CONST FPI fpi = { 53, 1-1023-53+1, 2046-1023-53+1, 1, SI };
 		Long exp;
 		__ULong bits[2];
 		switch(s[1]) {
@@ -331,11 +336,6 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 					Bfree(ptr,bb);
 					}
 				ULtod(rv.i, bits, exp, i);
-#ifndef NO_ERRNO
-                                /* try to avoid the bug of testing an 8087 register value */
-                                if ((dword0(rv)&Exp_mask) == 0)
-                                    errno = ERANGE;
-#endif
 			  }}
 			goto ret;
 		  }
@@ -354,7 +354,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 		else
 			z = 10*z + c - '0';
 	nd0 = nd;
-	if (strncmp (s, decimal_point, dec_len) == 0)
+	if (strncmp (s, lconv->decimal_point, dec_len) == 0)
 		{
 		decpt = 1;
 		c = *(s += dec_len);
@@ -430,7 +430,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 #ifdef INFNAN_CHECK
 			/* Check for Nan and Infinity */
 			__ULong bits[2];
-			static const FPI fpinan =	/* only 52 explicit bits */
+			static _CONST FPI fpinan =	/* only 52 explicit bits */
 				{ 52, 1-1023-53+1, 2046-1023-53+1, 1, SI };
 			if (!decpt)
 			 switch(c) {
@@ -461,7 +461,10 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 						}
 					else {
 #endif
-						dval(rv) = nan ("");
+						dword0(rv) = NAN_WORD0;
+#ifndef _DOUBLE_IS_32BITS
+						dword1(rv) = NAN_WORD1;
+#endif /*!_DOUBLE_IS_32BITS*/
 #ifndef No_Hex_NaN
 						}
 #endif
@@ -1200,16 +1203,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 #endif
 		if (y == z) {
 			/* Can we stop now? */
-#ifndef _DOUBLE_IS_32BITS
-			/* If FE_INVALID floating point exceptions are
-			   enabled, a conversion to a 32 bit value is
-			   dangerous.  A positive double value can result
-			   in a negative 32 bit int, thus raising SIGFPE.
-			   To avoid this, always convert into 64 bit here. */
-			__int64_t L = (__int64_t)aadj;
-#else
 			L = (Long)aadj;
-#endif
 			aadj -= L;
 			/* The tolerances below are conservative. */
 			if (dsign || dword1(rv) || dword0(rv) & Bndry_mask) {
@@ -1248,7 +1242,7 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 		dval(rv) *= dval(rv0);
 #ifndef NO_ERRNO
 		/* try to avoid the bug of testing an 8087 register value */
-		if ((dword0(rv) & Exp_mask) == 0)
+		if (dword0(rv) == 0 && dword1(rv) == 0)
 			ptr->_errno = ERANGE;
 #endif
 		}
@@ -1273,8 +1267,9 @@ _strtod_l (struct _reent *ptr, const char *__restrict s00, char **__restrict se,
 }
 
 double
-_strtod_r (struct _reent *ptr,
-	const char *__restrict s00,
+_DEFUN (_strtod_r, (ptr, s00, se),
+	struct _reent *ptr _AND
+	_CONST char *__restrict s00 _AND
 	char **__restrict se)
 {
   return _strtod_l (ptr, s00, se, __get_current_locale ());
@@ -1289,7 +1284,8 @@ strtod_l (const char *__restrict s00, char **__restrict se, locale_t loc)
 }
 
 double
-strtod (const char *__restrict s00, char **__restrict se)
+_DEFUN (strtod, (s00, se),
+	_CONST char *__restrict s00 _AND char **__restrict se)
 {
   return _strtod_l (_REENT, s00, se, __get_current_locale ());
 }
@@ -1297,52 +1293,21 @@ strtod (const char *__restrict s00, char **__restrict se)
 float
 strtof_l (const char *__restrict s00, char **__restrict se, locale_t loc)
 {
-  double val = _strtod_l (_REENT, s00, se, loc);
-  if (isnan (val))
-    return signbit (val) ? -nanf ("") : nanf ("");
-  float retval = (float) val;
-#ifndef NO_ERRNO
-  if (isinf (retval) && !isinf (val))
-    _REENT->_errno = ERANGE;
-#endif
-  return retval;
-}
-
-/*
- * These two functions are not quite correct as they return true for
- * zero, however they are 'good enough' for the test in strtof below
- * as we only need to know whether the double test is false when
- * the float test is true.
- */
-static inline int
-isdenorm(double d)
-{
-    U u;
-    dval(u) = d;
-    return (dword0(u) & Exp_mask) == 0;
-}
-
-static inline int
-isdenormf(float f)
-{
-    union { float f; __uint32_t i; } u;
-    u.f = f;
-    return (u.i & 0x7f800000) == 0;
+  double retval = _strtod_l (_REENT, s00, se, loc);
+  if (isnan (retval))
+    return nanf (NULL);
+  return (float)retval;
 }
 
 float
-strtof (const char *__restrict s00,
+_DEFUN (strtof, (s00, se),
+	_CONST char *__restrict s00 _AND
 	char **__restrict se)
 {
-  double val = _strtod_l (_REENT, s00, se, __get_current_locale ());
-  if (isnan (val))
-    return signbit (val) ? -nanf ("") : nanf ("");
-  float retval = (float) val;
-#ifndef NO_ERRNO
-  if ((isinf (retval) && !isinf (val)) || (isdenormf(retval) && !isdenorm(val)))
-    _REENT->_errno = ERANGE;
-#endif
-  return retval;
+  double retval = _strtod_l (_REENT, s00, se, __get_current_locale ());
+  if (isnan (retval))
+    return nanf (NULL);
+  return (float)retval;
 }
 
 #endif
