@@ -50,5 +50,5 @@ extern int cygserver_running;
     } while (false)
 
 #define debug_printf(f,...)	debug((f),##__VA_ARGS__)
-#define syscall_printf(f,...)	log(LOG_INFO,(f),##__VA_ARGS__)
+#define syscall_printf(f,...)	log(LOG_ERR,(f),##__VA_ARGS__)
 #define system_printf(f,...)	log(LOG_ERR,(f),##__VA_ARGS__)
