@@ -21,8 +21,7 @@ enum child_status
 {
   _CI_STRACED	 = 0x01,
   _CI_ISCYGWIN	 = 0x02,
-  _CI_SAW_CTRL_C = 0x04,
-  _CI_SILENTFAIL = 0x08
+  _CI_SAW_CTRL_C = 0x04
 };
 
 #define OPROC_MAGIC_MASK 0xff00ff00
@@ -37,7 +36,9 @@ enum child_status
 #define EXEC_MAGIC_SIZE sizeof(child_info)
 
 /* Change this value if you get a message indicating that it is out-of-sync. */
-#define CURR_CHILD_INFO_MAGIC 0xacbf4682U
+#define CURR_CHILD_INFO_MAGIC 0x30ea98f6U
+
+#define NPROCS	256
 
 #include "pinfo.h"
 struct cchildren
@@ -71,8 +72,6 @@ public:
   unsigned fhandler_union_cb;
   DWORD exit_code;	// process exit code
   static int retry_count;// retry count;
-  sigset_t sigmask;
-
   child_info (unsigned, child_info_types, bool);
   child_info (): subproc_ready (NULL), parent (NULL) {}
   ~child_info ();
@@ -83,7 +82,6 @@ public:
   bool isstraced () const {return !!(flag & _CI_STRACED);}
   bool iscygwin () const {return !!(flag & _CI_ISCYGWIN);}
   bool saw_ctrl_c () const {return !!(flag & _CI_SAW_CTRL_C);}
-  bool silentfail () const {return !!(flag & _CI_SILENTFAIL);}
   void prefork (bool = false);
   void cleanup ();
   void postfork (pinfo& child)
@@ -93,13 +91,6 @@ public:
     child.set_rd_proc_pipe (rd_proc_pipe);
     rd_proc_pipe = NULL;
   }
-  void silentfail (bool f)
-  {
-    if (f)
-      flag |= _CI_SILENTFAIL;
-    else
-      flag &= ~_CI_SILENTFAIL;
-  }
 };
 
 class mount_info;
@@ -115,6 +106,7 @@ public:
   void *stackbase;	// StackBase of parent thread
   size_t guardsize;     // size of POSIX guard region or (size_t) -1 if
 			// user stack
+  bool from_main;	// true if started from parent's main thread
   char filler[4];
   child_info_fork ();
   void __reg1 handle_fork ();
@@ -132,6 +124,7 @@ public:
   int envc;
   char **envp;
   HANDLE myself_pinfo;
+  sigset_t sigmask;
   int nchildren;
   cchildren children[0];
   static cygheap_exec_info *alloc ();
@@ -143,8 +136,6 @@ class child_info_spawn: public child_info
 {
   HANDLE hExeced;
   HANDLE ev;
-  HANDLE sem;
-  pid_t cygpid;
 public:
   cygheap_exec_info *moreinfo;
   int __stdin;
@@ -159,11 +150,6 @@ public:
   void *operator new (size_t, void *p) __attribute__ ((nothrow)) {return p;}
   void set (child_info_types ci, bool b) { new (this) child_info_spawn (ci, b);}
   void __reg1 handle_spawn ();
-  void set_sem (HANDLE _sem)
-  {
-    /* Don't leak semaphore handle into exec'ed process. */
-    SetHandleInformation (sem = _sem, HANDLE_FLAG_INHERIT, 0);
-  }
   bool set_saw_ctrl_c ()
   {
     if (!has_execed ())
@@ -193,8 +179,8 @@ public:
   bool get_parent_handle ();
   bool has_execed_cygwin () const { return iscygwin () && has_execed (); }
   operator HANDLE& () {return hExeced;}
-  int __reg3 worker (const char *, const char *const *, const char *const [],
-		     int, int = -1, int = -1);
+  int __reg3 worker (const char *, const char *const *, const char *const [], int,
+	      int = -1, int = -1);;
 };
 
 extern child_info_spawn ch_spawn;
