@@ -7,7 +7,7 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include "create_posix_thread.h"
+#include "miscfuncs.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include "glob.h"
@@ -29,12 +29,11 @@ details. */
 #include "shared_info.h"
 #include "cygwin_version.h"
 #include "dll_init.h"
-#include "cygmalloc.h"
 #include "heap.h"
 #include "tls_pbuf.h"
 #include "exception.h"
 #include "cygxdr.h"
-#include <fenv.h>
+#include "fenv.h"
 #include "ntdll.h"
 
 #define MAX_AT_FILE_LEVEL 10
@@ -61,10 +60,10 @@ do_global_dtors ()
     }
 }
 
-static void
+static void __stdcall
 do_global_ctors (void (**in_pfunc)(), int force)
 {
-  if (!force && __in_forkee == FORKING)
+  if (!force && in_forkee)
     return;		// inherit constructed stuff from parent pid
 
   /* Run ctors backwards, so skip the first entry and find how many
@@ -84,7 +83,7 @@ do_global_ctors (void (**in_pfunc)(), int force)
  * A \@file is replaced with @file so that echo \@foo would print
  * @foo and not the contents of foo.
  */
-static bool
+static bool __stdcall
 insert_file (char *name, char *&cmd)
 {
   HANDLE f;
@@ -203,7 +202,7 @@ quoted (char *cmd, int winshell)
 			    && isalpha ((s)[2]) \
 			    && strchr ((s) + 3, '\\')))
 
-static int
+static int __stdcall
 globify (char *word, char **&argv, int &argc, int &argvlen)
 {
   if (*word != '~' && strpbrk (word, "?*[\"\'(){}") == NULL)
@@ -236,13 +235,12 @@ globify (char *word, char **&argv, int &argc, int &argvlen)
 	char quote = *s;
 	while (*++s && *s != quote)
 	  {
-	    mbstate_t mbs = { 0 };
 	    if (dos_spec || *s != '\\')
 	      /* nothing */;
 	    else if (s[1] == quote || s[1] == '\\')
 	      s++;
 	    *p++ = '\\';
-	    size_t cnt = isascii (*s) ? 1 : mbrtowi (NULL, s, MB_CUR_MAX, &mbs);
+	    size_t cnt = isascii (*s) ? 1 : mbtowc (NULL, s, MB_CUR_MAX);
 	    if (cnt <= 1 || cnt == (size_t)-1)
 	      *p++ = *s;
 	    else
@@ -291,7 +289,7 @@ globify (char *word, char **&argv, int &argc, int &argvlen)
 
 /* Build argv, argc from string passed from Windows.  */
 
-static void
+static void __stdcall
 build_argv (char *cmd, char **&argv, int &argc, int winshell)
 {
   int argvlen = 0;
@@ -366,7 +364,7 @@ build_argv (char *cmd, char **&argv, int &argc, int winshell)
 }
 
 /* sanity and sync check */
-void
+void __stdcall
 check_sanity_and_sync (per_process *p)
 {
   /* Sanity check to make sure developers didn't change the per_process    */
@@ -375,7 +373,14 @@ check_sanity_and_sync (per_process *p)
   if (sizeof (per_process) != SIZEOF_PER_PROCESS)
     api_fatal ("per_process sanity check failed");
 
-  /* magic_biscuit must be SIZEOF_PER_PROCESS.  */
+  /* Make sure that the app and the dll are in sync. */
+
+  /* Complain if older than last incompatible change */
+  if (p->dll_major < CYGWIN_VERSION_DLL_EPOCH)
+    api_fatal ("cygwin DLL and APP are out of sync -- DLL version mismatch %u < %u",
+	       p->dll_major, CYGWIN_VERSION_DLL_EPOCH);
+
+  /* magic_biscuit != 0 if using the old style version numbering scheme.  */
   if (p->magic_biscuit != SIZEOF_PER_PROCESS)
     api_fatal ("Incompatible cygwin .dll -- incompatible per_process info %u != %u",
 	       p->magic_biscuit, SIZEOF_PER_PROCESS);
@@ -384,6 +389,14 @@ check_sanity_and_sync (per_process *p)
   if (p->api_major > cygwin_version.api_major)
     api_fatal ("cygwin DLL and APP are out of sync -- API version mismatch %u > %u",
 	       p->api_major, cygwin_version.api_major);
+
+#ifndef __x86_64__
+  /* This is a kludge to work around a version of _cygwin_common_crt0
+     which overwrote the cxx_malloc field with the local DLL copy.
+     Hilarity ensues if the DLL is not loaded while the process
+     is forking. */
+  __cygwin_user_data.cxx_malloc = &default_cygwin_cxx_malloc;
+#endif
 }
 
 child_info NO_COPY *child_proc_info;
@@ -528,14 +541,17 @@ get_cygwin_startup_info ()
     {
       if ((res->intro & OPROC_MAGIC_MASK) == OPROC_MAGIC_GENERIC)
 	multiple_cygwin_problem ("proc intro", res->intro, 0);
+      else if (res->cygheap != (void *) &_cygheap_start)
+	multiple_cygwin_problem ("cygheap base", (uintptr_t) res->cygheap,
+				 (uintptr_t) &_cygheap_start);
 
       unsigned should_be_cb = 0;
       switch (res->type)
 	{
 	  case _CH_FORK:
-	    __in_forkee = FORKING;
+	    in_forkee = true;
 	    should_be_cb = sizeof (child_info_fork);
-	    fallthrough;
+	    /* fall through */;
 	  case _CH_SPAWN:
 	  case _CH_EXEC:
 	    if (!should_be_cb)
@@ -554,7 +570,7 @@ get_cygwin_startup_info ()
 	    break;
 	  default:
 	    system_printf ("unknown exec type %u", res->type);
-	    fallthrough;
+	    /* intentionally fall through */
 	  case _CH_WHOOPS:
 	    res = NULL;
 	    break;
@@ -564,10 +580,17 @@ get_cygwin_startup_info ()
   return res;
 }
 
+#ifdef __x86_64__
 #define dll_data_start &__data_start__
 #define dll_data_end &__data_end__
 #define dll_bss_start &__bss_start__
 #define dll_bss_end &__bss_end__
+#else
+#define dll_data_start &_data_start__
+#define dll_data_end &_data_end__
+#define dll_bss_start &_bss_start__
+#define dll_bss_end &_bss_end__
+#endif
 
 void
 child_info_fork::handle_fork ()
@@ -578,7 +601,7 @@ child_info_fork::handle_fork ()
   myself->uid = cygheap->user.real_uid;
   myself->gid = cygheap->user.real_gid;
 
-  child_copy (parent, false, silentfail (),
+  child_copy (parent, false,
 	      "dll data", dll_data_start, dll_data_end,
 	      "dll bss", dll_bss_start, dll_bss_end,
 	      "user heap", cygheap->user_heap.base, cygheap->user_heap.ptr,
@@ -602,25 +625,20 @@ child_info_fork::handle_fork ()
 
   /* step 2 now that the dll has its heap filled in, we can fill in the
      user's data and bss since user_data is now filled out. */
-  child_copy (parent, false, silentfail (),
+  child_copy (parent, false,
 	      "data", user_data->data_start, user_data->data_end,
 	      "bss", user_data->bss_start, user_data->bss_end,
 	      NULL);
 
   if (fixup_mmaps_after_fork (parent))
     api_fatal ("recreate_mmaps_after_fork_failed");
-
-  /* We need to occupy the address space for dynamically loaded dlls
-     before we allocate any dynamic object, or we may end up with
-     error "address space needed by <dll> is already occupied"
-     for no good reason (seen with some relocated dll). */
-  dlls.reserve_space ();
 }
 
 bool
 child_info_spawn::get_parent_handle ()
 {
-  parent = OpenProcess (PROCESS_VM_READ, FALSE, parent_winpid);
+  parent = OpenProcess (PROCESS_VM_READ, false, parent_winpid);
+  moreinfo->myself_pinfo = NULL;
   return !!parent;
 }
 
@@ -628,22 +646,17 @@ void
 child_info_spawn::handle_spawn ()
 {
   extern void fixup_lockf_after_exec (bool);
-  HANDLE h = INVALID_HANDLE_VALUE;
+  HANDLE h;
   if (!dynamically_loaded || get_parent_handle ())
       {
 	cygheap_fixup_in_child (true);
 	memory_init ();
       }
-
-  cygheap->pid = cygpid;
-
-  /* Spawned process sets h to INVALID_HANDLE_VALUE to notify
-     pinfo::thisproc not to create another pid. */
   if (!moreinfo->myself_pinfo ||
       !DuplicateHandle (GetCurrentProcess (), moreinfo->myself_pinfo,
 			GetCurrentProcess (), &h, 0,
 			FALSE, DUPLICATE_SAME_ACCESS | DUPLICATE_CLOSE_SOURCE))
-    h = (type == _CH_SPAWN) ? INVALID_HANDLE_VALUE : NULL;
+    h = NULL;
 
   /* Setup our write end of the process pipe.  Clear the one in the structure.
      The destructor should never be called for this but, it can't hurt to be
@@ -672,11 +685,12 @@ child_info_spawn::handle_spawn ()
 
   ready (true);
 
-  if (child_proc_info->parent)
+  /* Keep pointer to parent open if we've execed so that pid will not be reused.
+     Otherwise, we no longer need this handle so close it.
+     Need to do this after debug_fixup_after_fork_exec or DEBUGGING handling of
+     handles might get confused. */
+  if (type != _CH_EXEC && child_proc_info->parent)
     {
-      /* We no longer need this handle so close it.  Need to do
-	 this after debug_fixup_after_fork_exec or DEBUGGING handling of
-	 handles might get confused. */
       CloseHandle (child_proc_info->parent);
       child_proc_info->parent = NULL;
     }
@@ -698,15 +712,15 @@ init_windows_system_directory ()
 	api_fatal ("can't find windows system directory");
       windows_system_directory[windows_system_directory_length++] = L'\\';
       windows_system_directory[windows_system_directory_length] = L'\0';
-      /* We need the Windows dir with NT prefix in path.cc.  Note that we
-	 don't append a backslash, because we need the dir without backslash
-	 for the environment. */
-      wcpcpy (windows_directory_buf, L"\\??\\");
-      windows_directory_length =
-	    GetSystemWindowsDirectoryW (windows_directory, MAX_PATH - 4);
-      RtlInitCountedUnicodeString (&windows_directory_path,
-	    windows_directory_buf,
-	    (windows_directory_length + 4) * sizeof (WCHAR));
+#ifndef __x86_64__
+      system_wow64_directory_length =
+	GetSystemWow64DirectoryW (system_wow64_directory, MAX_PATH);
+      if (system_wow64_directory_length)
+	{
+	  system_wow64_directory[system_wow64_directory_length++] = L'\\';
+	  system_wow64_directory[system_wow64_directory_length] = L'\0';
+	}
+#endif /* !__x86_64__ */
     }
 }
 
@@ -722,6 +736,10 @@ dll_crt0_0 ()
   SetErrorMode (SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX);
 
   lock_process::init ();
+  _impure_ptr = _GLOBAL_REENT;
+  _impure_ptr->_stdin = &_impure_ptr->__sf[0];
+  _impure_ptr->_stdout = &_impure_ptr->__sf[1];
+  _impure_ptr->_stderr = &_impure_ptr->__sf[2];
   user_data->impure_ptr = _impure_ptr;
   user_data->impure_ptr_ptr = &_impure_ptr;
 
@@ -766,8 +784,10 @@ dll_crt0_0 ()
   if (!dynamically_loaded)
     sigproc_init ();
 
+#ifdef __x86_64__
   /* See comment preceeding myfault_altstack_handler in exception.cc. */
   AddVectoredContinueHandler (0, myfault_altstack_handler);
+#endif
 
   debug_printf ("finished dll_crt0_0 initialization");
 }
@@ -776,7 +796,7 @@ static inline void
 main_thread_sinit ()
 {
   __sinit (_impure_ptr);
-  /* At this point, _impure_ptr == _GLOBAL_REENT is
+  /* At this point, _impure_ptr == _global_impure_ptr == _GLOBAL_REENT is
      initialized, but _REENT == _my_tls.local_clib doesn't know about it.
      It has been copied over from _GLOBAL_REENT in _cygtls::init_thread
      *before* the initialization took place.
@@ -784,13 +804,15 @@ main_thread_sinit ()
      As soon as the main thread calls a stdio function, this would be
      rectified.  But if another thread calls a stdio function on
      stdin/out/err before the main thread does, all the required
-     initialization of stdin/out/err will be done, but _REENT->__cleanup
-     is *still* NULL.  This in turn will result in a call to __sinit in the
+     initialization of stdin/out/err will be done, but _REENT->__sdidinit
+     is *still* 0.  This in turn will result in a call to __sinit in the
      wrong spot.  The input or output buffer will be NULLed and nothing is
      read or written in the first stdio function call in the main thread.
 
-     To fix this issue we set __cleanup to _cygtls::cleanup_early here. */
-  _REENT_CLEANUP(_REENT) = _cygtls::cleanup_early;
+     To fix this issue we have to copy over the relevant part of _GLOBAL_REENT
+     to _REENT here again. */
+  _REENT->__sdidinit = -1;
+  _REENT->__cleanup = _GLOBAL_REENT->__cleanup;
 }
 
 /* Take over from libc's crt0.o and start the application. Note the
@@ -804,15 +826,12 @@ dll_crt0_1 (void *)
 
   _my_tls.incyg++;
   /* Inherit "parent" exec'ed process sigmask */
-  if (spawn_info && __in_forkee != FORKING)
-    _my_tls.sigmask = spawn_info->sigmask;
+  if (spawn_info && !in_forkee)
+    _my_tls.sigmask = spawn_info->moreinfo->sigmask;
 
   if (dynamically_loaded)
     sigproc_init ();
 
-  /* Call this before accessing any files. */
-  RtlSetProcessPlaceholderCompatibilityMode (PHCM_EXPOSE_PLACEHOLDERS);
-
   check_sanity_and_sync (user_data);
 
   /* Initialize malloc and then call user_shared_initialize since it relies
@@ -844,7 +863,7 @@ dll_crt0_1 (void *)
 
   /* Initialize pthread mainthread when not forked and it is safe to call new,
      otherwise it is reinitalized in fixup_after_fork */
-  if (__in_forkee != FORKING)
+  if (!in_forkee)
     {
       pthread::init_mainthread ();
       _pei386_runtime_relocator (user_data);
@@ -854,8 +873,12 @@ dll_crt0_1 (void *)
   strace.microseconds ();
 #endif
 
+  /* Initialize debug muto, if DLL is built with --enable-debugging.
+     Need to do this before any helper threads start. */
+  debug_init ();
+
   cygbench ("pre-forkee");
-  if (__in_forkee == FORKING)
+  if (in_forkee)
     {
       /* Make sure to restore the TEB's stack info.  If guardsize is -1 the
 	 stack has been provided by the application and must not be deallocated
@@ -980,27 +1003,22 @@ dll_crt0_1 (void *)
     {
       /* Create a copy of Cygwin's version of __argv so that, if the user makes
 	 a change to an element of argv[] it does not affect Cygwin's argv.
-	 Changing the contents of what argv[n] points to will still affect
-	 Cygwin.  This is similar (but not exactly like) Linux.
-
-	 We used to allocate newargv on the stack, but all the rest of the
-	 argument and environment handling does not depend on the stack,
-	 as it does on Linux.  In fact, everything is stored by the parent
-	 in the cygheap, so the only reason this may fail is if we're out
-	 of resources in a big way.  If this malloc fails, we could either
-	 fail the entire process execution, or we could proceed with the
-	 original argv and potentially affect output of /proc/self/cmdline.
-	 We opt for the latter here because it's the lesser evil. */
-      char **newargv = (char **) malloc ((__argc + 1) * sizeof (char *));
-      if (newargv)
-	memcpy (newargv, __argv, (__argc + 1) * sizeof (char *));
-      else
-	newargv = __argv;
+	 Changing the the contents of what argv[n] points to will still
+	 affect Cygwin.  This is similar (but not exactly like) Linux. */
+      char *newargv[__argc + 1];
+      char **nav = newargv;
+      char **oav = __argv;
+      while ((*nav++ = *oav++) != NULL)
+	continue;
       /* Handle any signals which may have arrived */
       sig_dispatch_pending (false);
       _my_tls.call_signal_handler ();
       _my_tls.incyg--;	/* Not in Cygwin anymore */
-      cygwin_exit (user_data->main (__argc, newargv, environ));
+#ifdef __x86_64__
+      cygwin_exit (user_data->main (__argc, newargv, __cygwin_environ));
+#else
+      cygwin_exit (user_data->main (__argc, newargv, *user_data->envptr));
+#endif
     }
   __asm__ ("				\n\
 	.global _cygwin_exit_return	\n\
@@ -1011,9 +1029,10 @@ __cygwin_exit_return:			\n\
 ");
 }
 
-extern "C" void
+extern "C" void __stdcall
 _dll_crt0 ()
 {
+#ifdef __x86_64__
   /* Starting with Windows 10 rel 1511, the main stack of an application is
      not reproducible if a 64 bit process has been started from a 32 bit
      process.  Given that we have enough virtual address space on 64 bit
@@ -1022,15 +1041,18 @@ _dll_crt0 ()
      under our own control and avoids collision with the OS. */
   if (!dynamically_loaded)
     {
-      if (__in_forkee != FORKING)
+      if (!in_forkee || fork_info->from_main)
 	{
 	  /* Must be static since it's referenced after the stack and frame
 	     pointer registers have been changed. */
 	  static PVOID allocationbase;
-	  PVOID stackaddr = create_new_main_thread_stack (allocationbase);
+	  SIZE_T commitsize = in_forkee ? (PBYTE) fork_info->stackbase
+					  - (PBYTE) fork_info->stacklimit
+					: 0;
+	  PVOID stackaddr = create_new_main_thread_stack (allocationbase,
+							  commitsize);
 	  if (stackaddr)
 	    {
-#ifdef __x86_64__
 	      /* Set stack pointer to new address.  Set frame pointer to
 	         stack pointer and subtract 32 bytes for shadow space. */
 	      __asm__ ("\n\
@@ -1038,9 +1060,6 @@ _dll_crt0 ()
 		       movq  %%rsp, %%rbp  \n\
 		       subq  $32,%%rsp     \n"
 		       : : [ADDR] "r" (stackaddr));
-#else
-#error unimplemented for this target
-#endif
 	      /* We're on the new stack now.  Free up space taken by the former
 		 main thread stack and set DeallocationStack correctly. */
 	      VirtualFree (NtCurrentTeb ()->DeallocationStack, 0, MEM_RELEASE);
@@ -1050,8 +1069,13 @@ _dll_crt0 ()
       else
 	fork_info->alloc_stack ();
     }
+#else
+  main_environ = user_data->envptr;
+  if (in_forkee)
+    fork_info->alloc_stack ();
+#endif
 
-  fesetenv (FE_DFL_ENV);
+  _feinitialise ();
   _main_tls = &_my_tls;
   _main_tls->call ((DWORD (*) (void *, void *)) dll_crt0_1, NULL);
 }
@@ -1060,7 +1084,7 @@ void
 dll_crt0 (per_process *uptr)
 {
   /* Set the local copy of the pointer into the user space. */
-  if (__in_forkee != FORKING && uptr && uptr != user_data)
+  if (!in_forkee && uptr && uptr != user_data)
     {
       memcpy (user_data, uptr, per_process_overwrite);
       *(user_data->impure_ptr_ptr) = _GLOBAL_REENT;
@@ -1069,7 +1093,7 @@ dll_crt0 (per_process *uptr)
 }
 
 /* This must be called by anyone who uses LoadLibrary to load cygwin1.dll.
-   You must have __CYGTLS_PADSIZE__ bytes reserved at the bottom of the stack
+   You must have CYGTLS_PADSIZE bytes reserved at the bottom of the stack
    calling this function, and that storage must not be overwritten until you
    unload cygwin1.dll, as it is used for _my_tls.  It is best to load
    cygwin1.dll before spawning any additional threads in your process.
@@ -1079,10 +1103,18 @@ dll_crt0 (per_process *uptr)
 extern "C" void
 cygwin_dll_init ()
 {
+#ifndef __x86_64__
+  static char **envp;
+#endif
   static int _fmode;
 
   user_data->magic_biscuit = sizeof (per_process);
+
+#ifndef __x86_64__
+  user_data->envptr = &envp;
+#endif
   user_data->fmode_ptr = &_fmode;
+
   _dll_crt0 ();
 }
 
@@ -1104,7 +1136,7 @@ __main (void)
   sig_dispatch_pending (true);
 }
 
-void
+void __reg1
 do_exit (int status)
 {
   syscall_printf ("do_exit (%d), exit_state %d", status, exit_state);
@@ -1178,7 +1210,7 @@ do_exit (int status)
    a reproducible value which could also be easily evaluated in cygwin_atexit.
    However, when building C++ applications with -fuse-cxa-atexit, G++ creates
    calls to __cxa_atexit using the *address* of __dso_handle as DSO handle.
-
+   
    So what we do here is this:  A call to __cxa_atexit from the application
    actually calls cygwin__cxa_atexit.  From dso_handle (which is either
    &__dso_handle, or __dso_handle == ImageBase or NULL) we fetch the dll
@@ -1206,6 +1238,7 @@ cygwin_atexit (void (*fn) (void))
   int res;
 
   dll *d = dlls.find ((void *) _my_tls.retaddr ());
+#ifdef __x86_64__
   /* x86_64 DLLs created with GCC 4.8.3-3 register __gcc_deregister_frame
      as atexit function using a call to atexit, rather than __cxa_atexit.
      Due to GCC's tail call optimizing, cygwin_atexit doesn't get the correct
@@ -1222,11 +1255,12 @@ cygwin_atexit (void (*fn) (void))
      Workaround: If dlls.find fails, try to find the dll entry of the DLL
      containing fn.  If that works, proceed by calling __cxa_atexit, otherwise
      call atexit.
-
+     
      This *should* be sufficiently safe.  Ultimately, new applications will
      use the statically linked atexit function though, as outlined above. */
   if (!d)
     d = dlls.find ((void *) fn);
+#endif
   res = d ? __cxa_atexit ((void (*) (void *)) fn, NULL, d->handle) : atexit (fn);
   return res;
 }
@@ -1250,13 +1284,15 @@ extern "C" void
 vapi_fatal (const char *fmt, va_list ap)
 {
   char buf[4096];
-  int n = __small_sprintf (buf, "%P: *** fatal error %s- ",
-				(__in_forkee == FORKING)
-				? "in forked process " : "");
+  int n = __small_sprintf (buf, "%P: *** fatal error %s- ", in_forkee ? "in forked process " : "");
   __small_vsprintf (buf + n, fmt, ap);
   va_end (ap);
   strace.prntf (_STRACE_SYSTEM, NULL, "%s", buf);
-  api_fatal_debug();
+
+#ifdef DEBUGGING
+  try_to_debug ();
+#endif
+  cygwin_stackdump ();
   myself.exit (__api_fatal_exit_val);
 }
 
@@ -1272,7 +1308,7 @@ api_fatal (const char *fmt, ...)
 void
 multiple_cygwin_problem (const char *what, uintptr_t magic_version, uintptr_t version)
 {
-  if (_cygwin_testing && (strstr (what, "proc")))
+  if (_cygwin_testing && (strstr (what, "proc") || strstr (what, "cygheap")))
     {
       child_proc_info->type = _CH_WHOOPS;
       return;
@@ -1295,7 +1331,7 @@ are unable to find another cygwin DLL.",
 }
 
 #ifdef DEBUGGING
-void
+void __reg1
 cygbench (const char *s)
 {
   if (GetEnvironmentVariableA ("CYGWIN_BENCH", NULL, 0))
