@@ -19,6 +19,7 @@ typedef unsigned short _minor_t;
 #include <dirent.h>
 #include "cygheap_malloc.h"
 
+#define MAX_CONSOLES 63
 enum fh_devices
 {
   DEV_TTY_MAJOR = 5,
@@ -30,7 +31,7 @@ enum fh_devices
 
   DEV_CONS_MAJOR = 3,
   FH_CONS     = FHDEV (DEV_CONS_MAJOR, 0),
-  FH_CONS_MAX = FHDEV (DEV_CONS_MAJOR, 127),
+  FH_CONS_MAX = FHDEV (DEV_CONS_MAJOR, MAX_CONSOLES),
 
   DEV_PTYM_MAJOR = 128,
   FH_PTYM    = FHDEV (DEV_PTYM_MAJOR, 0),
@@ -71,9 +72,6 @@ enum fh_devices
   FH_DEV     = FHDEV (DEV_VIRTFS_MAJOR, 193),
   FH_CYGDRIVE= FHDEV (DEV_VIRTFS_MAJOR, 192),
 
-  FH_SIGNALFD= FHDEV (DEV_VIRTFS_MAJOR, 13),
-  FH_TIMERFD = FHDEV (DEV_VIRTFS_MAJOR, 14),
-
   DEV_FLOPPY_MAJOR = 2,
   FH_FLOPPY  = FHDEV (DEV_FLOPPY_MAJOR, 0),
 
@@ -238,15 +236,18 @@ enum fh_devices
   FH_FULL    = FHDEV (DEV_MEM_MAJOR, 7),
   FH_RANDOM  = FHDEV (DEV_MEM_MAJOR, 8),
   FH_URANDOM = FHDEV (DEV_MEM_MAJOR, 9),
+  FH_KMSG    = FHDEV (DEV_MEM_MAJOR, 11),
 
   DEV_SOUND_MAJOR = 14,
   FH_OSS_DSP = FHDEV (DEV_SOUND_MAJOR, 3),
 
-  DEV_SOCK_MAJOR = 30,
-  FH_SOCKET = FHDEV (DEV_SOCK_MAJOR, 0),
-  FH_INET = FHDEV (DEV_SOCK_MAJOR, 36),
-  FH_UNIX = FHDEV (DEV_SOCK_MAJOR, 42),
-  FH_LOCAL = FHDEV (DEV_SOCK_MAJOR, 120),
+  DEV_TCP_MAJOR = 30,
+  FH_TCP = FHDEV (DEV_TCP_MAJOR, 36),
+  FH_UDP = FHDEV (DEV_TCP_MAJOR, 39),
+  FH_ICMP = FHDEV (DEV_TCP_MAJOR, 33),
+  FH_UNIX = FHDEV (DEV_TCP_MAJOR, 120),
+  FH_STREAM = FHDEV (DEV_TCP_MAJOR, 121),
+  FH_DGRAM = FHDEV (DEV_TCP_MAJOR, 122),
 
   FH_NADA     = FHDEV (0, 0),
   FH_ERROR   = FHDEV (255, 255)	/* Set by fh constructor when error detected */
@@ -393,19 +394,15 @@ extern const _device *ptmx_dev;
 extern const _device *ptys_dev;
 extern const _device *urandom_dev;
 
-extern const _device dev_socket_storage;
-#define socket_dev ((device *) &dev_socket_storage)
-extern const _device dev_af_inet_storage;
-#define af_inet_dev ((device *) &dev_af_inet_storage)
-extern const _device dev_af_local_storage;
-#define af_local_dev ((device *) &dev_af_local_storage)
-extern const _device dev_af_unix_storage;
-#define af_unix_dev ((device *) &dev_af_unix_storage)
+extern const _device dev_dgram_storage;
+#define dgram_dev ((device *) &dev_dgram_storage)
+extern const _device dev_stream_storage;
+#define stream_dev ((device *) &dev_stream_storage)
+extern const _device dev_tcp_storage;
+#define tcp_dev ((device *) &dev_tcp_storage)
+extern const _device dev_udp_storage;
+#define udp_dev ((device *) &dev_udp_storage)
 
-extern const _device dev_signalfd_storage;
-#define signalfd_dev ((device *) &dev_signalfd_storage)
-extern const _device dev_timerfd_storage;
-#define timerfd_dev ((device *) &dev_timerfd_storage)
 extern const _device dev_piper_storage;
 #define piper_dev ((device *) &dev_piper_storage)
 extern const _device dev_pipew_storage;
