@@ -30,7 +30,6 @@ details. */
 #include "shared_info.h"
 #include "ntdll.h"
 
-/* If this is not NULL, it points to memory allocated by us. */
 static char **lastenviron;
 
 /* Parse CYGWIN options */
@@ -111,6 +110,7 @@ static struct parse_thing
       } values[2];
   } known[] NO_COPY =
 {
+  {"dosfilewarning", {&dos_file_warning}, setbool, NULL, {{false}, {true}}},
   {"error_start", {func: error_start_init}, isfunc, NULL, {{0}, {0}}},
   {"export", {&export_settings}, setbool, NULL, {{false}, {true}}},
   {"glob", {func: glob_init}, isfunc, NULL, {{0}, {s: "normal"}}},
@@ -119,7 +119,6 @@ static struct parse_thing
   {"reset_com", {&reset_com}, setbool, NULL, {{false}, {true}}},
   {"wincmdln", {&wincmdln}, setbool, NULL, {{false}, {true}}},
   {"winsymlinks", {func: set_winsymlinks}, isfunc, NULL, {{0}, {0}}},
-  {"disable_pcon", {&disable_pcon}, setbool, NULL, {{false}, {true}}},
   {NULL, {0}, setdword, 0, {{0}, {0}}}
 };
 
@@ -352,7 +351,7 @@ static const unsigned char conv_start_chars[256] =
 static inline char
 match_first_char (const char *s, unsigned char m)
 {
-  return conv_start_chars[*(unsigned char *)s] & m;
+  return conv_start_chars[(unsigned) *s] & m;
 }
 
 struct win_env&
@@ -455,7 +454,8 @@ posify_maybe (char **here, const char *value, char *outenv)
 
   memcpy (outenv, src, len);
   char *newvalue = outenv + len;
-  if (!conv->toposix (value, newvalue, NT_MAX_PATH - len) || errno != EIDRM)
+  if (!conv->toposix (value, newvalue, NT_MAX_PATH - len)
+      || _impure_ptr->_errno != EIDRM)
     conv->add_cache (newvalue, *value != '/' ? value : NULL);
   else
     {
@@ -484,9 +484,6 @@ my_findenv (const char *name, int *offset)
   register char **p;
   const char *c;
 
-  if (cur_environ () == NULL)
-    return NULL;
-
   c = name;
   len = 0;
   while (*c && *c != '=')
@@ -549,28 +546,13 @@ _getenv_r (struct _reent *, const char *name)
   return findenv_func (name, &offset);
 }
 
-/* Like getenv, but returns NULL if effective and real UID/GIDs do not match */
-extern "C" char *
-secure_getenv (const char *name)
-{
-  int offset;
-  if (cygheap->user.issetuid ())
-    return NULL;
-  return findenv_func (name, &offset);
-}
-
-/* Return number of environment entries, including terminating NULL. */
 static int __stdcall
 envsize (const char * const *in_envp)
 {
   const char * const *envp;
-
-  if (in_envp == NULL)
-    return 0;
-
   for (envp = in_envp; *envp; envp++)
     continue;
-  return 1 + envp - in_envp;
+  return (1 + envp - in_envp) * sizeof (const char *);
 }
 
 /* Takes similar arguments to setenv except that overwrite is
@@ -601,21 +583,19 @@ _addenv (const char *name, const char *value, int overwrite)
   else
     {				/* Create new slot. */
       int sz = envsize (cur_environ ());
+      int allocsz = sz + (2 * sizeof (char *));
 
-      /* If sz == 0, we need two new slots, one for the terminating NULL. */
-      int newsz = sz == 0 ? 2 : sz + 1;
-      int allocsz = newsz * sizeof (char *);
+      offset = (sz - 1) / sizeof (char *);
 
-      offset = newsz - 2;
-
-      /* Allocate space for additional element. */
+      /* Allocate space for additional element plus terminating NULL. */
       if (cur_environ () == lastenviron)
-	lastenviron = __cygwin_environ = (char **) realloc (lastenviron,
+	lastenviron = __cygwin_environ = (char **) realloc (cur_environ (),
 							    allocsz);
-      else if ((lastenviron = (char **) realloc (lastenviron, allocsz)) != NULL)
-	__cygwin_environ = (char **) memcpy (lastenviron, __cygwin_environ,
-					     sz * sizeof (char *));
-      if (!lastenviron)
+      else if ((lastenviron = (char **) malloc (allocsz)) != NULL)
+	__cygwin_environ = (char **) memcpy ((char **) lastenviron,
+					     __cygwin_environ, sz);
+
+      if (!__cygwin_environ)
 	{
 #ifdef DEBUGGING
 	  try_to_debug ();
@@ -644,7 +624,7 @@ _addenv (const char *name, const char *value, int overwrite)
 	return -1;		/* Oops.  No more memory. */
 
       /* Put name '=' value into current slot. */
-      memcpy (envhere, name, namelen);
+      strncpy (envhere, name, namelen);
       envhere[namelen] = '=';
       strcpy (envhere + namelen + 1, value);
     }
@@ -728,26 +708,6 @@ unsetenv (const char *name)
   return -1;
 }
 
-/* Clear the environment.  */
-extern "C" int
-clearenv (void)
-{
-  __try
-    {
-      if (cur_environ () == lastenviron)
-	{
-	  free (lastenviron);
-	  lastenviron = NULL;
-	}
-      __cygwin_environ = NULL;
-      update_envptrs ();
-      return 0;
-    }
-  __except (EFAULT) {}
-  __endtry
-  return -1;
-}
-
 /* Minimal list of Windows vars which must be converted to uppercase.
    Either for POSIX compatibility of for backward compatibility with
    existing applications. */
@@ -798,12 +758,18 @@ ucenv (char *p, const char *eq)
 void
 environ_init (char **envp, int envc)
 {
-  PWCHAR rawenv;
+  PWCHAR rawenv, w;
+  int i;
   char *p;
+  char *newp;
+  int sawTERM = 0;
   bool envp_passed_in;
+  static char NO_COPY cygterm[] = "TERM=cygwin";
+  tmp_pathbuf tp;
 
   __try
     {
+      char *tmpbuf = tp.t_get ();
       if (!envp)
 	envp_passed_in = 0;
       else
@@ -828,6 +794,9 @@ environ_init (char **envp, int envc)
 	  goto out;
 	}
 
+      /* Allocate space for environment + trailing NULL + CYGWIN env. */
+      lastenviron = envp = (char **) malloc ((4 + (envc = 100)) * sizeof (char *));
+
       rawenv = GetEnvironmentStringsW ();
       if (!rawenv)
 	{
@@ -836,8 +805,32 @@ environ_init (char **envp, int envc)
 	}
       debug_printf ("GetEnvironmentStrings returned %p", rawenv);
 
-      lastenviron = envp = win32env_to_cygenv (rawenv, true);
+      /* Current directory information is recorded as variables of the
+	 form "=X:=X:\foo\bar; these must be changed into something legal
+	 (we could just ignore them but maybe an application will
+	 eventually want to use them).  */
+      for (i = 0, w = rawenv; *w != L'\0'; w = wcschr (w, L'\0') + 1, i++)
+	{
+	  sys_wcstombs_alloc_no_path (&newp, HEAP_NOTHEAP, w);
+	  if (i >= envc)
+	    envp = (char **) realloc (envp, (4 + (envc += 100)) * sizeof (char *));
+	  envp[i] = newp;
+	  if (*newp == '=')
+	    *newp = '!';
+	  char *eq = strchrnul (newp, '=');
+	  ucenv (newp, eq);	/* uppercase env vars which need it */
+	  if (*newp == 'T' && strncmp (newp, "TERM=", 5) == 0)
+	    sawTERM = 1;
+	  else if (*newp == 'C' && strncmp (newp, "CYGWIN=", 7) == 0)
+	    parse_options (newp + 7);
+	  if (*eq)
+	    posify_maybe (envp + i, *++eq ? eq : --eq, tmpbuf);
+	  debug_printf ("%p: %s", envp[i], envp[i]);
+	}
 
+      if (!sawTERM)
+	envp[i++] = strdup (cygterm);
+      envp[i] = NULL;
       FreeEnvironmentStringsW (rawenv);
 
     out:
@@ -859,56 +852,6 @@ environ_init (char **envp, int envc)
   __endtry
 }
 
-int sawTERM = 0;
-
-char ** __reg2
-win32env_to_cygenv (PWCHAR rawenv, bool posify)
-{
-  tmp_pathbuf tp;
-  char **envp;
-  int envc;
-  char *newp;
-  int i;
-  const char cygterm[] = "TERM=cygwin";
-  const char xterm[] = "TERM=xterm-256color";
-  char *tmpbuf = tp.t_get ();
-  PWCHAR w;
-
-  /* Allocate space for environment + trailing NULL + CYGWIN env. */
-  envp = (char **) malloc ((4 + (envc = 100)) * sizeof (char *));
-
-  /* Current directory information is recorded as variables of the
-     form "=X:=X:\foo\bar; these must be changed into something legal
-     (we could just ignore them but maybe an application will
-     eventually want to use them).  */
-  for (i = 0, w = rawenv; *w != L'\0'; w = wcschr (w, L'\0') + 1, i++)
-    {
-      sys_wcstombs_alloc_no_path (&newp, HEAP_NOTHEAP, w);
-      if (i >= envc)
-        envp = (char **) realloc (envp, (4 + (envc += 100)) * sizeof (char *));
-      envp[i] = newp;
-      if (*newp == '=')
-        *newp = '!';
-      char *eq = strchrnul (newp, '=');
-      ucenv (newp, eq);    /* uppercase env vars which need it */
-      if (*newp == 'T' && strncmp (newp, "TERM=", 5) == 0)
-        sawTERM = 1;
-      else if (*newp == 'C' && strncmp (newp, "CYGWIN=", 7) == 0)
-        parse_options (newp + 7);
-      if (*eq && posify)
-        posify_maybe (envp + i, *++eq ? eq : --eq, tmpbuf);
-      debug_printf ("%p: %s", envp[i], envp[i]);
-    }
-
-  /* If console has 24 bit color capability, TERM=xterm-256color,
-     otherwise, TERM=cygwin */
-  if (!sawTERM)
-    envp[i++] = strdup (wincap.has_con_24bit_colors () ? xterm : cygterm);
-
-  envp[i] = NULL;
-  return envp;
-}
-
 /* Function called by qsort to sort environment strings.  */
 static int
 env_sort (const void *a, const void *b)
@@ -1067,13 +1010,8 @@ build_env (const char * const *envp, PWCHAR &envblock, int &envc,
   char **dstp;
   bool saw_spenv[SPENVS_SIZE] = {0};
 
-  static char *const empty_env[] = { NULL };
-
   debug_printf ("envp %p", envp);
 
-  if (!envp)
-    envp = empty_env;
-
   /* How many elements? */
   for (n = 0; envp[n]; n++)
     continue;
@@ -1295,11 +1233,10 @@ build_env (const char * const *envp, PWCHAR &envblock, int &envc,
 	 during execve. */
       if (!saw_PATH)
 	{
-	  new_tl += cygheap->installation_dir.Length / sizeof (WCHAR) + 5 + 1;
+	  new_tl += cygheap->installation_dir_len + 5;
 	  if (new_tl > tl)
 	    tl = raise_envblock (new_tl, envblock, s);
-	  s = wcpcpy (wcpcpy (s, L"PATH="),
-		      cygheap->installation_dir.Buffer) + 1;
+	  s = wcpcpy (wcpcpy (s, L"PATH="), cygheap->installation_dir) + 1;
 	}
       *s = L'\0';			/* Two null bytes at the end */
       assert ((s - envblock) <= tl);	/* Detect if we somehow ran over end
@@ -1310,7 +1247,7 @@ build_env (const char * const *envp, PWCHAR &envblock, int &envc,
   return newenv;
 }
 
-#ifdef __i386__
+#ifndef __x86_64__
 /* This idiocy is necessary because the early implementers of cygwin
    did not seem to know about importing data variables from the DLL.
    So, we have to synchronize cygwin's idea of the environment with the
