@@ -20,7 +20,6 @@ details. */
 #include "child_info.h"
 #include "cygtls.h"
 #include "tls_pbuf.h"
-#include "shared_info.h"
 #include "dll_init.h"
 #include "cygmalloc.h"
 #include "ntdll.h"
@@ -31,23 +30,18 @@ details. */
 /* FIXME: Once things stabilize, bump up to a few minutes.  */
 #define FORK_WAIT_TIMEOUT (300 * 1000)     /* 300 seconds */
 
-static int dofork (void **proc, bool *with_forkables);
 class frok
 {
-  frok (bool *forkables)
-    : with_forkables (forkables)
-  {}
-  bool *with_forkables;
   bool load_dlls;
   child_info_fork ch;
   const char *errmsg;
   int child_pid;
   int this_errno;
   HANDLE hchild;
-  int parent (volatile char * volatile here);
-  int child (volatile char * volatile here);
+  int __stdcall parent (volatile char * volatile here);
+  int __stdcall child (volatile char * volatile here);
   bool error (const char *fmt, ...);
-  friend int dofork (void **proc, bool *with_forkables);
+  friend int fork ();
 };
 
 static void
@@ -59,7 +53,7 @@ resume_child (HANDLE forker_finished)
 }
 
 /* Notify parent that it is time for the next step. */
-static void
+static void __stdcall
 sync_with_parent (const char *s, bool hang_self)
 {
   debug_printf ("signalling parent: %s", s);
@@ -131,21 +125,33 @@ child_info::prefork (bool detached)
     }
 }
 
-int
+int __stdcall
 frok::child (volatile char * volatile here)
 {
   HANDLE& hParent = ch.parent;
+  extern void fixup_hooks_after_fork ();
+  extern void fixup_timers_after_fork ();
+
+  /* NOTE: Logically this belongs in dll_list::load_after_fork, but by
+     doing it here, before the first sync_with_parent, we can exploit
+     the existing retry mechanism in hopes of getting a more favorable
+     address space layout next time. */
+  dlls.reserve_space ();
 
   sync_with_parent ("after longjmp", true);
   debug_printf ("child is running.  pid %d, ppid %d, stack here %p",
 		myself->pid, myself->ppid, __builtin_frame_address (0));
   sigproc_printf ("hParent %p, load_dlls %d", hParent, load_dlls);
 
-  /* Make sure threadinfo information is properly set up. */
-  if (&_my_tls != _main_tls)
+  /* If we've played with the stack, stacksize != 0.  That means that
+     fork() was invoked from other than the main thread.  Make sure that
+     the threadinfo information is properly set up.  */
+  if (!fork_info->from_main)
     {
       _main_tls = &_my_tls;
       _main_tls->init_thread (NULL, NULL);
+      _main_tls->local_clib = *_impure_ptr;
+      _impure_ptr = &_main_tls->local_clib;
     }
 
   set_cygwin_privileges (hProcToken);
@@ -175,19 +181,27 @@ frok::child (volatile char * volatile here)
   if (fixup_shms_after_fork ())
     api_fatal ("recreate_shm areas after fork failed");
 
-  /* load dynamic dlls, if any, re-track main-executable and cygwin1.dll */
-  dlls.load_after_fork (hParent);
-
-  cygheap->fdtab.fixup_after_fork (hParent);
-
-  /* Signal that we have successfully initialized, so the parent can
-     - transfer data/bss for dynamically loaded dlls (if any), or
-     - terminate the current fork call even if the child is initialized. */
-  sync_with_parent ("performed fork fixups and dynamic dll loading", true);
+  /* If we haven't dynamically loaded any dlls, just signal
+     the parent.  Otherwise, load all the dlls, tell the parent
+      that we're done, and wait for the parent to fill in the.
+      loaded dlls' data/bss. */
+  if (!load_dlls)
+    {
+      cygheap->fdtab.fixup_after_fork (hParent);
+      sync_with_parent ("performed fork fixup", false);
+    }
+  else
+    {
+      dlls.load_after_fork (hParent);
+      cygheap->fdtab.fixup_after_fork (hParent);
+      sync_with_parent ("loaded dlls", true);
+    }
 
+  init_console_handler (myself->ctty > 0);
   ForceCloseHandle1 (fork_info->forker_finished, forker_finished);
 
   pthread::atforkchild ();
+  fixup_timers_after_fork ();
   cygbench ("fork-child");
   ld_preload ();
   fixup_hooks_after_fork ();
@@ -196,13 +210,40 @@ frok::child (volatile char * volatile here)
      rd_proc_pipe that would be an invalid handle.  In the case of
      wr_proc_pipe it would be == my_wr_proc_pipe.  Both would be bad. */
   ch.rd_proc_pipe = ch.wr_proc_pipe = NULL;
-  CloseHandle (hParent);
-  hParent = NULL;
   cygwin_finished_initializing = true;
   return 0;
 }
 
-int
+#define NO_SLOW_PID_REUSE
+#ifndef NO_SLOW_PID_REUSE
+static void
+slow_pid_reuse (HANDLE h)
+{
+  static NO_COPY HANDLE last_fork_procs[NPIDS_HELD];
+  static NO_COPY unsigned nfork_procs;
+
+  if (nfork_procs >= (sizeof (last_fork_procs) / sizeof (last_fork_procs [0])))
+    nfork_procs = 0;
+  /* Keep a list of handles to child processes sitting around to prevent
+     Windows from reusing the same pid n times in a row.  Having the same pids
+     close in succesion confuses bash.  Keeping a handle open will stop
+     windows from reusing the same pid.  */
+  if (last_fork_procs[nfork_procs])
+    ForceCloseHandle1 (last_fork_procs[nfork_procs], fork_stupidity);
+  if (DuplicateHandle (GetCurrentProcess (), h,
+		       GetCurrentProcess (), &last_fork_procs[nfork_procs],
+		       0, FALSE, DUPLICATE_SAME_ACCESS))
+    ProtectHandle1 (last_fork_procs[nfork_procs], fork_stupidity);
+  else
+    {
+      last_fork_procs[nfork_procs] = NULL;
+      system_printf ("couldn't create last_fork_proc, %E");
+    }
+  nfork_procs++;
+}
+#endif
+
+int __stdcall
 frok::parent (volatile char * volatile stack_here)
 {
   HANDLE forker_finished;
@@ -259,6 +300,7 @@ frok::parent (volatile char * volatile stack_here)
 
   ch.forker_finished = forker_finished;
 
+  ch.from_main = &_my_tls == _main_tls;
   ch.stackbase = NtCurrentTeb ()->Tib.StackBase;
   ch.stackaddr = NtCurrentTeb ()->DeallocationStack;
   if (!ch.stackaddr)
@@ -279,7 +321,7 @@ frok::parent (volatile char * volatile stack_here)
 	 on whether we're running on a pthread or not.  If pthread, we fetch
 	 the guardpage size from the pthread attribs, otherwise we use the
 	 system default. */
-      ch.stacklimit = (void *) ((uintptr_t) stack_here & ~(wincap.page_size () - 1));
+      ch.stacklimit = (void *) ((uintptr_t) stack_here & ~wincap.page_size ());
       ch.guardsize = (&_my_tls != _main_tls && _my_tls.tid)
 		     ? _my_tls.tid->attr.guardsize
 		     : wincap.def_guard_page_size ();
@@ -296,10 +338,9 @@ frok::parent (volatile char * volatile stack_here)
   si.lpReserved2 = (LPBYTE) &ch;
   si.cbReserved2 = sizeof (ch);
 
-  /* NEVER, EVER, call a function which in turn calls malloc&friends while this
-     malloc lock is active! */
-  __malloc_lock ();
-  bool locked = true;
+  syscall_printf ("CreateProcessW (%W, %W, 0, 0, 1, %y, 0, 0, %p, %p)",
+		  myself->progname, myself->progname, c_flags, &si, &pi);
+  bool locked = __malloc_lock ();
 
   /* Remove impersonation */
   cygheap->user.deimpersonate ();
@@ -307,43 +348,21 @@ frok::parent (volatile char * volatile stack_here)
   ch.refresh_cygheap ();
   ch.prefork ();	/* set up process tracking pipes. */
 
-  *with_forkables = dlls.setup_forkables (*with_forkables);
-
-  ch.silentfail (!*with_forkables); /* fail silently without forkables */
-
-  PSECURITY_ATTRIBUTES sa = (PSECURITY_ATTRIBUTES) alloca (1024);
-  if (!sec_user_nih (sa, cygheap->user.saved_sid (),
-		     well_known_authenticated_users_sid,
-		     PROCESS_QUERY_LIMITED_INFORMATION))
-    sa = &sec_none_nih;
-
   while (1)
     {
-      PCWCHAR forking_progname = NULL;
-      if (dlls.main_executable)
-        forking_progname = dll_list::buffered_shortname
-			   (dlls.main_executable->forkedntname ());
-      if (!forking_progname || !*forking_progname)
-	forking_progname = myself->progname;
-
-      syscall_printf ("CreateProcessW (%W, %W, 0, 0, 1, %y, 0, 0, %p, %p)",
-		      forking_progname, myself->progname, c_flags, &si, &pi);
-
       hchild = NULL;
-      /* cygwin1.dll may reuse the forking_progname buffer, even
-	 in case of failure: don't reuse forking_progname later */
-      rc = CreateProcessW (forking_progname,	/* image to run */
+      rc = CreateProcessW (myself->progname,	/* image to run */
 			   GetCommandLineW (),	/* Take same space for command
 						   line as in parent to make
 						   sure child stack is allocated
 						   in the same memory location
 						   as in parent. */
-			   sa,
-			   sa,
-			   TRUE,		/* inherit handles */
+			   &sec_none_nih,
+			   &sec_none_nih,
+			   TRUE,		/* inherit handles from parent */
 			   c_flags,
-			   NULL,		/* environ filled in later */
-			   0,			/* use cwd */
+			   NULL,		/* environment filled in later */
+			   0,	  		/* use current drive/directory */
 			   &si,
 			   &pi);
 
@@ -353,7 +372,6 @@ frok::parent (volatile char * volatile stack_here)
 	{
 	  this_errno = geterrno_from_win_error ();
 	  error ("CreateProcessW failed for '%W'", myself->progname);
-	  dlls.release_forkables ();
 	  memset (&pi, 0, sizeof (pi));
 	  goto cleanup;
 	}
@@ -367,8 +385,6 @@ frok::parent (volatile char * volatile stack_here)
       CloseHandle (pi.hThread);
       hchild = pi.hProcess;
 
-      dlls.release_forkables ();
-
       /* Protect the handle but name it similarly to the way it will
 	 be called in subproc handling. */
       ProtectHandle1 (hchild, childhProc);
@@ -417,8 +433,9 @@ frok::parent (volatile char * volatile stack_here)
      it in afterwards.  This requires more bookkeeping than I like, though,
      so we'll just do it the easy way.  So, terminate any child process if
      we can't actually record the pid in the internal table. */
-  if (!child.remember ())
+  if (!child.remember (false))
     {
+      TerminateProcess (hchild, 1);
       this_errno = EAGAIN;
 #ifdef DEBUGGING0
       error ("child remember failed");
@@ -426,6 +443,10 @@ frok::parent (volatile char * volatile stack_here)
       goto cleanup;
     }
 
+#ifndef NO_SLOW_PID_REUSE
+  slow_pid_reuse (hchild);
+#endif
+
   /* CHILD IS STOPPED */
   debug_printf ("child is alive (but stopped)");
 
@@ -446,7 +467,7 @@ frok::parent (volatile char * volatile stack_here)
       impure_beg = _impure_ptr;
       impure_end = _impure_ptr + 1;
     }
-  rc = child_copy (hchild, true, !*with_forkables,
+  rc = child_copy (hchild, true,
 		   "stack", stack_here, ch.stackbase,
 		   impure, impure_beg, impure_end,
 		   NULL);
@@ -464,7 +485,7 @@ frok::parent (volatile char * volatile stack_here)
   for (dll *d = dlls.istart (DLL_LINK); d; d = dlls.inext ())
     {
       debug_printf ("copying data/bss of a linked dll");
-      if (!child_copy (hchild, true, !*with_forkables,
+      if (!child_copy (hchild, true,
 		       "linked dll data", d->p.data_start, d->p.data_end,
 		       "linked dll bss", d->p.bss_start, d->p.bss_end,
 		       NULL))
@@ -475,8 +496,7 @@ frok::parent (volatile char * volatile stack_here)
 	}
     }
 
-  /* Start the child up, and then wait for it to
-     perform fork fixups and dynamic dll loading (if any). */
+  /* Start thread, and then wait for it to reload dlls.  */
   resume_child (forker_finished);
   if (!ch.sync (child->pid, hchild, FORK_WAIT_TIMEOUT))
     {
@@ -495,7 +515,7 @@ frok::parent (volatile char * volatile stack_here)
       for (dll *d = dlls.istart (DLL_LOAD); d; d = dlls.inext ())
 	{
 	  debug_printf ("copying data/bss for a loaded dll");
-	  if (!child_copy (hchild, true, !*with_forkables,
+	  if (!child_copy (hchild, true,
 			   "loaded dll data", d->p.data_start, d->p.data_end,
 			   "loaded dll bss", d->p.bss_start, d->p.bss_end,
 			   NULL))
@@ -507,22 +527,10 @@ frok::parent (volatile char * volatile stack_here)
 	      goto cleanup;
 	    }
 	}
+      /* Start the child up again. */
+      resume_child (forker_finished);
     }
 
-  /* Do not attach to the child before it has successfully initialized.
-     Otherwise we may wait forever, or deliver an orphan SIGCHILD. */
-  if (!child.attach ())
-    {
-      this_errno = EAGAIN;
-#ifdef DEBUGGING0
-      error ("child attach failed");
-#endif
-      goto cleanup;
-    }
-
-  /* Finally start the child up. */
-  resume_child (forker_finished);
-
   ForceCloseHandle (forker_finished);
   forker_finished = NULL;
 
@@ -530,21 +538,14 @@ frok::parent (volatile char * volatile stack_here)
 
 /* Common cleanup code for failure cases */
 cleanup:
-  /* release procinfo before hProcess in destructor */
-  child.allow_remove ();
-
   if (fix_impersonation)
     cygheap->user.reimpersonate ();
   if (locked)
     __malloc_unlock ();
 
   /* Remember to de-allocate the fd table. */
-  if (hchild)
-    {
-      TerminateProcess (hchild, 1);
-      if (!child.hProcess) /* no child.procinfo */
-	ForceCloseHandle1 (hchild, childhProc);
-    }
+  if (hchild && !child.hProcess)
+    ForceCloseHandle1 (hchild, childhProc);
   if (forker_finished)
     ForceCloseHandle (forker_finished);
   debug_printf ("returning -1");
@@ -554,39 +555,7 @@ cleanup:
 extern "C" int
 fork ()
 {
-  bool with_forkables = false; /* do not force hardlinks on first try */
-  int res = dofork (NULL, &with_forkables);
-  if (res >= 0)
-    return res;
-  if (with_forkables)
-    return res; /* no need for second try when already enabled */
-  with_forkables = true; /* enable hardlinks for second try */
-  return dofork (NULL, &with_forkables);
-}
-
-
-/* __posix_spawn_fork is called from newlib's posix_spawn implementation.
-   The original code in newlib has been taken from FreeBSD, and the core
-   code relies on specific, non-portable behaviour of vfork(2).  Our
-   replacement implementation needs the forked child's HANDLE for
-   synchronization, so __posix_spawn_fork returns it in proc. */
-extern "C" int
-__posix_spawn_fork (void **proc)
-{
-  bool with_forkables = false; /* do not force hardlinks on first try */
-  int res = dofork (proc, &with_forkables);
-  if (res >= 0)
-    return res;
-  if (with_forkables)
-    return res; /* no need for second try when already enabled */
-  with_forkables = true; /* enable hardlinks for second try */
-  return dofork (proc, &with_forkables);
-}
-
-static int
-dofork (void **proc, bool *with_forkables)
-{
-  frok grouped (with_forkables);
+  frok grouped;
 
   debug_printf ("entering");
   grouped.load_dlls = 0;
@@ -631,7 +600,7 @@ dofork (void **proc, bool *with_forkables)
 #ifdef __x86_64__
     __asm__ volatile ("movq %%rsp,%0": "=r" (stackp));
 #else
-#error unimplemented for this target
+    __asm__ volatile ("movl %%esp,%0": "=r" (stackp));
 #endif
 
     if (!ischild)
@@ -639,7 +608,7 @@ dofork (void **proc, bool *with_forkables)
     else
       {
 	res = grouped.child (stackp);
-	__in_forkee = FORKED;
+	in_forkee = false;
 	ischild = true;	/* might have been reset by fork mem copy */
       }
   }
@@ -653,20 +622,17 @@ dofork (void **proc, bool *with_forkables)
     {
       if (!grouped.errmsg)
 	syscall_printf ("fork failed - child pid %d, errno %d", grouped.child_pid, grouped.this_errno);
-      else if (grouped.ch.silentfail ())
-	debug_printf ("child %d - %s, errno %d", grouped.child_pid,
-		       grouped.errmsg, grouped.this_errno);
       else
-	system_printf ("child %d - %s, errno %d", grouped.child_pid,
-		       grouped.errmsg, grouped.this_errno);
+	{
+	  char buf[strlen (grouped.errmsg) + sizeof ("child %d - , errno 4294967295  ")];
+	  strcpy (buf, "child %d - ");
+	  strcat (buf, grouped.errmsg);
+	  strcat (buf, ", errno %d");
+	  system_printf (buf, grouped.child_pid, grouped.this_errno);
+	}
 
       set_errno (grouped.this_errno);
     }
-  else if (proc)
-    {
-      /* Return child process handle to posix_fork. */
-      *proc = grouped.hchild;
-    }
   syscall_printf ("%R = fork()", res);
   return res;
 }
@@ -688,10 +654,10 @@ vfork ()
 /* Copy memory from one process to another. */
 
 bool
-child_copy (HANDLE hp, bool write, bool silentfail, ...)
+child_copy (HANDLE hp, bool write, ...)
 {
   va_list args;
-  va_start (args, silentfail);
+  va_start (args, write);
   static const char *huh[] = {"read", "write"};
 
   char *what;
@@ -717,14 +683,10 @@ child_copy (HANDLE hp, bool write, bool silentfail, ...)
 	    {
 	      if (!res)
 		__seterrno ();
-	      if (silentfail)
-		debug_printf ("%s %s copy failed, %p..%p, done %lu, windows pid %u, %E",
-			     what, huh[write], low, high, done, myself->dwProcessId);
-	      else
-		/* If this happens then there is a bug in our fork
-		   implementation somewhere. */
-		system_printf ("%s %s copy failed, %p..%p, done %lu, windows pid %u, %E",
-			      what, huh[write], low, high, done, myself->dwProcessId);
+	      /* If this happens then there is a bug in our fork
+		 implementation somewhere. */
+	      system_printf ("%s %s copy failed, %p..%p, done %lu, windows pid %u, %E",
+			    what, huh[write], low, high, done, myself->dwProcessId);
 	      goto err;
 	    }
 	}
