@@ -153,13 +153,13 @@ internal_getgrfull (fetch_acc_t &full_acc, cyg_ldap *pldap)
      internal_getgrsid_cachedonly. */
   if (cygheap->pg.nss_cygserver_caching ()
       && (ret = cygheap->pg.grp_cache.cygserver.add_group_from_cygserver
-							(full_acc.sid)))
+      							(full_acc.sid)))
     return ret;
   if (cygheap->pg.nss_grp_files ())
     {
       cygheap->pg.grp_cache.file.check_file ();
       if ((ret = cygheap->pg.grp_cache.file.add_group_from_file
-							(full_acc.sid)))
+      							(full_acc.sid)))
 	return ret;
     }
   if (cygheap->pg.nss_grp_db ())
@@ -237,7 +237,7 @@ internal_getgrgid (gid_t gid, cyg_ldap *pldap)
   return NULL;
 }
 
-#ifdef __i386__
+#ifndef __x86_64__
 static struct __group16 *
 grp32togrp16 (struct __group16 *gp16, struct group *gp32)
 {
@@ -612,7 +612,7 @@ internal_getgroups (int gidsetsize, gid_t *grouplist, cyg_ldap *pldap)
 	      goto out;
 	    }
 	}
-      else
+      else 
 	sidp_buf[scnt++] = sid;
     }
   /* If there are non-cached groups left, try to fetch them. */
@@ -738,7 +738,7 @@ get_groups (const char *user, gid_t gid, cygsidlist &gsids)
   struct group *grp = internal_getgrgid (gid, &cldap);
   cygsid usersid, grpsid;
   if (usersid.getfrompw (pw))
-    get_server_groups (gsids, usersid, NO_CHK_DISABLED);
+    get_server_groups (gsids, usersid);
   if (gid != ILLEGAL_GID && grpsid.getfromgr (grp))
     gsids += grpsid;
   cygheap->user.reimpersonate ();
@@ -837,7 +837,9 @@ setgroups32 (int ngroups, const gid_t *grouplist)
   return 0;
 }
 
-#ifdef __i386__
+#ifdef __x86_64__
+EXPORT_ALIAS (setgroups32, setgroups)
+#else
 extern "C" int
 setgroups (int ngroups, const __gid16_t *grouplist)
 {
@@ -853,6 +855,4 @@ setgroups (int ngroups, const __gid16_t *grouplist)
     }
   return setgroups32 (ngroups, grouplist32);
 }
-#else
-EXPORT_ALIAS (setgroups32, setgroups)
 #endif
