@@ -20,7 +20,7 @@ extern "C" {
 
 #define _CYGWIN_SIGNAL_STRING "cYgSiGw00f"
 
-#ifdef __i386__
+#ifndef __x86_64__
 /* DEPRECATED INTERFACES.  These are restricted to MAX_PATH length.
    Don't use in modern applications.  They don't exist on x86_64. */
 extern int cygwin_win32_to_posix_path_list (const char *, char *)
@@ -39,7 +39,7 @@ extern int cygwin_conv_to_posix_path (const char *, char *)
   __attribute__ ((__deprecated__));
 extern int cygwin_conv_to_full_posix_path (const char *, char *)
   __attribute__ ((__deprecated__));
-#endif /* __i386__ */
+#endif /* !__x86_64__ */
 
 /* Use these interfaces in favor of the above. */
 
@@ -54,9 +54,9 @@ enum
   CCP_CONVTYPE_MASK = 3,
 
   /* Or these values to the above as needed. */
-  CCP_ABSOLUTE = 0,		/* Request absolute path (default).	*/
-  CCP_RELATIVE = 0x100,		/* Request to keep path relative.	*/
-  CCP_PROC_CYGDRIVE = 0x200,	/* Request to return /proc/cygdrive
+  CCP_ABSOLUTE = 0,	  	/* Request absolute path (default). 	*/
+  CCP_RELATIVE = 0x100,    	/* Request to keep path relative.   	*/
+  CCP_PROC_CYGDRIVE = 0x200,   	/* Request to return /proc/cygdrive
 				   path (only with CCP_*_TO_POSIX).   */
 
   CCP_CONVFLAGS_MASK = 0x300,
@@ -158,8 +158,6 @@ typedef enum
     CW_GETNSS_GRP_SRC,
     CW_EXCEPTION_RECORD_FROM_SIGINFO_T,
     CW_CYGHEAP_PROFTHR_ALL,
-    CW_WINPID_TO_CYGWIN_PID,
-    CW_MAX_CYGWIN_PID,
   } cygwin_getinfo_types;
 
 #define CW_LOCK_PINFO CW_LOCK_PINFO
@@ -222,8 +220,6 @@ typedef enum
 #define CW_GETNSS_GRP_SRC CW_GETNSS_GRP_SRC
 #define CW_EXCEPTION_RECORD_FROM_SIGINFO_T CW_EXCEPTION_RECORD_FROM_SIGINFO_T
 #define CW_CYGHEAP_PROFTHR_ALL CW_CYGHEAP_PROFTHR_ALL
-#define CW_WINPID_TO_CYGWIN_PID CW_WINPID_TO_CYGWIN_PID
-#define CW_MAX_CYGWIN_PID CW_MAX_CYGWIN_PID
 
 /* Token type for CW_SET_EXTERNAL_TOKEN */
 enum
@@ -273,9 +269,8 @@ enum
   PID_INITIALIZING     = 0x00800, /* Set until ready to receive signals. */
   PID_NEW	       = 0x01000, /* Available. */
   PID_ALLPIDS	       = 0x02000, /* used by pinfo scanner */
-  PID_PROCINFO	       = 0x08000, /* caller just asks for process info */
-  PID_NEW_PG	       = 0x10000, /* Process created with
-				     CREATE_NEW_PROCESS_GROUOP flag */
+  PID_EXECED	       = 0x04000, /* redirect to original pid info block */
+  PID_NOREDIR	       = 0x08000, /* don't redirect if execed */
   PID_EXITED	       = 0x40000000, /* Free entry. */
   PID_REAPED	       = 0x80000000  /* Reaped */
 };
@@ -315,7 +310,7 @@ struct per_process
   uint32_t dll_minor;
 
   struct _reent **impure_ptr_ptr;
-#ifdef __i386__
+#ifndef __x86_64__
   char ***envptr;
 #endif
 
