@@ -26,9 +26,7 @@ enum picom
   PICOM_ROOT = 3,
   PICOM_FDS = 4,
   PICOM_FD = 5,
-  PICOM_PIPE_FHANDLER = 6,
-  PICOM_FILE_PATHCONV = 7,
-  PICOM_ENVIRON = 8
+  PICOM_PIPE_FHANDLER = 6
 };
 
 #define EXITCODE_SET		0x8000000
@@ -53,6 +51,8 @@ public:
 
   DWORD exitcode;	/* set when process exits */
 
+#define PINFO_REDIR_SIZE ((char *) &myself.procinfo->exitcode - (char *) myself.procinfo)
+
   /* > 0 if started by a cygwin process */
   DWORD cygstarted;
 
@@ -101,13 +101,11 @@ public:
   commune_result commune_request (__uint32_t, ...);
   bool alive ();
   fhandler_pipe *pipe_fhandler (int64_t, size_t &);
-  void *file_pathconv (int, uint32_t, size_t &);
   char *fd (int fd, size_t &);
   char *fds (size_t &);
   char *root (size_t &);
   char *cwd (size_t &);
   char *cmdline (size_t &);
-  char *environ (size_t &);
   char *win_heap_info (size_t &);
   bool set_ctty (class fhandler_termios *, int);
   bool alert_parent (char);
@@ -139,32 +137,28 @@ public:
   HANDLE rd_proc_pipe;
   pinfo_minimal (): h (NULL), hProcess (NULL), rd_proc_pipe (NULL) {}
   void set_rd_proc_pipe (HANDLE& h) {rd_proc_pipe = h;}
-  void set_inheritance (bool);
   friend class pinfo;
 };
 
 class pinfo: public pinfo_minimal
 {
   bool destroy;
-  HANDLE winpid_hdl;
   _pinfo *procinfo;
 public:
   bool waiter_ready;
   class cygthread *wait_thread;
 
   void __reg3 init (pid_t, DWORD, HANDLE);
-  pinfo (_pinfo *x = NULL)
-  : pinfo_minimal (), destroy (false), winpid_hdl (NULL), procinfo (x),
-    waiter_ready (false), wait_thread (NULL) {}
-  pinfo (pid_t n, DWORD flag = 0)
-  : pinfo_minimal (), destroy (false), winpid_hdl (NULL), procinfo (NULL),
-    waiter_ready (false), wait_thread (NULL)
+  pinfo (_pinfo *x = NULL): pinfo_minimal (), destroy (false), procinfo (x),
+		     waiter_ready (false), wait_thread (NULL) {}
+  pinfo (pid_t n, DWORD flag = 0): pinfo_minimal (), destroy (false),
+				   procinfo (NULL), waiter_ready (false),
+				   wait_thread (NULL)
   {
     init (n, flag, NULL);
   }
   pinfo (HANDLE, pinfo_minimal&, pid_t);
   void __reg2 thisproc (HANDLE);
-  void create_winpid_symlink ();
   inline void _pinfo_release ();
   void release ();
   bool __reg1 wait ();
@@ -187,24 +181,24 @@ public:
   void preserve () { destroy = false; }
   void allow_remove () { destroy = true; }
 #ifndef SIG_BAD_MASK		// kludge to ensure that sigproc.h included
-  // int attach () {system_printf ("attach is not here"); return 0;}
+  // int reattach () {system_printf ("reattach is not here"); return 0;}
   // int remember (bool) {system_printf ("remember is not here"); return 0;}
 #else
-  int attach ()
+  int reattach ()
   {
-    int res = proc_subproc (PROC_ATTACH_CHILD, (uintptr_t) this);
+    int res = proc_subproc (PROC_REATTACH_CHILD, (uintptr_t) this);
     destroy = res ? false : true;
     return res;
   }
-  int remember ()
+  int remember (bool detach)
   {
-    int res = proc_subproc (PROC_ADD_CHILD, (uintptr_t) this);
+    int res = proc_subproc (detach ? PROC_DETACHED_CHILD : PROC_ADDCHILD,
+			    (uintptr_t) this);
     destroy = res ? false : true;
     return res;
   }
 #endif
   HANDLE shared_handle () {return h;}
-  HANDLE shared_winpid_handle () {return winpid_hdl;}
   void set_acl ();
   friend class _pinfo;
   friend class winpids;
@@ -212,8 +206,6 @@ private:
   DWORD status_exit (DWORD);
 };
 
-#define MAX_PID 65536
-
 #define ISSTATE(p, f)	(!!((p)->process_state & f))
 #define NOTSTATE(p, f)	(!((p)->process_state & f))
 
@@ -232,6 +224,8 @@ public:
   inline void reset () { release (); npids = 0;}
   void set (bool winpid);
   winpids (): make_copy (true) {}
+  winpids (int): make_copy (false), npidlist (0), pidlist (NULL),
+		 pinfolist (NULL), pinfo_access (0), npids (0) {}
   winpids (DWORD acc): make_copy (false), npidlist (0), pidlist (NULL),
 		       pinfolist (NULL), pinfo_access (acc), npids (0)
   {
@@ -243,8 +237,11 @@ public:
   void release ();
 };
 
-pid_t create_cygwin_pid ();
-pid_t cygwin_pid (DWORD);
+extern __inline pid_t
+cygwin_pid (pid_t pid)
+{
+  return pid;
+}
 
 void __stdcall pinfo_init (char **, int);
 extern pinfo myself;
