@@ -130,7 +130,7 @@ cygpsid::get_id (BOOL search_grp, int *type, cyg_ldap *pldap)
 		  if (pldap->open (cygheap->dom.get_rfc2307_domain ())
 		      == NO_ERROR)
 		    map_gid = pldap->remap_gid (gid);
-		  if (map_gid == ILLEGAL_GID)
+		  if (map_gid == ILLEGAL_GID) 
 		    map_gid = MAP_UNIX_TO_CYGWIN_ID (gid);
 		  cygheap->ugid_cache.add_gid (gid, map_gid);
 		}
@@ -169,16 +169,10 @@ cygpsid::get_id (BOOL search_grp, int *type, cyg_ldap *pldap)
 	}
       else if ((pw = internal_getpwsid (*this, pldap)))
 	id = pw->pw_uid;
-      if (id != ILLEGAL_UID)
-	{
-	  if (type)
-	    *type = USER;
-	  return id;
-	}
+      if (id != ILLEGAL_UID && type)
+	*type = USER;
     }
-  if (type)
-    *type = 0; /* undefined type */
-  return ILLEGAL_UID;
+  return id;
 }
 
 PWCHAR
@@ -827,16 +821,12 @@ authz_ctx::get_user_attribute (mode_t *attribute, PSECURITY_DESCRIPTOR psd,
   if (RtlEqualSid (user_sid, cygheap->user.sid ())
       && !cygheap->user.issetuid ())
     {
-      /* Avoid lock in default case. */
       if (!user_ctx_hdl)
 	{
 	  authz_guard.acquire ();
-	  /* Check user_ctx_hdl again under lock to avoid overwriting
-	     user_ctx_hdl if it has already been initialized. */
-	  if (!user_ctx_hdl
-	      && !AuthzInitializeContextFromToken (0, hProcToken, authz, NULL,
-						   authz_dummy_luid, NULL,
-						   &user_ctx_hdl))
+	  if (!AuthzInitializeContextFromToken (0, hProcToken, authz, NULL,
+						authz_dummy_luid, NULL,
+						&user_ctx_hdl))
 	    debug_printf ("AuthzInitializeContextFromToken, %E");
 	  authz_guard.release ();
 	}
