@@ -11,10 +11,7 @@ details. */
 
 #include "winsup.h"
 #include <stdlib.h>
-#include <unistd.h>
 #include <sys/cygwin.h>
-#include <sys/signalfd.h>
-#include <sys/reent.h> /* needed for __stdio_exit_handler declaration */
 #include "pinfo.h"
 #include "sigproc.h"
 #include "cygtls.h"
@@ -23,11 +20,10 @@ details. */
 #include "dtable.h"
 #include "cygheap.h"
 #include "cygwait.h"
-#include "posix_timer.h"
 
 #define _SA_NORESTART	0x8000
 
-static int sigaction_worker (int, const struct sigaction *, struct sigaction *, bool);
+static int __reg3 sigaction_worker (int, const struct sigaction *, struct sigaction *, bool);
 
 #define sigtrapped(func) ((func) != SIG_IGN && (func) != SIG_DFL)
 
@@ -38,7 +34,7 @@ signal (int sig, _sig_func_ptr func)
   _sig_func_ptr prev;
 
   /* check that sig is in right range */
-  if (sig <= 0 || sig >= _NSIG || sig == SIGKILL || sig == SIGSTOP)
+  if (sig < 0 || sig >= NSIG || sig == SIGKILL || sig == SIGSTOP)
     {
       set_errno (EINVAL);
       syscall_printf ("SIG_ERR = signal (%d, %p)", sig, func);
@@ -69,16 +65,8 @@ clock_nanosleep (clockid_t clk_id, int flags, const struct timespec *rqtp,
   sig_dispatch_pending ();
   pthread_testcancel ();
 
-  __try
-    {
-      if (!valid_timespec (*rqtp))
-	return EINVAL;
-    }
-  __except (NO_ERROR)
-    {
-      return EFAULT;
-    }
-  __endtry
+  if (rqtp->tv_sec < 0 || rqtp->tv_nsec < 0 || rqtp->tv_nsec > 999999999L)
+    return EINVAL;
 
   /* Explicitly disallowed by POSIX. Needs to be checked first to avoid
      being caught by the following test. */
@@ -88,15 +76,21 @@ clock_nanosleep (clockid_t clk_id, int flags, const struct timespec *rqtp,
   /* support for CPU-time clocks is optional */
   if (CLOCKID_IS_PROCESS (clk_id) || CLOCKID_IS_THREAD (clk_id))
     return ENOTSUP;
-  /* All other valid clocks are valid */
-  if (clk_id >= MAX_CLOCKS)
-    return EINVAL;
+
+  switch (clk_id)
+    {
+    case CLOCK_REALTIME:
+    case CLOCK_MONOTONIC:
+      break;
+    default:
+      /* unknown or illegal clock ID */
+      return EINVAL;
+    }
 
   LARGE_INTEGER timeout;
 
-  timeout.QuadPart = (LONGLONG) rqtp->tv_sec * NS100PERSEC
-		     + ((LONGLONG) rqtp->tv_nsec + (NSPERSEC/NS100PERSEC) - 1)
-		       / (NSPERSEC/NS100PERSEC);
+  timeout.QuadPart = (LONGLONG) rqtp->tv_sec * NSPERSEC
+		     + ((LONGLONG) rqtp->tv_nsec + 99LL) / 100LL;
 
   if (abstime)
     {
@@ -108,18 +102,13 @@ clock_nanosleep (clockid_t clk_id, int flags, const struct timespec *rqtp,
 	  || (tp.tv_sec == rqtp->tv_sec && tp.tv_nsec > rqtp->tv_nsec))
 	return 0;
 
-      switch (clk_id)
+      if (clk_id == CLOCK_REALTIME)
+	timeout.QuadPart += FACTOR;
+      else
 	{
-	case CLOCK_REALTIME_COARSE:
-	case CLOCK_REALTIME:
-	  timeout.QuadPart += FACTOR;
-	  break;
-	default:
 	  /* other clocks need to be handled with a relative timeout */
-	  timeout.QuadPart -= tp.tv_sec * NS100PERSEC
-			      + tp.tv_nsec / (NSPERSEC/NS100PERSEC);
+	  timeout.QuadPart -= tp.tv_sec * NSPERSEC + tp.tv_nsec / 100LL;
 	  timeout.QuadPart *= -1LL;
-	  break;
 	}
     }
   else /* !abstime */
@@ -134,17 +123,8 @@ clock_nanosleep (clockid_t clk_id, int flags, const struct timespec *rqtp,
   /* according to POSIX, rmtp is used only if !abstime */
   if (rmtp && !abstime)
     {
-      __try
-	{
-	  rmtp->tv_sec = (time_t) (timeout.QuadPart / NS100PERSEC);
-	  rmtp->tv_nsec = (long) ((timeout.QuadPart % NS100PERSEC)
-				  * (NSPERSEC/NS100PERSEC));
-	}
-      __except (NO_ERROR)
-	{
-	  res = EFAULT;
-	}
-      __endtry
+      rmtp->tv_sec = (time_t) (timeout.QuadPart / NSPERSEC);
+      rmtp->tv_nsec = (long) ((timeout.QuadPart % NSPERSEC) * 100LL);
     }
 
   syscall_printf ("%d = clock_nanosleep(%lu, %d, %ld.%09ld, %ld.%09.ld)",
@@ -176,12 +156,12 @@ sleep (unsigned int seconds)
   return 0;
 }
 
-extern "C" int
+extern "C" unsigned int
 usleep (useconds_t useconds)
 {
   struct timespec req;
-  req.tv_sec = useconds / USPERSEC;
-  req.tv_nsec = (useconds % USPERSEC) * (NSPERSEC/USPERSEC);
+  req.tv_sec = useconds / 1000000;
+  req.tv_nsec = (useconds % 1000000) * 1000;
   int res = clock_nanosleep (CLOCK_REALTIME, 0, &req, NULL);
   if (res != 0)
     {
@@ -204,18 +184,18 @@ sigprocmask (int how, const sigset_t *set, sigset_t *oldset)
   return res;
 }
 
-int
+int __reg3
 handle_sigprocmask (int how, const sigset_t *set, sigset_t *oldset, sigset_t& opmask)
 {
-  /* check that how is in right range if set is not NULL */
-  if (set && how != SIG_BLOCK && how != SIG_UNBLOCK && how != SIG_SETMASK)
+  /* check that how is in right range */
+  if (how != SIG_BLOCK && how != SIG_UNBLOCK && how != SIG_SETMASK)
     {
       syscall_printf ("Invalid how value %d", how);
       return EINVAL;
     }
 
   __try
-    {
+	{
       if (oldset)
 	*oldset = opmask;
 
@@ -248,7 +228,7 @@ handle_sigprocmask (int how, const sigset_t *set, sigset_t *oldset, sigset_t& op
   return 0;
 }
 
-int
+int __reg2
 _pinfo::kill (siginfo_t& si)
 {
   int res;
@@ -266,7 +246,7 @@ _pinfo::kill (siginfo_t& si)
 
       if (si.si_signo == 0)
 	res = 0;
-      else if ((res = (int) sig_send (this, si)))
+      else if ((res = sig_send (this, si)))
 	{
 	  sigproc_printf ("%d = sig_send, %E ", res);
 	  res = -1;
@@ -280,7 +260,7 @@ _pinfo::kill (siginfo_t& si)
 	}
       this_pid = pid;
     }
-  else if (process_state == PID_EXITED)
+  else if (this && process_state == PID_EXITED)
     {
       this_process_state = process_state;
       this_pid = pid;
@@ -299,18 +279,10 @@ _pinfo::kill (siginfo_t& si)
   return res;
 }
 
-extern "C" int
+int
 raise (int sig)
 {
-  pthread *thread = _my_tls.tid;
-  if (!thread || !__isthreaded)
-    return kill (myself->pid, sig);
-
-  /* Make sure to return -1 and set errno, as on Linux. */
-  int err = pthread_kill (thread, sig);
-  if (err)
-    set_errno (err);
-  return err ? -1 : 0;
+  return kill (myself->pid, sig);
 }
 
 static int
@@ -318,23 +290,14 @@ kill0 (pid_t pid, siginfo_t& si)
 {
   syscall_printf ("kill (%d, %d)", pid, si.si_signo);
   /* check that sig is in right range */
-  if (si.si_signo < 0 || si.si_signo >= _NSIG)
+  if (si.si_signo < 0 || si.si_signo >= NSIG)
     {
       set_errno (EINVAL);
       syscall_printf ("signal %d out of range", si.si_signo);
       return -1;
     }
-  if (pid > 0)
-    {
-      pinfo p (pid);
-      if (!p)
-	{
-	  set_errno (ESRCH);
-	  return -1;
-	}
-      return p->kill (si);
-    }
-  return kill_pgrp (-pid, si);
+
+  return (pid > 0) ? pinfo (pid)->kill (si) : kill_pgrp (-pid, si);
 }
 
 int
@@ -360,7 +323,7 @@ kill_pgrp (pid_t pid, siginfo_t& si)
     {
       _pinfo *p = pids[i];
 
-      if (!p || !p->exists ())
+      if (!p->exists ())
 	continue;
 
       /* Is it a process we want to kill?  */
@@ -410,12 +373,12 @@ abort (void)
   _my_tls.call_signal_handler (); /* Call any signal handler */
 
   /* Flush all streams as per SUSv2.  */
-  if (__stdio_exit_handler)
-    (*__stdio_exit_handler) ();
+  if (_GLOBAL_REENT->__cleanup)
+    _GLOBAL_REENT->__cleanup (_GLOBAL_REENT);
   do_exit (SIGABRT);	/* signal handler didn't exit.  Goodbye. */
 }
 
-static int
+static int __reg3
 sigaction_worker (int sig, const struct sigaction *newact,
 		  struct sigaction *oldact, bool isinternal)
 {
@@ -424,7 +387,7 @@ sigaction_worker (int sig, const struct sigaction *newact,
     {
       sig_dispatch_pending ();
       /* check that sig is in right range */
-      if (sig <= 0 || sig >= _NSIG)
+      if (sig < 0 || sig >= NSIG)
 	set_errno (EINVAL);
       else
 	{
@@ -451,9 +414,9 @@ sigaction_worker (int sig, const struct sigaction *newact,
 	      if (!(gs.sa_flags & SA_NODEFER))
 		gs.sa_mask |= SIGTOMASK(sig);
 	      if (gs.sa_handler == SIG_IGN)
-		sig_clear (sig, true);
+		sig_clear (sig);
 	      if (gs.sa_handler == SIG_DFL && sig == SIGCHLD)
-		sig_clear (sig, true);
+		sig_clear (sig);
 	      if (sig == SIGCHLD)
 		{
 		  myself->process_state &= ~PID_NOCLDSTOP;
@@ -487,7 +450,7 @@ extern "C" int
 sigaddset (sigset_t *set, const int sig)
 {
   /* check that sig is in right range */
-  if (sig <= 0 || sig >= _NSIG)
+  if (sig <= 0 || sig >= NSIG)
     {
       set_errno (EINVAL);
       syscall_printf ("SIG_ERR = sigaddset signal %d out of range", sig);
@@ -502,7 +465,7 @@ extern "C" int
 sigdelset (sigset_t *set, const int sig)
 {
   /* check that sig is in right range */
-  if (sig <= 0 || sig >= _NSIG)
+  if (sig <= 0 || sig >= NSIG)
     {
       set_errno (EINVAL);
       syscall_printf ("SIG_ERR = sigdelset signal %d out of range", sig);
@@ -517,7 +480,7 @@ extern "C" int
 sigismember (const sigset_t *set, int sig)
 {
   /* check that sig is in right range */
-  if (sig <= 0 || sig >= _NSIG)
+  if (sig <= 0 || sig >= NSIG)
     {
       set_errno (EINVAL);
       syscall_printf ("SIG_ERR = sigdelset signal %d out of range", sig);
@@ -560,18 +523,6 @@ sigpause (int signal_mask)
   return res;
 }
 
-extern "C" int
-__xpg_sigpause (int sig)
-{
-  int res;
-  sigset_t signal_mask;
-  sigprocmask (0, NULL, &signal_mask);
-  sigdelset (&signal_mask, sig);
-  res = handle_sigsuspend (signal_mask);
-  syscall_printf ("%R = __xpg_sigpause(%y)", res, sig);
-  return res;
-}
-
 extern "C" int
 pause (void)
 {
@@ -584,113 +535,22 @@ extern "C" int
 siginterrupt (int sig, int flag)
 {
   struct sigaction act;
-  int res = sigaction_worker (sig, NULL, &act, false);
-  if (res == 0)
+  sigaction (sig, NULL, &act);
+  if (flag)
     {
-      if (flag)
-	{
-	  act.sa_flags &= ~SA_RESTART;
-	  act.sa_flags |= _SA_NORESTART;
-	}
-      else
-	{
-	  act.sa_flags &= ~_SA_NORESTART;
-	  act.sa_flags |= SA_RESTART;
-	}
-      res = sigaction_worker (sig, &act, NULL, true);
+      act.sa_flags &= ~SA_RESTART;
+      act.sa_flags |= _SA_NORESTART;
     }
+  else
+    {
+      act.sa_flags &= ~_SA_NORESTART;
+      act.sa_flags |= SA_RESTART;
+    }
+  int res = sigaction_worker (sig, &act, NULL, true);
   syscall_printf ("%R = siginterrupt(%d, %y)", sig, flag);
   return res;
 }
 
-int
-sigwait_common (const sigset_t *set, siginfo_t *info, PLARGE_INTEGER waittime)
-{
-  int res = -1;
-
-  pthread_testcancel ();
-
-  __try
-    {
-      set_signal_mask (_my_tls.sigwait_mask, *set);
-      sig_dispatch_pending (true);
-
-do_wait:
-      switch (cygwait (NULL, waittime,
-		       cw_sig_eintr | cw_cancel | cw_cancel_self))
-	{
-	case WAIT_SIGNALED:
-	  if (!sigismember (set, _my_tls.infodata.si_signo))
-	    set_errno (EINTR);
-	  else
-	    {
-	      _my_tls.lock ();
-	      if (_my_tls.infodata.si_code == SI_TIMER)
-		{
-		  timer_tracker *tt = (timer_tracker *)
-				      _my_tls.infodata.si_tid;
-		  _my_tls.infodata.si_overrun = tt->disarm_overrun_event ();
-		}
-	      if (info)
-		*info = _my_tls.infodata;
-	      res = _my_tls.infodata.si_signo;
-	      _my_tls.sig = 0;
-	      if (_my_tls.retaddr () == (__tlsstack_t) sigdelayed)
-		_my_tls.pop ();
-	      _my_tls.unlock ();
-	    }
-	  break;
-	case WAIT_TIMEOUT:
-	  _my_tls.lock ();
-	  if (_my_tls.sigwait_mask == 0)
-	    {
-	      /* sigpacket::process() already started.
-	         Will surely be signalled soon. */
-	      waittime = cw_infinite;
-	      _my_tls.unlock ();
-	      goto do_wait;
-	    }
-	  _my_tls.sigwait_mask = 0;
-	  _my_tls.unlock ();
-	  set_errno (EAGAIN);
-	  break;
-	default:
-	  __seterrno ();
-	  break;
-	}
-    }
-  __except (EFAULT)
-    {
-      res = -1;
-    }
-  __endtry
-  sigproc_printf ("returning signal %d", res);
-  return res;
-}
-
-extern "C" int
-sigtimedwait (const sigset_t *set, siginfo_t *info, const timespec *timeout)
-{
-  LARGE_INTEGER waittime;
-
-  if (timeout)
-    {
-      if (!valid_timespec (*timeout))
-	{
-	  set_errno (EINVAL);
-	  return -1;
-	}
-      /* convert timespec to 100ns units */
-      waittime.QuadPart = (LONGLONG) timeout->tv_sec * NS100PERSEC
-                          + ((LONGLONG) timeout->tv_nsec + (NSPERSEC/NS100PERSEC) - 1)
-			    / (NSPERSEC/NS100PERSEC);
-      /* negate waittime to code as duration for NtSetTimer() below cygwait() */
-      waittime.QuadPart = -waittime.QuadPart;
-    }
-
-  return sigwait_common (set, info, timeout ? &waittime : cw_infinite);
-}
-
 extern "C" int
 sigwait (const sigset_t *set, int *sig_ptr)
 {
@@ -698,7 +558,7 @@ sigwait (const sigset_t *set, int *sig_ptr)
 
   do
     {
-      sig = sigwait_common (set, NULL, cw_infinite);
+      sig = sigwaitinfo (set, NULL);
     }
   while (sig == -1 && get_errno () == EINTR);
   if (sig > 0)
@@ -709,7 +569,43 @@ sigwait (const sigset_t *set, int *sig_ptr)
 extern "C" int
 sigwaitinfo (const sigset_t *set, siginfo_t *info)
 {
-  return sigwait_common (set, info, cw_infinite);
+  int res = -1;
+
+  pthread_testcancel ();
+
+  __try
+    {
+      set_signal_mask (_my_tls.sigwait_mask, *set);
+      sig_dispatch_pending (true);
+
+      switch (cygwait (NULL, cw_infinite, cw_sig_eintr | cw_cancel | cw_cancel_self))
+	{
+	case WAIT_SIGNALED:
+	  if (!sigismember (set, _my_tls.infodata.si_signo))
+	    set_errno (EINTR);
+	  else
+	    {
+	      _my_tls.lock ();
+	      if (info)
+		*info = _my_tls.infodata;
+	      res = _my_tls.infodata.si_signo;
+	      _my_tls.sig = 0;
+	      if (_my_tls.retaddr () == (__tlsstack_t) sigdelayed)
+		_my_tls.pop ();
+	      _my_tls.unlock ();
+	    }
+	  break;
+	default:
+	  __seterrno ();
+	  break;
+	}
+    }
+  __except (EFAULT) {
+    res = -1;
+  }
+  __endtry
+  sigproc_printf ("returning signal %d", res);
+  return res;
 }
 
 /* FIXME: SUSv3 says that this function should block until the signal has
@@ -726,17 +622,10 @@ sigqueue (pid_t pid, int sig, const union sigval value)
       set_errno (ESRCH);
       return -1;
     }
-  if (sig == 0)
-    return 0;
-  if (sig < 0 || sig >= _NSIG)
-    {
-      set_errno (EINVAL);
-      return -1;
-    }
   si.si_signo = sig;
   si.si_code = SI_QUEUE;
   si.si_value = value;
-  return (int) sig_send (dest, si);
+  return sig_send (dest, si);
 }
 
 extern "C" int
@@ -804,62 +693,3 @@ sigaltstack (const stack_t *ss, stack_t *oss)
   __endtry
   return 0;
 }
-
-extern "C" int
-signalfd (int fd_in, const sigset_t *mask, int flags)
-{
-  int ret = -1;
-  fhandler_signalfd *fh;
-
-  debug_printf ("signalfd (%d, %p, %y)", fd_in, mask, flags);
-
-  if ((flags & ~(SFD_NONBLOCK | SFD_CLOEXEC)) != 0)
-    {
-      set_errno (EINVAL);
-      goto done;
-    }
-
-  if (fd_in != -1)
-    {
-      /* Change signal mask. */
-      cygheap_fdget fd (fd_in);
-
-      if (fd < 0)
-	goto done;
-      fh = fd->is_signalfd ();
-      if (!fh)
-	{
-	  set_errno (EINVAL);
-	  goto done;
-	}
-      __try
-        {
-	  if (fh->signalfd (mask, flags) == 0)
-	    ret = fd_in;
-	}
-      __except (EINVAL) {}
-      __endtry
-    }
-  else
-    {
-      /* Create new signalfd descriptor. */
-      cygheap_fdnew fd;
-
-      if (fd < 0)
-	goto done;
-      fh = (fhandler_signalfd *) build_fh_dev (*signalfd_dev);
-      if (fh && fh->signalfd (mask, flags) == 0)
-	{
-	  fd = fh;
-	  if (fd <= 2)
-	    set_std_handle (fd);
-	  ret = fd;
-	}
-      else
-	delete fh;
-    }
-
-done:
-  syscall_printf ("%R = signalfd (%d, %p, %y)", ret, fd_in, mask, flags);
-  return ret;
-}
