@@ -38,106 +38,66 @@ int __sp_ln;
 
 bool no_thread_exit_protect::flag;
 
-/* Flag to sig_send that signal goes to current process but no wait is
-   required. */
-char NO_COPY myself_nowait_dummy[1] = {'0'};
+char NO_COPY myself_nowait_dummy[1] = {'0'};// Flag to sig_send that signal goes to
+					//  current process but no wait is required
 
-/* All my children info.  Avoid expensive constructor ops at DLL
-   startup.
+#define Static static NO_COPY
 
-   This class can allocate memory.  But there's no need to free it
-   because only one instance of the class is created per process. */
-class child_procs {
-    static const int _NPROCS = 1024;
-    static const int _NPROCS_2 = 4095;
-    int _count;
-    uint8_t _procs[_NPROCS * sizeof (pinfo)] __attribute__ ((__aligned__));
-    pinfo *_procs_2;
-  public:
-    int count () const { return _count; }
-    int add_one () { return ++_count; }
-    int del_one () { return --_count; }
-    int reset () { return _count = 0; }
-    pinfo &operator[] (int idx)
-    {
-      if (idx >= _NPROCS)
-	{
-	  if (!_procs_2)
-	    {
-	      /* Use HeapAlloc to avoid propagating this memory area
-		 to the child processes. */
-	      _procs_2 = (pinfo *) HeapAlloc (GetProcessHeap (),
-					      HEAP_GENERATE_EXCEPTIONS
-					      | HEAP_ZERO_MEMORY,
-					      (_NPROCS_2 + 1) * sizeof (pinfo));
-	    }
-	  return _procs_2[idx - _NPROCS];
-	}
-      return ((pinfo *) _procs)[idx];
-    }
-    int max_child_procs () const { return _NPROCS + _NPROCS_2; }
-};
-static NO_COPY child_procs chld_procs;
 
-/* Start of queue for waiting threads. */
-static NO_COPY waitq waitq_head;
+Static int nprocs;			// Number of deceased children
+Static char cprocs[(NPROCS + 1) * sizeof (pinfo)];// All my children info
+#define procs ((pinfo *) cprocs)	// All this just to avoid expensive
+					// constructor operation  at DLL startup
+Static waitq waitq_head;		// Start of queue for wait'ing threads
 
-/* Controls access to subproc stuff. */
-static NO_COPY muto sync_proc_subproc;
+Static muto sync_proc_subproc;	// Control access to subproc stuff
 
 _cygtls NO_COPY *_sig_tls;
 
-static NO_COPY HANDLE my_sendsig;
-static NO_COPY HANDLE my_readsig;
-
-/* Used in select if a signalfd is part of the read descriptor set */
-HANDLE NO_COPY my_pendingsigs_evt;
+Static HANDLE my_sendsig;
+Static HANDLE my_readsig;
 
 /* Function declarations */
-static int checkstate (waitq *);
+static int __reg1 checkstate (waitq *);
 static __inline__ bool get_proc_lock (DWORD, DWORD);
-static int remove_proc (int);
-static bool stopped_or_terminated (waitq *, _pinfo *);
-static void wait_sig (VOID *arg);
+static bool __stdcall remove_proc (int);
+static bool __stdcall stopped_or_terminated (waitq *, _pinfo *);
+static void WINAPI wait_sig (VOID *arg);
 
 /* wait_sig bookkeeping */
 
 class pending_signals
 {
-  sigpacket sigs[_NSIG + 1];
+  sigpacket sigs[NSIG + 1];
   sigpacket start;
-  SRWLOCK queue_lock;
   bool retry;
-  void lock () { AcquireSRWLockExclusive (&queue_lock); }
-  void unlock () { ReleaseSRWLockExclusive (&queue_lock); }
 
 public:
-  pending_signals (): queue_lock (SRWLOCK_INIT) {}
   void add (sigpacket&);
-  bool pending () {retry = !!start.next; return retry;}
-  void clear (int sig, bool need_lock);
+  bool pending () {retry = true; return !!start.next;}
+  void clear (int sig) {sigs[sig].si.si_signo = 0;}
   void clear (_cygtls *tls);
-  friend void sig_dispatch_pending (bool);
-  friend void wait_sig (VOID *arg);
+  friend void __reg1 sig_dispatch_pending (bool);
+  friend void WINAPI wait_sig (VOID *arg);
 };
 
-static NO_COPY pending_signals sigq;
+Static pending_signals sigq;
 
 /* Functions */
-void
+void __stdcall
 sigalloc ()
 {
   cygheap->sigs = global_sigs =
-    (struct sigaction *) ccalloc_abort (HEAP_SIGS, _NSIG, sizeof (struct sigaction));
+    (struct sigaction *) ccalloc_abort (HEAP_SIGS, NSIG, sizeof (struct sigaction));
   global_sigs[SIGSTOP].sa_flags = SA_RESTART | SA_NODEFER;
 }
 
-void
+void __stdcall
 signal_fixup_after_exec ()
 {
   global_sigs = cygheap->sigs;
   /* Set up child's signal handlers */
-  for (int i = 0; i < _NSIG; i++)
+  for (int i = 0; i < NSIG; i++)
     {
       global_sigs[i].sa_mask = 0;
       if (global_sigs[i].sa_handler != SIG_IGN)
@@ -158,7 +118,7 @@ get_proc_lock (DWORD what, DWORD val)
 {
   if (!cygwin_finished_initializing)
     return true;
-  static NO_COPY int lastwhat = -1;
+  Static int lastwhat = -1;
   if (!sync_proc_subproc)
     {
       sigproc_printf ("sync_proc_subproc is NULL");
@@ -174,7 +134,7 @@ get_proc_lock (DWORD what, DWORD val)
   return false;
 }
 
-static bool
+static bool __stdcall
 proc_can_be_signalled (_pinfo *p)
 {
   if (!(p->exitcode & EXITCODE_SET))
@@ -189,29 +149,27 @@ proc_can_be_signalled (_pinfo *p)
   return false;
 }
 
-bool
+bool __reg1
 pid_exists (pid_t pid)
 {
-  pinfo p (pid);
-  return p && p->exists ();
+  return pinfo (pid)->exists ();
 }
 
 /* Return true if this is one of our children, false otherwise.  */
-static inline bool
+static inline bool __stdcall
 mychild (int pid)
 {
-  for (int i = 0; i < chld_procs.count (); i++)
-    if (chld_procs[i]->pid == pid)
+  for (int i = 0; i < nprocs; i++)
+    if (procs[i]->pid == pid)
       return true;
   return false;
 }
 
 /* Handle all subprocess requests
  */
-int
+int __reg2
 proc_subproc (DWORD what, uintptr_t val)
 {
-  int slot;
   int rc = 1;
   int potential_match;
   int clearing;
@@ -233,17 +191,19 @@ proc_subproc (DWORD what, uintptr_t val)
     /* Add a new subprocess to the children arrays.
      * (usually called from the main thread)
      */
-    case PROC_ADD_CHILD:
+    case PROC_ADDCHILD:
       /* Filled up process table? */
-      if (chld_procs.count () >= chld_procs.max_child_procs ())
+      if (nprocs >= NPROCS)
 	{
 	  sigproc_printf ("proc table overflow: hit %d processes, pid %d\n",
-			  chld_procs.count (), vchild->pid);
+			  nprocs, vchild->pid);
 	  rc = 0;
 	  set_errno (EAGAIN);
 	  break;
 	}
+      /* fall through intentionally */
 
+    case PROC_DETACHED_CHILD:
       if (vchild != myself)
 	{
 	  vchild->uid = myself->uid;
@@ -253,19 +213,20 @@ proc_subproc (DWORD what, uintptr_t val)
 	  vchild->ctty = myself->ctty;
 	  vchild->cygstarted = true;
 	  vchild->process_state |= PID_INITIALIZING;
-	  vchild->ppid = myself->pid;	/* always set last */
+	  vchild->ppid = what == PROC_DETACHED_CHILD ? 1 : myself->pid;	/* always set last */
 	}
-      break;
+      if (what == PROC_DETACHED_CHILD)
+	break;
+      /* fall through intentionally */
 
-    case PROC_ATTACH_CHILD:
-      slot = chld_procs.count ();
-      chld_procs[slot] = vchild;
-      rc = chld_procs[slot].wait ();
+    case PROC_REATTACH_CHILD:
+      procs[nprocs] = vchild;
+      rc = procs[nprocs].wait ();
       if (rc)
 	{
 	  sigproc_printf ("added pid %d to proc table, slot %d", vchild->pid,
-			  slot);
-	  chld_procs.add_one ();
+			  nprocs);
+	  nprocs++;
 	}
       break;
 
@@ -300,10 +261,8 @@ proc_subproc (DWORD what, uintptr_t val)
       goto scan_wait;
 
     case PROC_EXEC_CLEANUP:
-      /* Cleanup backwards to eliminate redundant copying of chld_procs
-	 array members inside remove_proc. */
-      while (chld_procs.count ())
-	remove_proc (chld_procs.count () - 1);
+      while (nprocs)
+	remove_proc (0);
       for (w = &waitq_head; w->next != NULL; w = w->next)
 	CloseHandle (w->next->ev);
       break;
@@ -316,8 +275,7 @@ proc_subproc (DWORD what, uintptr_t val)
       if (val)
 	sigproc_printf ("clear waiting threads");
       else
-	sigproc_printf ("looking for processes to reap, count %d",
-			chld_procs.count ());
+	sigproc_printf ("looking for processes to reap, nprocs %d", nprocs);
       clearing = val;
 
     scan_wait:
@@ -355,7 +313,7 @@ proc_subproc (DWORD what, uintptr_t val)
 	}
 
       if (global_sigs[SIGCHLD].sa_handler == (void *) SIG_IGN)
-	for (int i = 0; i < chld_procs.count (); i += remove_proc (i))
+	for (int i = 0; i < nprocs; i += remove_proc (i))
 	  continue;
   }
 
@@ -395,63 +353,45 @@ _cygtls::remove_wq (DWORD wait)
    Called on process exit.
    Also called by spawn_guts to disassociate any subprocesses from this
    process.  Subprocesses will then know to clean up after themselves and
-   will not become chld_procs.  */
-void
+   will not become procs.  */
+void __stdcall
 proc_terminate ()
 {
-  sigproc_printf ("child_procs count %d", chld_procs.count ());
-  if (chld_procs.count ())
+  sigproc_printf ("nprocs %d", nprocs);
+  if (nprocs)
     {
       sync_proc_subproc.acquire (WPSP);
 
       proc_subproc (PROC_CLEARWAIT, 1);
 
       /* Clean out proc processes from the pid list. */
-      for (int i = 0; i < chld_procs.count (); i++)
+      for (int i = 0; i < nprocs; i++)
 	{
 	  /* If we've execed then the execed process will handle setting ppid
 	     to 1 iff it is a Cygwin process.  */
 	  if (!have_execed || !have_execed_cygwin)
-	    chld_procs[i]->ppid = 1;
-	  if (chld_procs[i].wait_thread)
-	    chld_procs[i].wait_thread->terminate_thread ();
+	    procs[i]->ppid = 1;
+	  if (procs[i].wait_thread)
+	    procs[i].wait_thread->terminate_thread ();
 	  /* Release memory associated with this process unless it is 'myself'.
-	     'myself' is only in the chld_procs table when we've execed.  We
-	     reach here when the next process has finished initializing but we
-	     still can't free the memory used by 'myself' since it is used
-	     later on during cygwin tear down.  */
-	  if (chld_procs[i] != myself)
-	    chld_procs[i].release ();
+	     'myself' is only in the procs table when we've execed.  We reach
+	     here when the next process has finished initializing but we still
+	     can't free the memory used by 'myself' since it is used later on
+	     during cygwin tear down.  */
+	  if (procs[i] != myself)
+	    procs[i].release ();
 	}
-      chld_procs.reset ();
+      nprocs = 0;
       sync_proc_subproc.release ();
     }
   sigproc_printf ("leaving");
 }
 
 /* Clear pending signal */
-void
-sig_clear (int sig, bool need_lock)
+void __reg1
+sig_clear (int sig)
 {
-  sigq.clear (sig, need_lock);
-}
-
-/* Clear pending signals of specific si_signo.
-   Called from sigpacket::process(). */
-void
-pending_signals::clear (int sig, bool need_lock)
-{
-  sigpacket *q = sigs + sig;
-  if (!sig || !q->si.si_signo)
-    return;
-  if (need_lock)
-    lock ();
-  q->si.si_signo = 0;
-  q->prev->next = q->next;
-  if (q->next)
-    q->next->prev = q->prev;
-  if (need_lock)
-    unlock ();
+  sigq.clear (sig);
 }
 
 /* Clear pending signals of specific thread.  Called under TLS lock from
@@ -459,18 +399,16 @@ pending_signals::clear (int sig, bool need_lock)
 void
 pending_signals::clear (_cygtls *tls)
 {
-  sigpacket *q = &start;
+  sigpacket *q = &start, *qnext;
 
-  lock ();
-  while ((q = q->next))
-    if (q->sigtls == tls)
+  while ((qnext = q->next))
+    if (qnext->sigtls == tls)
       {
-	q->si.si_signo = 0;
-	q->prev->next = q->next;
-	if (q->next)
-	  q->next->prev = q->prev;
+	qnext->si.si_signo = 0;
+	q->next = qnext->next;
       }
-  unlock ();
+    else
+      q = qnext;
 }
 
 /* Clear pending signals of specific thread.  Called from _cygtls::remove */
@@ -483,7 +421,7 @@ _cygtls::remove_pending_sigs ()
 extern "C" int
 sigpending (sigset_t *mask)
 {
-  sigset_t outset = sig_send (myself, __SIGPENDING, &_my_tls);
+  sigset_t outset = (sigset_t) sig_send (myself, __SIGPENDING, &_my_tls);
   if (outset == SIG_BAD_MASK)
     return -1;
   *mask = outset;
@@ -491,7 +429,7 @@ sigpending (sigset_t *mask)
 }
 
 /* Force the wait_sig thread to wake up and scan for pending signals */
-void
+void __reg1
 sig_dispatch_pending (bool fast)
 {
   /* Non-atomically test for any signals pending and wake up wait_sig if any are
@@ -503,13 +441,13 @@ sig_dispatch_pending (bool fast)
 
 /* Signal thread initialization.  Called from dll_crt0_1.
    This routine starts the signal handling thread.  */
-void
+void __stdcall
 sigproc_init ()
 {
   char char_sa_buf[1024];
   PSECURITY_ATTRIBUTES sa = sec_user_nih ((PSECURITY_ATTRIBUTES) char_sa_buf, cygheap->user.sid());
   DWORD err = fhandler_pipe::create (sa, &my_readsig, &my_sendsig,
-				     _NSIG * sizeof (sigpacket), "sigwait",
+				     NSIG * sizeof (sigpacket), "sigwait",
 				     PIPE_ADD_PID);
   if (err)
     {
@@ -518,10 +456,6 @@ sigproc_init ()
     }
   ProtectHandle (my_readsig);
   myself->sendsig = my_sendsig;
-  my_pendingsigs_evt = CreateEvent (NULL, TRUE, FALSE, NULL);
-  if (!my_pendingsigs_evt)
-    api_fatal ("couldn't create pending signal event, %E");
-
   /* sync_proc_subproc is used by proc_subproc.  It serializes
      access to the children and proc arrays.  */
   sync_proc_subproc.init ("sync_proc_subproc");
@@ -568,7 +502,7 @@ exit_thread (DWORD res)
   ExitThread (res);
 }
 
-sigset_t
+int __reg3
 sig_send (_pinfo *p, int sig, _cygtls *tls)
 {
   siginfo_t si = {};
@@ -581,7 +515,7 @@ sig_send (_pinfo *p, int sig, _cygtls *tls)
    If pinfo *p == NULL, send to the current process.
    If sending to this process, wait for notification that a signal has
    completed before returning.  */
-sigset_t
+int __reg3
 sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
 {
   int rc = 1;
@@ -611,14 +545,6 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
       p = myself;
     }
 
-  /* If myself is the stub process, send signal to the child process
-     rather than myself. The fact that myself->dwProcessId is not equal
-     to the current process id indicates myself is the stub process. */
-  if (its_me && myself->dwProcessId != GetCurrentProcessId ())
-    {
-      wait_for_completion = false;
-      its_me = false;
-    }
 
   if (its_me)
     sendsig = my_sendsig;
@@ -700,7 +626,7 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
   sigset_t pending;
   if (!its_me)
     pack.mask = NULL;
-  else if (si.si_signo == __SIGPENDING || si.si_signo == __SIGPENDINGALL)
+  else if (si.si_signo == __SIGPENDING)
     pack.mask = &pending;
   else if (si.si_signo == __SIGFLUSH || si.si_signo > 0)
     {
@@ -742,9 +668,6 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
       memcpy (p, si._si_commune._si_str, n); p += n;
     }
 
-  unsigned cw_mask;
-  cw_mask = pack.si.si_signo == __SIGFLUSHFAST ? 0 : cw_sig_restart;
-
   DWORD nb;
   BOOL res;
   /* Try multiple times to send if packsize != nb since that probably
@@ -754,7 +677,7 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
       res = WriteFile (sendsig, leader, packsize, &nb, NULL);
       if (!res || packsize == nb)
 	break;
-      cygwait (NULL, 10, cw_mask);
+      Sleep (10);
       res = 0;
     }
 
@@ -787,7 +710,7 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
   if (wait_for_completion)
     {
       sigproc_printf ("Waiting for pack.wakeup %p", pack.wakeup);
-      rc = cygwait (pack.wakeup, WSSC, cw_mask);
+      rc = WaitForSingleObject (pack.wakeup, WSSC);
       ForceCloseHandle (pack.wakeup);
     }
   else
@@ -808,6 +731,9 @@ sig_send (_pinfo *p, siginfo_t& si, _cygtls *tls)
       rc = -1;
     }
 
+  if (wait_for_completion && si.si_signo != __SIGFLUSHFAST)
+    _my_tls.call_signal_handler ();
+
 out:
   if (communing && rc)
     {
@@ -818,10 +744,10 @@ out:
     }
   if (pack.wakeup)
     ForceCloseHandle (pack.wakeup);
-  if (si.si_signo != __SIGPENDING && si.si_signo != __SIGPENDINGALL)
+  if (si.si_signo != __SIGPENDING)
     /* nothing */;
   else if (!rc)
-    rc = pending;
+    rc = (int) pending;
   else
     rc = SIG_BAD_MASK;
   sigproc_printf ("returning %p from sending signal %d", rc, si.si_signo);
@@ -834,11 +760,31 @@ int child_info::retry_count = 0;
    by fork/spawn/exec. */
 child_info::child_info (unsigned in_cb, child_info_types chtype,
 			bool need_subproc_ready):
-  msv_count (0), cb (in_cb), intro (PROC_MAGIC_GENERIC),
-  magic (CHILD_INFO_MAGIC), type (chtype), cygheap (::cygheap),
-  cygheap_max (::cygheap_max), flag (0), retry (child_info::retry_count),
-  rd_proc_pipe (NULL), wr_proc_pipe (NULL), sigmask (_my_tls.sigmask)
+  cb (in_cb), intro (PROC_MAGIC_GENERIC), magic (CHILD_INFO_MAGIC),
+  type (chtype), cygheap (::cygheap), cygheap_max (::cygheap_max),
+  flag (0), retry (child_info::retry_count), rd_proc_pipe (NULL),
+  wr_proc_pipe (NULL)
 {
+  /* It appears that when running under WOW64 on Vista 64, the first DWORD
+     value in the datastructure lpReserved2 is pointing to (msv_count in
+     Cygwin), has to reflect the size of that datastructure as used in the
+     Microsoft C runtime (a count value, counting the number of elements in
+     two subsequent arrays, BYTE[count and HANDLE[count]), even though the C
+     runtime isn't used.  Otherwise, if msv_count is 0 or too small, the
+     datastructure gets overwritten.
+
+     This seems to be a bug in Vista's WOW64, which apparently copies the
+     lpReserved2 datastructure not using the cbReserved2 size information,
+     but using the information given in the first DWORD within lpReserved2
+     instead.  However, it's not clear if a non-0 count doesn't result in
+     trying to evaluate the content, so we do this really only for Vista 64.
+
+     The value is sizeof (child_info_*) / 5 which results in a count which
+     covers the full datastructure, plus not more than 4 extra bytes.  This
+     is ok as long as the child_info structure is cosily stored within a bigger
+     datastructure. */
+  msv_count = wincap.needs_count_in_si_lpres2 () ? in_cb / 5 : 0;
+
   fhandler_union_cb = sizeof (fhandler_union);
   user_h = cygwin_user_h;
   if (strace.active ())
@@ -864,16 +810,11 @@ child_info::child_info (unsigned in_cb, child_info_types chtype,
     }
   sigproc_printf ("subproc_ready %p", subproc_ready);
   /* Create an inheritable handle to pass to the child process.  This will
-     allow the child to copy cygheap etc. from the parent to itself.  If
-     we're forking, we also need handle duplicate access. */
+     allow the child to duplicate handles from the parent to itself. */
   parent = NULL;
-  DWORD perms = PROCESS_QUERY_LIMITED_INFORMATION | PROCESS_VM_READ
-		| PROCESS_VM_OPERATION | SYNCHRONIZE;
-  if (type == _CH_FORK)
-    perms |= PROCESS_DUP_HANDLE;
-
   if (!DuplicateHandle (GetCurrentProcess (), GetCurrentProcess (),
-			GetCurrentProcess (), &parent, perms, TRUE, 0))
+			GetCurrentProcess (), &parent, 0, true,
+			DUPLICATE_SAME_ACCESS))
     system_printf ("couldn't create handle to myself for child, %E");
 }
 
@@ -910,8 +851,8 @@ cygheap_exec_info::alloc ()
   cygheap_exec_info *res =
     (cygheap_exec_info *) ccalloc_abort (HEAP_1_EXEC, 1,
 					 sizeof (cygheap_exec_info)
-					 + (chld_procs.count ()
-					    * sizeof (children[0])));
+					 + (nprocs * sizeof (children[0])));
+  res->sigmask = _my_tls.sigmask;
   return res;
 }
 
@@ -919,8 +860,7 @@ void
 child_info_spawn::wait_for_myself ()
 {
   postfork (myself);
-  if (myself.remember ())
-    myself.attach ();
+  myself.remember (false);
   WaitForSingleObject (ev, INFINITE);
 }
 
@@ -986,13 +926,10 @@ child_info_spawn::cleanup ()
 inline void
 cygheap_exec_info::record_children ()
 {
-  for (nchildren = 0; nchildren < chld_procs.count (); nchildren++)
+  for (nchildren = 0; nchildren < nprocs; nchildren++)
     {
-      children[nchildren].pid = chld_procs[nchildren]->pid;
-      children[nchildren].p = chld_procs[nchildren];
-      /* Set inheritance of required child handles for reattach_children
-	 in the about-to-be-execed process. */
-      children[nchildren].p.set_inheritance (true);
+      children[nchildren].pid = procs[nchildren]->pid;
+      children[nchildren].p = procs[nchildren];
     }
 }
 
@@ -1014,7 +951,7 @@ cygheap_exec_info::reattach_children (HANDLE parent)
       pinfo p (parent, children[i].p, children[i].pid);
       if (!p)
 	debug_only_printf ("couldn't reattach child %d from previous process", children[i].pid);
-      else if (!p.attach ())
+      else if (!p.reattach ())
 	debug_only_printf ("attach of child process %d failed", children[i].pid);
       else
 	debug_only_printf ("reattached pid %d<%u>, process handle %p, rd_proc_pipe %p->%p",
@@ -1106,7 +1043,7 @@ child_info::proc_retry (HANDLE h)
   if (!exit_code)
     return EXITCODE_OK;
   sigproc_printf ("exit_code %y", exit_code);
-  switch ((NTSTATUS) exit_code)
+  switch (exit_code)
     {
     case STILL_ACTIVE:	/* shouldn't happen */
       sigproc_printf ("STILL_ACTIVE?  How'd we get here?");
@@ -1119,7 +1056,7 @@ child_info::proc_retry (HANDLE h)
     case STATUS_CONTROL_C_EXIT:
       if (saw_ctrl_c ())
 	return EXITCODE_OK;
-      fallthrough;
+      /* fall through intentionally */
     case STATUS_DLL_INIT_FAILED:
     case STATUS_DLL_INIT_FAILED_LOGOFF:
     case EXITCODE_RETRY:
@@ -1151,10 +1088,7 @@ child_info_fork::abort (const char *fmt, ...)
     {
       va_list ap;
       va_start (ap, fmt);
-      if (silentfail ())
-	strace_vprintf (DEBUG, fmt, ap);
-      else
-	strace_vprintf (SYSTEM, fmt, ap);
+      strace_vprintf (SYSTEM, fmt, ap);
       TerminateProcess (GetCurrentProcess (), EXITCODE_FORK_FAILED);
     }
   if (retry > 0)
@@ -1165,18 +1099,18 @@ child_info_fork::abort (const char *fmt, ...)
 /* Check the state of all of our children to see if any are stopped or
  * terminated.
  */
-static int
+static int __reg1
 checkstate (waitq *parent_w)
 {
   int potential_match = 0;
 
-  sigproc_printf ("child_procs count %d", chld_procs.count ());
+  sigproc_printf ("nprocs %d", nprocs);
 
   /* Check already dead processes first to see if they match the criteria
    * given in w->next.  */
   int res;
-  for (int i = 0; i < chld_procs.count (); i++)
-    if ((res = stopped_or_terminated (parent_w, chld_procs[i])))
+  for (int i = 0; i < nprocs; i++)
+    if ((res = stopped_or_terminated (parent_w, procs[i])))
       {
 	remove_proc (i);
 	potential_match = 1;
@@ -1184,40 +1118,40 @@ checkstate (waitq *parent_w)
       }
 
   sigproc_printf ("no matching terminated children found");
-  potential_match = -!!chld_procs.count ();
+  potential_match = -!!nprocs;
 
 out:
   sigproc_printf ("returning %d", potential_match);
   return potential_match;
 }
 
-/* Remove a proc from chld_procs by swapping it with the last child in the list.
+/* Remove a proc from procs by swapping it with the last child in the list.
    Also releases shared memory of exited processes.  */
-static int
+static bool __stdcall
 remove_proc (int ci)
 {
   if (have_execed)
     {
-      if (_my_tls._ctinfo != chld_procs[ci].wait_thread)
-	chld_procs[ci].wait_thread->terminate_thread ();
+      if (_my_tls._ctinfo != procs[ci].wait_thread)
+	procs[ci].wait_thread->terminate_thread ();
     }
-  else if (chld_procs[ci] && chld_procs[ci]->exists ())
-    return 1;
+  else if (procs[ci]->exists ())
+    return true;
 
-  sigproc_printf ("removing chld_procs[%d], pid %d, child_procs count %d",
-		  ci, chld_procs[ci]->pid, chld_procs.count ());
-  if (chld_procs[ci] != myself)
-    chld_procs[ci].release ();
-  if (ci < chld_procs.del_one ())
+  sigproc_printf ("removing procs[%d], pid %d, nprocs %d", ci, procs[ci]->pid,
+		  nprocs);
+  if (procs[ci] != myself)
+    procs[ci].release ();
+  if (ci < --nprocs)
     {
       /* Wait for proc_waiter thread to make a copy of this element before
 	 moving it or it may become confused.  The chances are very high that
 	 the proc_waiter thread has already done this by the time we
 	 get here.  */
       if (!have_execed && !exit_state)
-	while (!chld_procs[chld_procs.count ()].waiter_ready)
+	while (!procs[nprocs].waiter_ready)
 	  yield ();
-      chld_procs[ci] = chld_procs[chld_procs.count ()];
+      procs[ci] = procs[nprocs];
     }
   return 0;
 }
@@ -1228,7 +1162,7 @@ remove_proc (int ci)
    child is the subprocess being considered.
 
    Returns non-zero if waiting thread released.  */
-static bool
+static bool __stdcall
 stopped_or_terminated (waitq *parent_w, _pinfo *child)
 {
   int might_match;
@@ -1322,18 +1256,13 @@ pending_signals::add (sigpacket& pack)
   if (se->si.si_signo)
     return;
   *se = pack;
-  lock ();
   se->next = start.next;
-  se->prev = &start;
-  se->prev->next = se;
-  if (se->next)
-    se->next->prev = se;
-  unlock ();
+  start.next = se;
 }
 
 /* Process signals by waiting for signal data to arrive in a pipe.
    Set a completion event if one was specified. */
-static void
+static void WINAPI
 wait_sig (VOID *)
 {
   _sig_tls = &_my_tls;
@@ -1344,26 +1273,12 @@ wait_sig (VOID *)
 
   hntdll = GetModuleHandle ("ntdll.dll");
 
-  SetThreadPriority (GetCurrentThread (), THREAD_PRIORITY_NORMAL);
-
-  /* GetTickCount() here is enough because GetTickCount() - t0 does
-     not overflow until 49 days psss. Even if GetTickCount() overflows,
-     GetTickCount() - t0 returns correct value, since underflow in
-     unsigned wraps correctly. Pending a signal for more thtn 49
-     days would be noncense. */
-  DWORD t0 = GetTickCount ();
   for (;;)
     {
       DWORD nb;
       sigpacket pack = {};
       if (sigq.retry)
 	pack.si.si_signo = __SIGFLUSH;
-      else if (sigq.start.next
-	       && PeekNamedPipe (my_readsig, NULL, 0, NULL, &nb, NULL) && !nb)
-	{
-	  Sleep (GetTickCount () - t0 > 10 ? 1 : 0);
-	  pack.si.si_signo = __SIGFLUSH;
-	}
       else if (!ReadFile (my_readsig, &pack, sizeof (pack), &nb, NULL))
 	Sleep (INFINITE);	/* Assume were exiting.  Never exit this thread */
       else if (nb != sizeof (pack) || !pack.si.si_signo)
@@ -1371,39 +1286,24 @@ wait_sig (VOID *)
 	  system_printf ("garbled signal pipe data nb %u, sig %d", nb, pack.si.si_signo);
 	  continue;
 	}
-      if (pack.si.si_signo != __SIGFLUSH)
-	t0 = GetTickCount ();
 
       sigq.retry = false;
       /* Don't process signals when we start exiting */
       if (exit_state > ES_EXIT_STARTING && pack.si.si_signo > 0)
-	goto skip_process_signal;
+	continue;
 
       sigset_t dummy_mask;
       threadlist_t *tl_entry;
       if (!pack.mask)
 	{
-	  /* There's a short time at process startup of a forked process,
-	     when _main_tls points to the system-allocated stack, not to
-	     the parent thread. In that case find_tls fails, and we fetch
-	     the sigmask from the child_info passed from the parent. */
-	  if (cygwin_finished_initializing)
-	    {
-	      tl_entry = cygheap->find_tls (_main_tls);
-	      dummy_mask = _main_tls->sigmask;
-	      cygheap->unlock_tls (tl_entry);
-	    }
-	  else if (child_proc_info)
-	    dummy_mask = child_proc_info->sigmask;
-	  else
-	    dummy_mask = 0;
+	  tl_entry = cygheap->find_tls (_main_tls);
+	  dummy_mask = _main_tls->sigmask;
+	  cygheap->unlock_tls (tl_entry);
 	  pack.mask = &dummy_mask;
 	}
 
-      sigpacket *q;
-      q = &sigq.start;
-      bool clearwait;
-      clearwait = false;
+      sigpacket *q = &sigq.start;
+      bool clearwait = false;
       switch (pack.si.si_signo)
 	{
 	case __SIGCOMMUNE:
@@ -1412,45 +1312,15 @@ wait_sig (VOID *)
 	case __SIGSTRACE:
 	  strace.activate (false);
 	  break;
-	case __SIGPENDINGALL:
-	  {
-	    unsigned bit;
-	    bool issig_wait;
-
-	    *pack.mask = 0;
-	    sigq.lock ();
-	    while ((q = q->next))
-	      {
-		_cygtls *sigtls = q->sigtls ?: _main_tls;
-		if (sigtls->sigmask & (bit = SIGTOMASK (q->si.si_signo)))
-		  {
-		    tl_entry = cygheap->find_tls (q->si.si_signo, issig_wait);
-		    if (tl_entry)
-		      {
-			*pack.mask |= bit;
-			cygheap->unlock_tls (tl_entry);
-		      }
-		  }
-	      }
-	    sigq.unlock ();
-	  }
-	  break;
 	case __SIGPENDING:
 	  {
 	    unsigned bit;
 
 	    *pack.mask = 0;
 	    tl_entry = cygheap->find_tls (pack.sigtls);
-	    sigq.lock ();
 	    while ((q = q->next))
-	      {
-		/* Skip thread-specific signals for other threads. */
-		if (q->sigtls && pack.sigtls != q->sigtls)
-		  continue;
-		if (pack.sigtls->sigmask & (bit = SIGTOMASK (q->si.si_signo)))
-		  *pack.mask |= bit;
-	      }
-	    sigq.unlock ();
+	      if (pack.sigtls->sigmask & (bit = SIGTOMASK (q->si.si_signo)))
+		*pack.mask |= bit;
 	    cygheap->unlock_tls (tl_entry);
 	  }
 	  break;
@@ -1458,8 +1328,7 @@ wait_sig (VOID *)
 	  sig_held = true;
 	  break;
 	case __SIGSETPGRP:
-	  init_console_handler (::cygheap->ctty
-				&& ::cygheap->ctty->is_console ());
+	  init_console_handler (true);
 	  break;
 	case __SIGTHREADEXIT:
 	  {
@@ -1485,52 +1354,35 @@ wait_sig (VOID *)
 	  break;
 	default:	/* Normal (positive) signal */
 	  if (pack.si.si_signo < 0)
-	    sig_clear (-pack.si.si_signo, true);
+	    sig_clear (-pack.si.si_signo);
 	  else
 	    sigq.add (pack);
-	  fallthrough;
 	case __SIGNOHOLD:
 	  sig_held = false;
-	  fallthrough;
 	case __SIGFLUSH:
 	case __SIGFLUSHFAST:
 	  if (!sig_held)
 	    {
+	      sigpacket *qnext;
 	      /* Check the queue for signals.  There will always be at least one
 		 thing on the queue if this was a valid signal.  */
-	      sigq.lock ();
-	      while ((q = q->next))
+	      while ((qnext = q->next))
 		{
-		  if (q->si.si_signo && q->process () > 0)
+		  if (qnext->si.si_signo && qnext->process () <= 0)
+		    q = qnext;
+		  else
 		    {
-		      q->si.si_signo = 0;
-		      q->prev->next = q->next;
-		      if (q->next)
-			q->next->prev = q->prev;
+		      q->next = qnext->next;
+		      qnext->si.si_signo = 0;
 		    }
 		}
-	      sigq.unlock ();
-	      /* At least one signal still queued?  The event is used in select
-		 only, and only to decide if WFMO should wake up in case a
-		 signalfd is waiting via select/poll for being ready to read a
-		 pending signal.  This method wakes up all threads hanging in
-		 select and having a signalfd, as soon as a pending signal is
-		 available, but it's certainly better than constant polling. */
-	      if (sigq.start.next)
-		SetEvent (my_pendingsigs_evt);
-	      else
-		ResetEvent (my_pendingsigs_evt);
 	      if (pack.si.si_signo == SIGCHLD)
 		clearwait = true;
 	    }
 	  break;
-	case __SIGNONCYGCHLD:
-	  fhandler_pipe::sigproc_worker ();
-	  break;
 	}
       if (clearwait && !have_execed)
 	proc_subproc (PROC_CLEARWAIT, 0);
-skip_process_signal:
       if (pack.wakeup)
 	{
 	  sigproc_printf ("signalling pack.wakeup %p", pack.wakeup);
