@@ -22,7 +22,7 @@ details. */
    is affected as well, but we can't transform it as long as we accept Win32
    paths as input. */
 static const WCHAR tfx_chars[] = {
- 0xf000 |   0, 0xf000 |   1, 0xf000 |   2, 0xf000 |   3,
+	    0, 0xf000 |   1, 0xf000 |   2, 0xf000 |   3,
  0xf000 |   4, 0xf000 |   5, 0xf000 |   6, 0xf000 |   7,
  0xf000 |   8, 0xf000 |   9, 0xf000 |  10, 0xf000 |  11,
  0xf000 |  12, 0xf000 |  13, 0xf000 |  14, 0xf000 |  15,
@@ -61,7 +61,7 @@ static const WCHAR tfx_chars[] = {
    converting back space and dot on filesystems only supporting DOS
    filenames. */
 static const WCHAR tfx_rev_chars[] = {
- 0xf000 |   0, 0xf000 |   1, 0xf000 |   2, 0xf000 |   3,
+	    0, 0xf000 |   1, 0xf000 |   2, 0xf000 |   3,
  0xf000 |   4, 0xf000 |   5, 0xf000 |   6, 0xf000 |   7,
  0xf000 |   8, 0xf000 |   9, 0xf000 |  10, 0xf000 |  11,
  0xf000 |  12, 0xf000 |  13, 0xf000 |  14, 0xf000 |  15,
@@ -103,15 +103,6 @@ transform_chars (PWCHAR path, PWCHAR path_end)
       *path = tfx_chars[*path];
 }
 
-PWCHAR
-transform_chars_af_unix (PWCHAR out, const char *path, __socklen_t len)
-{
-  len -= sizeof (__sa_family_t);
-  for (const unsigned char *p = (const unsigned char *) path; len-- > 0; ++p)
-    *out++ = (*p <= 0x7f) ? tfx_chars[*p] : *p;
-  return out;
-}
-
 /* The SJIS, JIS and eucJP conversion in newlib does not use UTF as
    wchar_t character representation.  That's unfortunate for us since
    we require UTF for the OS.  What we do here is to have our own
@@ -410,9 +401,9 @@ __big5_mbtowc (struct _reent *r, wchar_t *pwc, const char *s, size_t n,
        to buffer size, it's a bug in Cygwin and the buffer in the calling
        function should be raised.
 */
-size_t
-_sys_wcstombs (char *dst, size_t len, const wchar_t *src, size_t nwc,
-	       bool is_path)
+static size_t __reg3
+sys_wcstombs (char *dst, size_t len, const wchar_t *src, size_t nwc,
+	      bool is_path)
 {
   char buf[10];
   char *ptr = dst;
@@ -436,7 +427,7 @@ _sys_wcstombs (char *dst, size_t len, const wchar_t *src, size_t nwc,
       /* Convert UNICODE private use area.  Reverse functionality for the
 	 ASCII area <= 0x7f (only for path names) is transform_chars above.
 	 Reverse functionality for invalid bytes in a multibyte sequence is
-	 in _sys_mbstowcs below. */
+	 in sys_cp_mbstowcs below. */
       if (is_path && (pw & 0xff00) == 0xf000
 	  && (((cwc = (pw & 0xff)) <= 0x7f && tfx_rev_chars[cwc] >= 0xf000)
 	      || (cwc >= 0x80 && MB_CUR_MAX > 1)))
@@ -498,6 +489,18 @@ _sys_wcstombs (char *dst, size_t len, const wchar_t *src, size_t nwc,
   return n;
 }
 
+size_t __reg3
+sys_wcstombs (char *dst, size_t len, const wchar_t * src, size_t nwc)
+{
+  return sys_wcstombs (dst, len, src, nwc, true);
+}
+
+size_t __reg3
+sys_wcstombs_no_path (char *dst, size_t len, const wchar_t * src, size_t nwc)
+{
+  return sys_wcstombs (dst, len, src, nwc, false);
+}
+
 /* Allocate a buffer big enough for the string, always including the
    terminating '\0'.  The buffer pointer is returned in *dst_p, the return
    value is the number of bytes written to the buffer, as usual.
@@ -508,13 +511,13 @@ _sys_wcstombs (char *dst, size_t len, const wchar_t *src, size_t nwc,
    Note that this code is shared by cygserver (which requires it via
    __small_vsprintf) and so when built there plain calloc is the
    only choice.  */
-size_t
-_sys_wcstombs_alloc (char **dst_p, int type, const wchar_t *src, size_t nwc,
+static size_t __reg3
+sys_wcstombs_alloc (char **dst_p, int type, const wchar_t *src, size_t nwc,
 		bool is_path)
 {
   size_t ret;
 
-  ret = _sys_wcstombs (NULL, (size_t) -1, src, nwc, is_path);
+  ret = sys_wcstombs (NULL, (size_t) -1, src, nwc, is_path);
   if (ret > 0)
     {
       size_t dlen = ret + 1;
@@ -525,19 +528,32 @@ _sys_wcstombs_alloc (char **dst_p, int type, const wchar_t *src, size_t nwc,
 	*dst_p = (char *) ccalloc ((cygheap_types) type, dlen, sizeof (char));
       if (!*dst_p)
 	return 0;
-      ret = _sys_wcstombs (*dst_p, dlen, src, nwc, is_path);
+      ret = sys_wcstombs (*dst_p, dlen, src, nwc, is_path);
     }
   return ret;
 }
 
-/* _sys_mbstowcs is actually most of the time called as sys_mbstowcs with
+size_t __reg3
+sys_wcstombs_alloc (char **dst_p, int type, const wchar_t *src, size_t nwc)
+{
+  return sys_wcstombs_alloc (dst_p, type, src, nwc, true);
+}
+
+size_t __reg3
+sys_wcstombs_alloc_no_path (char **dst_p, int type, const wchar_t *src,
+		size_t nwc)
+{
+  return sys_wcstombs_alloc (dst_p, type, src, nwc, false);
+}
+
+/* sys_cp_mbstowcs is actually most of the time called as sys_mbstowcs with
    a 0 codepage.  If cp is not 0, the codepage is evaluated and used for the
    conversion.  This is so that fhandler_console can switch to an alternate
    charset, which is the charset returned by GetConsoleCP ().  Most of the
    time this is used for box and line drawing characters. */
-size_t
-_sys_mbstowcs (mbtowc_p f_mbtowc, wchar_t *dst, size_t dlen, const char *src,
-	       size_t nms)
+size_t __reg3
+sys_cp_mbstowcs (mbtowc_p f_mbtowc, wchar_t *dst, size_t dlen,
+		 const char *src, size_t nms)
 {
   wchar_t *ptr = dst;
   unsigned const char *pmbs = (unsigned const char *) src;
@@ -610,7 +626,7 @@ _sys_mbstowcs (mbtowc_p f_mbtowc, wchar_t *dst, size_t dlen, const char *src,
 	  /* The technique is based on a discussion here:
 	     http://www.mail-archive.com/linux-utf8@nl.linux.org/msg00080.html
 
-	     Invalid bytes in a multibyte sequence are converted to
+	     Invalid bytes in a multibyte secuence are converted to
 	     the private use area which is already used to store ASCII
 	     chars invalid in Windows filenames.  This technque allows
 	     to store them in a symmetric way. */
@@ -645,8 +661,17 @@ _sys_mbstowcs (mbtowc_p f_mbtowc, wchar_t *dst, size_t dlen, const char *src,
   return count;
 }
 
+size_t __reg3
+sys_mbstowcs (wchar_t * dst, size_t dlen, const char *src, size_t nms)
+{
+  mbtowc_p f_mbtowc = __MBTOWC;
+  if (f_mbtowc == __ascii_mbtowc)
+    f_mbtowc = __utf8_mbtowc;
+  return sys_cp_mbstowcs (f_mbtowc, dst, dlen, src, nms);
+}
+
 /* Same as sys_wcstombs_alloc, just backwards. */
-size_t
+size_t __reg3
 sys_mbstowcs_alloc (wchar_t **dst_p, int type, const char *src, size_t nms)
 {
   size_t ret;
@@ -681,205 +706,6 @@ strccpy (char *__restrict s1, const char **__restrict s2, char c)
   return s1;
 }
 
-const unsigned char case_folded_lower[] = {
-   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
-  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
-  32, '!', '"', '#', '$', '%', '&',  39, '(', ')', '*', '+', ',', '-', '.', '/',
- '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?',
- '@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
- 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '[',  92, ']', '^', '_',
- '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
- 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~', 127,
- 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
- 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
- 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
- 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191,
- 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
- 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
- 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
- 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
-};
-
-const unsigned char case_folded_upper[] = {
-   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
-  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
-  32, '!', '"', '#', '$', '%', '&',  39, '(', ')', '*', '+', ',', '-', '.', '/',
- '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?',
- '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
- 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[',  92, ']', '^', '_',
- '`', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
- 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '{', '|', '}', '~', 127,
- 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
- 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
- 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
- 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191,
- 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
- 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
- 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
- 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
-};
-
-const char isalpha_array[] = {
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
-0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,   0,   0,   0,   0,   0,
-   0,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
-0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0
-};
-
-extern "C" int __stdcall
-cygwin_wcscasecmp (const wchar_t *ws, const wchar_t *wt)
-{
-  UNICODE_STRING us, ut;
-
-  RtlInitUnicodeString (&us, ws);
-  RtlInitUnicodeString (&ut, wt);
-  return RtlCompareUnicodeString (&us, &ut, TRUE);
-}
-
-extern "C" int __stdcall
-cygwin_wcsncasecmp (const wchar_t  *ws, const wchar_t *wt, size_t n)
-{
-  UNICODE_STRING us, ut;
-  size_t ls = 0, lt = 0;
-
-  while (ws[ls] && ls < n)
-    ++ls;
-  RtlInitCountedUnicodeString (&us, ws, ls * sizeof (WCHAR));
-  while (wt[lt] && lt < n)
-    ++lt;
-  RtlInitCountedUnicodeString (&ut, wt, lt * sizeof (WCHAR));
-  return RtlCompareUnicodeString (&us, &ut, TRUE);
-}
-
-extern "C" int __stdcall
-cygwin_strcasecmp (const char *cs, const char *ct)
-{
-  UNICODE_STRING us, ut;
-  ULONG len, ulen;
-
-  len = strlen (cs) + 1;
-  ulen = len * sizeof (WCHAR);
-  RtlInitEmptyUnicodeString (&us, (PWCHAR) alloca (ulen), ulen);
-  us.Length = sys_mbstowcs (us.Buffer, len, cs) * sizeof (WCHAR);
-
-  len = strlen (ct) + 1;
-  ulen = len * sizeof (WCHAR);
-  RtlInitEmptyUnicodeString (&ut, (PWCHAR) alloca (ulen), ulen);
-  ut.Length = sys_mbstowcs (ut.Buffer, len, ct) * sizeof (WCHAR);
-
-  return RtlCompareUnicodeString (&us, &ut, TRUE);
-}
-
-extern "C" int __stdcall
-cygwin_strncasecmp (const char *cs, const char *ct, size_t n)
-{
-  UNICODE_STRING us, ut;
-  ULONG ulen;
-  size_t ls = 0, lt = 0;
-
-  while (cs[ls] && ls < n)
-    ++ls;
-  ulen = (ls + 1) * sizeof (WCHAR);
-  RtlInitEmptyUnicodeString (&us, (PWCHAR) alloca (ulen), ulen);
-  us.Length = sys_mbstowcs (us.Buffer, ls + 1, cs, ls) * sizeof (WCHAR);
-
-  while (ct[lt] && lt < n)
-    ++lt;
-  ulen = (lt + 1) * sizeof (WCHAR);
-  RtlInitEmptyUnicodeString (&ut, (PWCHAR) alloca (ulen), ulen);
-  ut.Length = sys_mbstowcs (ut.Buffer, lt + 1, ct, lt)  * sizeof (WCHAR);
-
-  return RtlCompareUnicodeString (&us, &ut, TRUE);
-}
-
-extern "C" char *
-strlwr (char *string)
-{
-  UNICODE_STRING us;
-  size_t len = (strlen (string) + 1) * sizeof (WCHAR);
-
-  us.MaximumLength = len; us.Buffer = (PWCHAR) alloca (len);
-  us.Length = sys_mbstowcs (us.Buffer, len, string) * sizeof (WCHAR)
-	      - sizeof (WCHAR);
-  RtlDowncaseUnicodeString (&us, &us, FALSE);
-  sys_wcstombs (string, len / sizeof (WCHAR), us.Buffer);
-  return string;
-}
-
-extern "C" char *
-strupr (char *string)
-{
-  UNICODE_STRING us;
-  size_t len = (strlen (string) + 1) * sizeof (WCHAR);
-
-  us.MaximumLength = len; us.Buffer = (PWCHAR) alloca (len);
-  us.Length = sys_mbstowcs (us.Buffer, len, string) * sizeof (WCHAR)
-	      - sizeof (WCHAR);
-  RtlUpcaseUnicodeString (&us, &us, FALSE);
-  sys_wcstombs (string, len / sizeof (WCHAR), us.Buffer);
-  return string;
-}
-
-/* backslashify: Convert all forward slashes in src path to back slashes
-   in dst path.  Add a trailing slash to dst when trailing_slash_p arg
-   is set to 1. */
-
-void
-backslashify (const char *src, char *dst, bool trailing_slash_p)
-{
-  const char *start = src;
-
-  while (*src)
-    {
-      if (*src == '/')
-	*dst++ = '\\';
-      else
-	*dst++ = *src;
-      ++src;
-    }
-  if (trailing_slash_p
-      && src > start
-      && !isdirsep (src[-1]))
-    *dst++ = '\\';
-  *dst++ = 0;
-}
-
-/* slashify: Convert all back slashes in src path to forward slashes
-   in dst path.  Add a trailing slash to dst when trailing_slash_p arg
-   is set to 1. */
-
-void
-slashify (const char *src, char *dst, bool trailing_slash_p)
-{
-  const char *start = src;
-
-  while (*src)
-    {
-      if (*src == '\\')
-	*dst++ = '/';
-      else
-	*dst++ = *src;
-      ++src;
-    }
-  if (trailing_slash_p
-      && src > start
-      && !isdirsep (src[-1]))
-    *dst++ = '/';
-  *dst++ = 0;
-}
-
 static WCHAR hex_wchars[] = L"0123456789abcdef";
 
 NTSTATUS NTAPI
