@@ -14,7 +14,6 @@
 #include <string.h>
 #include <sys/time.h>
 #include <ctype.h>
-#include <fcntl.h>
 #include <io.h>
 #include <windows.h>
 #include <wininet.h>
@@ -62,6 +61,8 @@ void package_find (int, char **);
 void package_list (int, char **);
 /* In bloda.cc  */
 void dump_dodgy_apps (int verbose);
+/* Forward declaration */
+static void usage (FILE *, int);
 
 static const char *known_env_vars[] = {
   "c_include_path",
@@ -97,43 +98,28 @@ static common_apps[] = {
   {"awk", 0},
   {"bash", 0},
   {"cat", 0},
-  {"certutil", 0},
-  {"clinfo", 0},
-  {"comp", 0},
-  {"convert", 0},
   {"cp", 0},
   {"cpp", 1},
   {"crontab", 0},
-  {"curl", 0},
-  {"expand", 0},
   {"find", 0},
-  {"ftp", 0},
   {"gcc", 0},
   {"gdb", 0},
   {"grep", 0},
-  {"hostname", 0},
   {"kill", 0},
-  {"klist", 0},
   {"ld", 0},
   {"ls", 0},
   {"make", 0},
   {"mv", 0},
-  {"nslookup", 0},
   {"patch", 0},
   {"perl", 0},
-  {"replace", 0},
   {"rm", 0},
   {"sed", 0},
-  {"sh", 0},
-  {"shutdown", 0},
-  {"sort", 0},
   {"ssh", 0},
+  {"sh", 0},
   {"tar", 0},
   {"test", 0},
-  {"timeout", 0},
   {"vi", 0},
   {"vim", 0},
-  {"whoami", 0},
   {0, 0}
 };
 
@@ -325,7 +311,10 @@ pathlike::check_existence (const char *fn, int showall, int verbose,
 			   char* first, const char *ext1, const char *ext2)
 {
   char file[4000];
-  snprintf (file, sizeof file, "%s%s%s%s", dir, fn, ext1, ext2);
+  strcpy (file, dir);
+  strcat (file, fn);
+  strcat (file, ext1);
+  strcat (file, ext2);
 
   wide_path wpath (file);
   if (GetFileAttributesW (wpath) != (DWORD) - 1)
@@ -361,8 +350,7 @@ find_on_path (const char *in_file, const char *ext, bool showall = false,
 {
   static char rv[4000];
 
-  /* Sort of a kludge but we've already tested this once, so don't try it
-     again */
+  /* Sort of a kludge but we've already tested this once, so don't try it again */
   if (in_file == rv)
     return in_file;
 
@@ -375,15 +363,13 @@ find_on_path (const char *in_file, const char *ext, bool showall = false,
   *rv = '\0';
   if (!in_file)
     {
-      display_error ("internal error find_on_path: NULL pointer for file",
-		     false, false);
+      display_error ("internal error find_on_path: NULL pointer for file", false, false);
       return 0;
     }
 
   if (!ext)
     {
-      display_error ("internal error find_on_path: "
-		     "NULL pointer for default_extension", false, false);
+      display_error ("internal error find_on_path: NULL pointer for default_extension", false, false);
       return 0;
     }
 
@@ -403,8 +389,7 @@ find_on_path (const char *in_file, const char *ext, bool showall = false,
 
   if (!file)
     {
-      display_error ("internal error find_on_path: "
-		     "cygpath conversion failed for %s\n", in_file);
+      display_error ("internal error find_on_path: cygpath conversion failed for %s\n", in_file);
       return 0;
     }
 
@@ -418,8 +403,7 @@ find_on_path (const char *in_file, const char *ext, bool showall = false,
 	pth->check_existence (file, showall, verbose, rv, ext);
 
 	if (checklinks)
-	  pth->check_existence (file, showall, verbose, rv, ext,
-				LINK_EXTENSION);
+	  pth->check_existence (file, showall, verbose, rv, ext, LINK_EXTENSION);
 
 	if (!*ext)
 	  continue;
@@ -811,6 +795,12 @@ track_down (const char *file, const char *suffix, int lvl)
   if (lvl)
     printf ("%*c", lvl, ' ');
 
+  if (!path)
+    {
+      display_error ("file not found - '%s'\n", file);
+      return false;
+    }
+
   printf ("%s", path);
 
   wide_path wpath (path);
@@ -1199,17 +1189,15 @@ dump_sysinfo_services ()
   int ret = fscanf (f, "cygrunsrv V%u.%u", &maj, &min);
   if (ferror (f) || feof (f) || ret == EOF || maj < 1 || min < 10)
     {
-      puts ("The version of cygrunsrv installed is too old to dump "
-	    "service info.\n");
+      puts ("The version of cygrunsrv installed is too old to dump service info.\n");
       return;
     }
-  pclose (f);
+  fclose (f);
 
   /* For verbose mode, just run cygrunsrv --list --verbose and copy output
      verbatim; otherwise run cygrunsrv --list and then cygrunsrv --query for
      each service.  */
-  snprintf (buf, sizeof (buf),
-	    (verbose ? "\"%s\" --list --verbose" : "\"%s\" --list"),
+  snprintf (buf, sizeof (buf), (verbose ? "\"%s\" --list --verbose" : "\"%s\" --list"),
 	    cygrunsrv);
   if ((f = popen (buf, "rt")) == NULL)
     {
@@ -1243,8 +1231,7 @@ dump_sysinfo_services ()
 	    snprintf (buf2, sizeof (buf2), "\"%s\" --query %s", cygrunsrv, srv);
 	    if ((f = popen (buf2, "rt")) == NULL)
 	      {
-		printf ("Failed to execute '%s', skipping services check.\n",
-			buf2);
+		printf ("Failed to execute '%s', skipping services check.\n", buf2);
 		return;
 	      }
 
@@ -1808,14 +1795,20 @@ dump_sysinfo ()
 	printf ("%7dMb %3d%% ", (int) capacity_mb, (int) percent_full);
       else
 	printf ("    N/A    N/A ");
-      printf ("%s %s %s %s %s %s %s  %s\n",
+      printf ("%s %s %s %s %s %s  %s\n",
 	      flags & FS_CASE_IS_PRESERVED ? "CP" : "  ",
 	      flags & FS_CASE_SENSITIVE ? "CS" : "  ",
 	      flags & FS_UNICODE_STORED_ON_DISK ? "UN" : "  ",
 	      flags & FS_PERSISTENT_ACLS ? "PA" : "  ",
 	      flags & FS_FILE_COMPRESSION ? "FC" : "  ",
 	      flags & FS_VOL_IS_COMPRESSED ? "VC" : "  ",
+#if 0
+	      flags & FILE_SUPPORTS_ENCRYPTION ? "EN" : "  ",
+	      flags & FILE_SUPPORTS_OBJECT_IDS ? "OI" : "  ",
+	      flags & FILE_SUPPORTS_REPARSE_POINTS ? "RP" : "  ",
+	      flags & FILE_SUPPORTS_SPARSE_FILES ? "SP" : "  ",
 	      flags & FILE_VOLUME_QUOTAS ? "QU" : "  ",
+#endif
 	      name);
     }
 
@@ -2019,20 +2012,13 @@ check_keys ()
   return 0;
 }
 
-/* These do not need to be escaped in application/x-www-form-urlencoded */
-static const char safe_chars[] = "$-_.!*'(),";
+/* RFC1738 says that these do not need to be escaped.  */
+static const char safe_chars[] = "$-_.+!*'(),";
 
 /* the URL to query.  */
 static const char base_url[] =
 	"http://cygwin.com/cgi-bin2/package-grep.cgi?text=1&grep=";
 
-#ifdef __x86_64__
-#define ARCH_STR  "&arch=x86_64"
-#else
-#define ARCH_STR  "&arch=x86"
-#endif
-static const char *ARCH_str = ARCH_STR;
-
 /* Queries Cygwin web site for packages containing files matching a regexp.
    Return value is 1 if there was a problem, otherwise 0.  */
 static int
@@ -2041,8 +2027,7 @@ package_grep (char *search)
   char buf[1024];
 
   /* construct the actual URL by escaping  */
-  char *url = (char *) alloca (sizeof (base_url) + strlen (ARCH_str)
-			       + strlen (search) * 3);
+  char *url = (char *) alloca (sizeof (base_url) + strlen ("&arch=x86_64") + strlen (search) * 3);
   strcpy (url, base_url);
 
   char *dest;
@@ -2060,7 +2045,11 @@ package_grep (char *search)
 	  dest += 2;
 	}
     }
-  strcpy (dest, ARCH_str);
+#ifdef __x86_64__
+  strcpy (dest, "&arch=x86_64");
+#else
+  strcpy (dest, "&arch=x86");
+#endif
 
   /* Connect to the net and open the URL.  */
   if (InternetAttemptConnect (0) != ERROR_SUCCESS)
@@ -2071,8 +2060,7 @@ package_grep (char *search)
 
   /* Initialize WinInet and attempt to fetch our URL.  */
   HINTERNET hi = NULL, hurl = NULL;
-  if (!(hi = InternetOpenA ("cygcheck", INTERNET_OPEN_TYPE_PRECONFIG,
-			    NULL, NULL, 0)))
+  if (!(hi = InternetOpenA ("cygcheck", INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0)))
     return display_internet_error ("InternetOpen() failed", NULL);
 
   if (!(hurl = InternetOpenUrlA (hi, url, NULL, 0, 0, 0)))
@@ -2108,7 +2096,7 @@ package_grep (char *search)
   return 0;
 }
 
-static void __attribute__ ((__noreturn__))
+static void
 usage (FILE * stream, int status)
 {
   fprintf (stream, "\
@@ -2177,10 +2165,8 @@ print_version ()
   printf ("cygcheck (cygwin) %d.%d.%d\n"
 	  "System Checker for Cygwin\n"
 	  "Copyright (C) 1998 - %s Cygwin Authors\n"
-	  "This is free software; see the source for copying conditions.  "
-	  "There is NO\n"
-	  "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR "
-	  "PURPOSE.\n",
+	  "This is free software; see the source for copying conditions.  There is NO\n"
+	  "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
 	  CYGWIN_VERSION_DLL_MAJOR / 1000,
 	  CYGWIN_VERSION_DLL_MAJOR % 1000,
 	  CYGWIN_VERSION_DLL_MINOR,
@@ -2261,9 +2247,6 @@ main (int argc, char **argv)
   bool ok = true;
   load_cygwin (argc, argv);
 
-  _setmode (1, _O_BINARY);
-  _setmode (2, _O_BINARY);
-
   /* Need POSIX sorting while parsing args, but don't forget the
      user's original environment.  */
   char *posixly = getenv ("POSIXLY_CORRECT");
