@@ -26,18 +26,17 @@ static struct option longopts[] =
   {"list", optional_argument, NULL, 'l'},
   {"force", no_argument, NULL, 'f'},
   {"signal", required_argument, NULL, 's'},
-  {"winpid", no_argument, NULL, 'W'},
   {"version", no_argument, NULL, 'V'},
   {NULL, 0, NULL, 0}
 };
 
-static char opts[] = "hl::fs:WV";
+static char opts[] = "hl::fs:V";
 
-static void __attribute__ ((__noreturn__))
+static void
 usage (FILE *where = stderr)
 {
   fprintf (where , ""
-	"Usage: %1$s [-fW] [-signal] [-s signal] pid1 [pid2 ...]\n"
+	"Usage: %1$s [-f] [-signal] [-s signal] pid1 [pid2 ...]\n"
 	"       %1$s -l [signal]\n"
 	"\n"
 	"Send signals to processes\n"
@@ -45,8 +44,6 @@ usage (FILE *where = stderr)
 	" -f, --force     force, using win32 interface if necessary\n"
 	" -l, --list      print a list of signal names\n"
 	" -s, --signal    send signal (use %1$s --list for a list)\n"
-	" -W, --winpid    specified pids are windows PIDs, not Cygwin PIDs\n"
-	"                 (use with extreme caution!)\n"
 	" -h, --help      output usage information and exit\n"
 	" -V, --version   output version information and exit\n"
 	"\n", prog_name);
@@ -93,7 +90,7 @@ getsig (const char *in_sig)
     }
   intsig = strtosigno (sig) ?: atoi (in_sig);
   char *p;
-  if (!intsig && (strcmp (sig, "SIG0") != 0 && (strtol (in_sig, &p, 10) != 0 || *p)))
+  if (!intsig && (strcmp (buf, "SIG0") != 0 && (strtol (in_sig, &p, 10) != 0 || *p)))
     intsig = -1;
   return intsig;
 }
@@ -155,41 +152,29 @@ get_debug_priv (void)
 }
 
 static void __stdcall
-forcekill (pid_t pid, DWORD winpid, int sig, int wait)
+forcekill (int pid, int sig, int wait)
 {
-  DWORD dwpid;
-
-  /* try to acquire SeDebugPrivilege */
+  // try to acquire SeDebugPrivilege
   get_debug_priv();
 
-  if (!winpid)
-    {
-      external_pinfo *p = (external_pinfo *)
-			  cygwin_internal (CW_GETPINFO_FULL, pid);
-      if (!p)
-	{
-	  fprintf (stderr, "%s: %d: No such process\n", prog_name, pid);
-	  return;
-	}
-      dwpid = p->dwProcessId;
-    }
-  else
-    /* pid is used for printing only after this point */
-    pid = dwpid = winpid;
-
-  HANDLE h = OpenProcess (PROCESS_TERMINATE, FALSE, dwpid);
+  external_pinfo *p = NULL;
+  /* cygwin_internal misinterprets negative pids (Win9x pids) */
+  if (pid > 0)
+    p = (external_pinfo *) cygwin_internal (CW_GETPINFO_FULL, pid);
+  DWORD dwpid = p ? p->dwProcessId : (DWORD) pid;
+  HANDLE h = OpenProcess (PROCESS_TERMINATE, FALSE, (DWORD) dwpid);
   if (!h)
     {
       if (!wait || GetLastError () != ERROR_INVALID_PARAMETER)
 	fprintf (stderr, "%s: couldn't open pid %u\n",
-		 prog_name, (unsigned int) dwpid);
+		 prog_name, (unsigned) dwpid);
       return;
     }
   if (!wait || WaitForSingleObject (h, 200) != WAIT_OBJECT_0)
     if (sig && !TerminateProcess (h, sig << 8)
 	&& WaitForSingleObject (h, 200) != WAIT_OBJECT_0)
       fprintf (stderr, "%s: couldn't kill pid %u, %u\n",
-	       prog_name, (unsigned int) dwpid, (unsigned int) GetLastError ());
+	       prog_name, (unsigned) dwpid, (unsigned int) GetLastError ());
   CloseHandle (h);
 }
 
@@ -198,7 +183,6 @@ main (int argc, char **argv)
 {
   int sig = SIGTERM;
   int force = 0;
-  int winpids = 0;
   int ret = 0;
   char *gotasig = NULL;
 
@@ -210,6 +194,7 @@ main (int argc, char **argv)
   opterr = 0;
 
   char *p;
+  long long int pid = 0;
 
   for (;;)
     {
@@ -240,9 +225,6 @@ main (int argc, char **argv)
 	case 'f':
 	  force = 1;
 	  break;
-	case 'W':
-	  winpids = 1;
-	  break;
 	case 'h':
 	  usage (stdout);
 	  break;
@@ -250,7 +232,7 @@ main (int argc, char **argv)
 	  print_version ();
 	  break;
 	case '?':
-	  if (gotasig) /* this is a negative pid, go ahead */
+	  if (gotasig)
 	    {
 	      --optind;
 	      goto out;
@@ -270,42 +252,33 @@ out:
   test_for_unknown_sig (sig, gotasig);
 
   argv += optind;
-  if (*argv == 0)
+  while (*argv != NULL)
     {
-      fprintf (stderr, "%s: not enough arguments\n", prog_name);
-      return 1;
-    }
-  for (long long int pid = 0; *argv != NULL; argv++)
-    {
-      DWORD dwpid = 0;
-
-      pid = strtoll (*argv, &p, 10);
-      /* INT_MIN <= pid <= INT_MAX.  -f only takes positive pids. */
-      if (*p != '\0' || pid < (force ? 1 : INT_MIN) || pid > INT_MAX)
+      if (!pid)
+	pid = strtoll (*argv, &p, 10);
+      if (*p != '\0'
+	  || (!force && (pid < INT_MIN || pid > INT_MAX))
+	  || (force && (pid <= 0 || pid > UINT_MAX)))
 	{
 	  fprintf (stderr, "%s: illegal pid: %s\n", prog_name, *argv);
 	  ret = 1;
-	  continue;
 	}
-      if (winpids)
-	{
-	  dwpid = pid;
-	  pid = (pid_t) cygwin_internal (CW_WINPID_TO_CYGWIN_PID, dwpid);
-	}
-      if (kill ((pid_t) pid, sig) == 0)
+      else if (pid <= INT_MAX && kill ((pid_t) pid, sig) == 0)
 	{
 	  if (force)
-	    forcekill ((pid_t) pid, dwpid, sig, 1);
+	    forcekill ((pid_t) pid, sig, 1);
 	}
       else if (force)
-	forcekill ((pid_t) pid, dwpid, sig, 0);
+	forcekill ((pid_t) pid, sig, 0);
       else
 	{
 	  char buf[1000];
-	  sprintf (buf, "%s: %lld", prog_name, dwpid ?: pid);
+	  sprintf (buf, "%s: %lld", prog_name, pid);
 	  perror (buf);
 	  ret = 1;
 	}
+      argv++;
+      pid = 0;
     }
   return ret;
 }
