@@ -1,8 +1,5 @@
 /* setfacl.c
 
-   Copyright 2000, 2001, 2002, 2003, 2006, 2008, 2009, 2010, 2011, 2014,
-   2015 Red Hat Inc.
-
    Written by Corinna Vinschen <vinschen@redhat.com>
 
 This file is part of Cygwin.
@@ -20,8 +17,7 @@ details. */
 #include <getopt.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/types.h>
-#include <sys/acl.h>
+#include <cygwin/acl.h>
 #include <cygwin/version.h>
 
 #ifndef BOOL
@@ -675,7 +671,7 @@ print_version ()
 {
   printf ("setfacl (cygwin) %d.%d.%d\n"
 	  "POSIX ACL modification utility\n"
-	  "Copyright (C) 2000 - %s Red Hat, Inc.\n"
+	  "Copyright (C) 2000 - %s Cygwin Authors\n"
 	  "This is free software; see the source for copying conditions.  There is NO\n"
 	  "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
 	  CYGWIN_VERSION_DLL_MAJOR / 1000,
